package nova;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** A block is a component dedicated to display module values of Nova
    It has a list of setting (Nibble with offset, min/max values and table)
    loadValues( sysex )
    saveValues( sysex )
*/
public class Block extends JPanel {
    Vector<Nibble> nibbles;
    Patch parent;
    SystemDump system;
    boolean switchable = true;

    Checkbox switchCb;
    boolean switchon = false;
    Box box;

    String name;
    int rowspan = 1;

    int[] activeNibbles;
    
    final static LineBorder border = new LineBorder( Constants.grisvert, 1, false);

    public Block( String title, Patch parent, boolean switchable ) {	
	super();	
	this.parent = parent;
	this.switchable = switchable;
	this.name=title;
	nibbles = new Vector<Nibble>();
	box = Box.createVerticalBox();
	box.setBorder( border );
	setLayout(new GridBagLayout());
	GridBagConstraints c  = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1;
	c.weighty = 1;
        add(box,c);
    }


    public Block( String title, SystemDump parent) {	
	super();	
	this.system = parent;
	this.name=title;
	nibbles = new Vector<Nibble>();
	setLayout(new GridBagLayout());
	GridBagConstraints c  = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;
	c.weightx = 1;
	c.weighty = 1;
    }

    public String getName() {
	return name;
    }

    public String toXML() {
	String s = "";
	s+="<block name=\""+name+"\">\n";
	if (activeNibbles != null) {
	    for (int i=0; i<activeNibbles.length;i++) {
		Nibble nib;
		if (parent != null) // Patch
		    nib = parent.getNibble( activeNibbles[i] );
		else //SystemDump
		    nib = system.getNibble( activeNibbles[i] );
		s+=nib.toXML();
	    }
	}
	s+="</block>\n";
	return s;
    }

    public String toHTML() {
	String s = "";
	s+="        <TD align=\"left\" valign=\"top\" WIDTH=\"20%\" rowspan=\""+rowspan+"\">\n";
	s+="          <B>"+name+"</B>\n";
	if (activeNibbles != null) {
	    for (int i=0; i<activeNibbles.length;i++) {
		Nibble nib;
		if (parent != null) // Patch
		    nib = parent.getNibble( activeNibbles[i] );
		else //SystemDump
		    nib = system.getNibble( activeNibbles[i] );

		s+=nib.toHTML();
	    }
	}
	s+="        </TD>\n";
	return s;
    }

    Vector<Nibble> getNibbles(){
	return nibbles;
    }

    int getLength(){
	return nibbles.size();
    }

    
}

