package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Comp Block
 * Extension of Block class to manage Comp parameters
 */
public class Comp extends CopyBlock {

    Nibble type;
    Nibble threshold;
    Nibble ratio;
    Nibble attack;
    Nibble release;
    Nibble response;
    Nibble drive;
    Nibble level;
    Nibble onoff;

    public Comp(Patch par) {	
	super("Compressor", par, true);

	activeNibbles = new int[] { 24, 9, 10, 11, 12, 13, 14, 15, 16 } ;

	try {
	    for (int i=0; i<activeNibbles.length; i++) 
	    nibbles.add( parent.getNibble( activeNibbles[i]));

	    onoff = parent.getNibble(24);
	    type = parent.getNibble(9);

	    threshold = parent.getNibble(10);
	    ratio = parent.getNibble(11);
	    attack = parent.getNibble(12);
	    release = parent.getNibble(13);
	    response = parent.getNibble(14);
	    drive = parent.getNibble(15);
	    level = parent.getNibble(16);

	    type.updateNibble("Type", Nibble.TYPE_COMP);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Comp Block");
	    e.printStackTrace();
	}

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();
	//remettre les activeNibbles en TYPE_INT
	for (int j=2; j<activeNibbles.length; j++) {
	    Nibble n = parent.getNibble( activeNibbles[j]);
	    n.setType(Nibble.TYPE_INT);
	}

	int i = 0;

	box.add( new Row("Compressor", onoff, i++));
	box.add( new Row("Comp", type, i++));

	if (type.getValue()<2) {
	    activeNibbles = new int[]{ 24, 9, 15, 14, 16};
	    drive.updateNibble("Drive", Nibble.TYPE_1_20);
	    response.updateNibble("Response", Nibble.TYPE_1_10 );
	    level.updateNibble("Level (dB)", Nibble.TYPE_N99_12);
	    box.add( new Row("Comp", drive, i++));
	    box.add( new Row("Comp", response, i++));
	    box.add( new Row("Comp", level, i++));
	} else {
	    activeNibbles = new int[]{ 24, 9, 10, 11, 12, 13, 16 };
	    threshold.updateNibble("Threshold (dB)", Nibble.TYPE_N40_0);
	    ratio.updateNibble("Ratio", Nibble.TYPE_RATIO);
	    release.updateNibble("Release (ms)", Nibble.TYPE_RELEASE);
	    attack.updateNibble("Attack (ms)", Nibble.TYPE_ATTACK);
	    level.updateNibble("Level (dB)",Nibble.TYPE_N99_12);
	    box.add( new Row("Comp", threshold, i++));
	    box.add( new Row("Comp", ratio, i++));
	    box.add( new Row("Comp", attack, i++));
	    box.add( new Row("Comp", release, i++));

	    box.add( new Row("Comp", level, i++));
	}
	while (i<9) box.add( new EmptyRow(true, i++));
    }

    int getBlockType(Box b) {
	return COMP;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

