package nova;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/** A CopyBlock is subtype of Block, with a Copy/Paste feature
    Only applies to Patch Blocks (Comp, Drive, etc.)
*/
public abstract class CopyBlock extends Block implements MouseListener {    
    final static int NONE = 0;
    final static int DRIVE = 1;
    final static int COMP = 2;
    final static int MOD = 3;
    final static int PITCH = 4;
    final static int DELAY = 5;
    final static int REVERB = 6;
    final static int EQ = 7;
    final static int GATE = 8;
    final static int BOOST = 9;

    final static int DRIVEBOOST = 20;
    final static int EQGATE = 21;
    final static int GLOBAL = 30;

    static int blockType = NONE;
    static int bufferType = NONE;
    
    static int[] buffer;
    static int[] bufferActiveNibbles;
    static boolean isBufferFull = false;

    NovaManager nm;

    public CopyBlock( String title, Patch parent, boolean switchable ) {	
	super(title, parent, switchable);
	nm = parent.parent;

	box.addMouseListener(this);
    }

    /* copy the values of a given block into the buffer */
    static void copy(CopyBlock block, Box b) {
	bufferType = block.getBlockType(b);
	int[] blockActiveNibbles = block.getActiveNibbles(b);

	buffer = new int[Patch.PATCH_SIZE];
	bufferActiveNibbles = (int[]) blockActiveNibbles.clone();

	for (int i=0; i<bufferActiveNibbles.length; i++) {
	    int pos = bufferActiveNibbles[i];
	    Nibble nib = block.parent.getNibble( pos );
	    buffer[pos] = nib.getValue();
	}

	isBufferFull = true;
    }

    /* paste buffer values into a given block */
    static void paste(CopyBlock block, Box b) {
	block.setActiveNibbles(b, (int[]) bufferActiveNibbles.clone());

	for (int i=0; i< bufferActiveNibbles.length; i++) {
	    int pos = bufferActiveNibbles[i];
	    Nibble nib = block.parent.getNibble( pos );
	    try {
		nib.setValue( buffer[pos] );
	    } catch (NibbleException e) {
		block.parent.log(e.getMessage());
	    }
	}	
	block.refresh();
	block.validate();
    }

    abstract void refresh();
    abstract int getBlockType(Box b);
    abstract int[] getActiveNibbles(Box b);
    abstract void setActiveNibbles(Box b, int[] an);

    public void mouseClicked(MouseEvent e) {

	Box b = (Box) e.getSource();
	Point pos = b.getLocationOnScreen();
	int x = e.getX() + pos.x;
	int y = e.getY() + pos.y;


	CopyPastePopup np = new CopyPastePopup( nm, this, b, x, y);
	np.pack();
	nm.setPopup(np);
    }

    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
}

