package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Delay Block
 * Extension of Block class to manage Delay parameters
 */
public class Delay extends CopyBlock {

    Nibble type;
    Nibble d1;
    Nibble d2;
    Nibble tempo1;
    Nibble tempo2;
    Nibble fb1;
    Nibble fb2;
    Nibble hicut;
    Nibble locut;
    Nibble pan1;
    Nibble pan2;
    Nibble damp;
    Nibble release;
    Nibble mix;
    Nibble onoff;

    Nibble width;
    Nibble clip;
    Nibble offset;
    Nibble sense;

    public Delay(Patch par) {	
	super("Delay", par, true);

	activeNibbles = new int[] { 72, 57, 58, 59,
				    60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70} ;

	try {
	    for (int i=0; i<activeNibbles.length; i++) 
		nibbles.add( parent.getNibble( activeNibbles[i]));
	    
	    onoff = parent.getNibble(72);
	    type = parent.getNibble(57);
	    d1 = parent.getNibble(58);
	    d2 = parent.getNibble(59);
	    tempo1 = parent.getNibble(60);
	    tempo2 = parent.getNibble(61);
	    fb1 = parent.getNibble(62);
	    fb2 = parent.getNibble(63);
	    hicut = parent.getNibble(64);
	    locut = parent.getNibble(65);
	    pan1 = parent.getNibble(66);
	    pan2 = parent.getNibble(67);
	    damp = parent.getNibble(68);
	    release = parent.getNibble(69);
	    mix = parent.getNibble(70);

	    type.updateNibble("Type", Nibble.TYPE_DELAY);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Delay Block");
	}

	width  = tempo2;
	clip = fb2; 
	offset = pan1;
	sense = pan2;
	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();
	for (int j=2; j<activeNibbles.length; j++) {
	    Nibble n = parent.getNibble( activeNibbles[j]);
	    n.setType(Nibble.TYPE_INT);
	}

	int i=0;
	Row r ;
	box.add( new Row("Delay", onoff, i++));
	box.add( new Row("", type, i++));

	switch (type.getValue()) {
	case 0: //clean
	    activeNibbles = new int[] { 72, 57,
					58, 60, 62,
					64, 65, 70
	    };

	    d1.updateNibble("Delay (ms)", Nibble.TYPE_0_1800);
	    tempo1.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    fb1.updateNibble("Feedback (%)",Nibble.TYPE_0_100);

	    hicut.updateNibble("HiCut (Hz)", Nibble.TYPE_HICUT);
	    locut.updateNibble("LoCut (Hz)",Nibble.TYPE_LOCUT);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("", d1, i++));
	    box.add( new Row("", tempo1, i++));
	    box.add( new Row("", fb1, i++));

	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", locut, i++));
	    box.add( new Row("", mix, i++));
	    break;
	case 1: //analog
	    activeNibbles = new int[] { 72, 57,
					58, 60, 63,
					62, 64, 65, 
					70
	    };

	    d1.updateNibble("Delay (ms)", Nibble.TYPE_0_1800);
	    tempo1.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    clip.updateNibble("Clip (dB)",Nibble.TYPE_0_18);

	    fb1.updateNibble("Feedback (%)",Nibble.TYPE_0_100);
	    hicut.updateNibble("HiCut (Hz)", Nibble.TYPE_HICUT);
	    locut.updateNibble("LoCut (Hz)",Nibble.TYPE_LOCUT);

	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("", d1, i++));
	    box.add( new Row("", tempo1, i++));
	    box.add( new Row("", clip, i++));

	    box.add( new Row("", fb1, i++));
	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", locut, i++));

	    box.add( new Row("", mix, i++));
	    break;
	case 2: //tape
	    activeNibbles = new int[] { 72, 57,
					58, 60, 63,
					62, 64, 65, 
					70
	    };

	    d1.updateNibble("Delay (ms)", Nibble.TYPE_0_1800);
	    tempo1.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    clip.updateNibble("Clip (dB)",Nibble.TYPE_0_18);

	    fb1.updateNibble("Feedback (%)",Nibble.TYPE_0_100);
	    hicut.updateNibble("HiCut (Hz)", Nibble.TYPE_HICUT);
	    locut.updateNibble("LoCut (Hz)",Nibble.TYPE_LOCUT);

	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("", d1, i++));
	    box.add( new Row("", tempo1, i++));
	    box.add( new Row("", clip, i++));

	    box.add( new Row("", fb1, i++));
	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", locut, i++));

	    box.add( new Row("", mix, i++));
	    break;

	case 3: //dynamic
	    activeNibbles = new int[] { 72, 57,
					58, 60, 62,
					64, 65, 66,
					67, 68, 69,
					70
	    };


	    d1.updateNibble("Delay (ms)", Nibble.TYPE_0_1800);
	    tempo1.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    fb1.updateNibble("Feedback (%)",Nibble.TYPE_0_100);
	    hicut.updateNibble("HiCut (Hz)", Nibble.TYPE_HICUT);
	    locut.updateNibble("LoCut (Hz)",Nibble.TYPE_LOCUT);

	    offset.updateNibble("Offset (ms)",Nibble.TYPE_0_200);
	    sense.updateNibble("Sense (dB)",Nibble.TYPE_N50_0);
	    damp.updateNibble("Damp (dB)",Nibble.TYPE_0_100);
	    release.updateNibble("Release (ms)",Nibble.TYPE_RELEASE);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("", d1, i++));
	    box.add( new Row("", tempo1, i++));
	    box.add( new Row("", fb1, i++));
	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", locut, i++));

	    box.add( new Row("", offset, i++));
	    box.add( new Row("", sense, i++));
	    box.add( new Row("", damp, i++));
	    box.add( new Row("", release, i++));
	    box.add( new Row("", mix, i++));
	    break;
	case 4: //dual
	    activeNibbles = new int[] { 72, 57,
					58, 59, 60, 61,
					62, 63,	64, 65, 
					66, 67, 70
	    };

	    d1.updateNibble("Delay 1 (ms)",Nibble.TYPE_0_1800);
	    d2.updateNibble("Delay 2 (ms)",Nibble.TYPE_0_1800);
	    tempo1.updateNibble("Tempo 1",Nibble.TYPE_TEMPO);
	    tempo2.updateNibble("Tempo 2",Nibble.TYPE_TEMPO);

	    fb1.updateNibble("Feedback 1 (%)",Nibble.TYPE_0_120);
	    fb2.updateNibble("Feedback 2 (%)",Nibble.TYPE_0_120);
	    hicut.updateNibble("HiCut (Hz)", Nibble.TYPE_HICUT);
	    locut.updateNibble("LoCut (Hz)",Nibble.TYPE_LOCUT);

	    pan1.updateNibble("Pan 1",Nibble.TYPE_N50_50);
	    pan2.updateNibble("Pan 2",Nibble.TYPE_N50_50);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("", d1, i++));
	    box.add( new Row("", d2, i++));
	    box.add( new Row("", tempo1, i++));
	    box.add( new Row("", tempo2, i++));

	    box.add( new Row("", fb1, i++));
	    box.add( new Row("", fb2, i++));
	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", locut, i++));

	    box.add( new Row("", pan1, i++));
	    box.add( new Row("", pan2, i++));
	    box.add( new Row("", mix, i++));
	    break;

	case 5: //ping-pong
	    activeNibbles = new int[] { 72, 57,
					58, 60, 61,
					62, 64, 65, 
					70
	    };

	    d1.updateNibble("Delay (ms)", Nibble.TYPE_0_1800);
	    tempo1.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    width.updateNibble("Width (%)",Nibble.TYPE_0_100);

	    fb1.updateNibble("Feedback (%)",Nibble.TYPE_0_100);
	    hicut.updateNibble("HiCut (Hz)", Nibble.TYPE_HICUT);
	    locut.updateNibble("LoCut (Hz)",Nibble.TYPE_LOCUT);

	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("", d1, i++));
	    box.add( new Row("", tempo1, i++));
	    box.add( new Row("", width, i++));

	    box.add( new Row("", fb1, i++));
	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", locut, i++));

	    box.add( new Row("", mix, i++));
	    break;
	}

	while (i<14) {
	    box.add( new EmptyRow(true, i++));    
	}
		    
    }

    int getBlockType(Box b) {
	return DELAY;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

