package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Drive Block
 * Extension of Block class to manage Drive parameters
 */
public class Drive extends CopyBlock {
    String name="";
    int bankN;
    int presN;

    JLabel bankLbl;
    JLabel presLbl;
    JTextField bankTf;
    JTextField presTf;
    JTextField nameTf;

    Nibble type;
    Nibble gain;
    Nibble tone;
    Nibble level;
    Nibble onoff;

    public Drive(Patch par) {	
	super("Drive", par, true);

	try {
	    type = parent.getNibble(25);
	    type.updateNibble("Type", Nibble.TYPE_DRIVE);
	    nibbles.add( type );

	    gain = parent.getNibble(26);
	    gain.updateNibble("Gain (dB)", Nibble.TYPE_0_30);
	    nibbles.add( gain );

	    tone = parent.getNibble(27);
	    tone.updateNibble("Tone (%)", Nibble.TYPE_0_100);
	    nibbles.add( tone );

	    level = parent.getNibble(39);
	    level.updateNibble("Level (dB)",Nibble.TYPE_N99_15);
	    nibbles.add( level );

	    onoff = parent.getNibble(40);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	    nibbles.add( onoff );

	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Drive Block");
	}

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();
	int i = 0;

	activeNibbles = new int[]{ 40, 25, 26, 27, 39};

	box.add( new Row("Drive", onoff, i++));
	box.add( new Row("", type, i++));
	box.add( new Row("", gain, i++));
	box.add( new Row("", tone, i++));
	box.add( new Row("", level, i++));

	while (i<9) box.add( new EmptyRow(true, i++));

    }

    int getBlockType(Box b) {
	return DRIVE;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

