package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The DriveBoost Block
 * Extension of Block class to manage DriveBoost parameters
 * In Variations
 */
public class DriveBoost extends CopyBlock {
    String name="";
    int bankN;
    int presN;

    JLabel bankLbl;
    JLabel presLbl;
    JTextField bankTf;
    JTextField presTf;
    JTextField nameTf;

    Nibble type;
    Nibble gain;
    Nibble tone;
    Nibble level;
    Nibble onoff;

    Nibble boostlevel;
    Nibble boostonoff;

    Box driveBox;
    Box boostBox;
    Box hb;

    int[] activeNibblesDrive;
    int[] activeNibblesBoost;

    public DriveBoost(Patch par) {	
	super("Drive", par, true);
	// se rajoute lui-meme les nibbles qu'il veut
	// ie. les nibbles de 1 à 8

	activeNibbles = new int[] { 25, 26, 27, 39, 40, 37, 38 };
	activeNibblesDrive = new int[] { 25, 26, 27, 39, 40};
	activeNibblesBoost = new int[] { 37, 38 };

	try {
	    type = parent.getNibble(25);
	    type.updateNibble("Type", Nibble.TYPE_DRIVE);
	    nibbles.add( type );

	    gain = parent.getNibble(26);
	    gain.updateNibble("Gain (dB)", Nibble.TYPE_0_30);
	    nibbles.add( gain );

	    tone = parent.getNibble(27);
	    tone.updateNibble("Tone (%)", Nibble.TYPE_0_100);
	    nibbles.add( tone );

	    level = parent.getNibble(39);
	    level.updateNibble("Level (dB)",Nibble.TYPE_N99_15);
	    nibbles.add( level );

	    onoff = parent.getNibble(40);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	    nibbles.add( onoff );

	    boostlevel = parent.getNibble(37);
	    boostlevel.updateNibble("Level (dB)", Nibble.TYPE_0_10);
	    nibbles.add( boostlevel );

	    boostonoff = parent.getNibble(38);
	    boostonoff.updateNibble("On",Nibble.TYPE_ONOFF);
	    nibbles.add( boostonoff );

	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Drive Block");
	}

	box.removeMouseListener(this);
	hb = Box.createHorizontalBox();
	hb.setOpaque(true);
	hb.setBackground(Constants.grisvert);
	
	driveBox = Box.createVerticalBox();
	driveBox.addMouseListener(this);

	boostBox = Box.createVerticalBox();
	boostBox.addMouseListener(this);

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();

	int i = 0;
	driveBox.removeAll();
	driveBox.add( new Row("Drive", onoff, i++));
	driveBox.add( new Row("", type, i++));
	driveBox.add( new Row("", gain, i++));
	driveBox.add( new Row("", tone, i++));
	driveBox.add( new Row("", level, i++));
	while (i<5) driveBox.add( new EmptyRow(true, i++));

	i = 0;
	boostBox.removeAll();
	boostBox.add( new Row("Boost", boostonoff, i++));
	boostBox.add( new Row("Boost", boostlevel, i++));
	while (i<5) boostBox.add( new EmptyRow(true, i++));

	hb.add(driveBox);
	hb.add(Box.createHorizontalStrut(10));
	hb.add(boostBox);
	box.add(hb);
    }

    public String toHTML() {
	String s = "";
	s+="        <TD align=\"left\" valign=\"top\">\n";
	s+="          <B>Drive</B>\n";
	s+=onoff.toHTML();
	s+=type.toHTML();
	s+=gain.toHTML();
	s+=tone.toHTML();
	s+=level.toHTML();
	s+="        </TD>\n";

	s+="        <TD align=\"left\" valign=\"top\">\n";
	s+="          <B>Boost</B>\n";
	s+=boostonoff.toHTML();
	s+=boostlevel.toHTML();
	s+="        </TD>\n";
	return s;
    }

    public String toXML() {
	String s = "";
	s+="<block name=\"Drive\">\n";
	s+=onoff.toXML();
	s+=type.toXML();
	s+=gain.toXML();
	s+=tone.toXML();
	s+=level.toXML();
	s+="</block>\n";
	s+="<block name=\"Boost\">\n";
	s+=boostonoff.toXML();
	s+=boostlevel.toXML();
	s+="</block>\n";
	return s;
    }

    int getBlockType(Box b) {
	if (b.equals(driveBox))
	    return DRIVE;
	else return BOOST;
    }

    int[] getActiveNibbles(Box b) {
	if (b.equals(driveBox))
	    return activeNibblesDrive;
	else return activeNibblesBoost;
    }

    void setActiveNibbles(Box b, int[] an) {
	if (b.equals(driveBox))
	    activeNibblesDrive = an;
	else activeNibblesBoost = an;
    }

}

