package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The EQ Block
 * Extension of Block class to manage EQ parameters
 */
public class EQ extends CopyBlock {


    Nibble f1;
    Nibble g1;
    Nibble w1;
    Nibble f2;
    Nibble g2;
    Nibble w2;
    Nibble f3;
    Nibble g3;
    Nibble w3;

    Nibble onoff;

    public EQ(Patch par) {	
	super("EQ", par, true);

	try {
	    onoff = parent.getNibble(93);
	    onoff.updateNibble("On",Nibble.TYPE_OFFON);
	    nibbles.add( onoff );

	    f1 = parent.getNibble(94);
	    f1.updateNibble("Freq1 (Hz)", Nibble.TYPE_EQFREQ);
	    nibbles.add( f1 );

	    g1 = parent.getNibble(95);
	    g1.updateNibble("Gain1 (dB)", Nibble.TYPE_N12_12);
	    nibbles.add( g1 );

	    w1 = parent.getNibble(96);
	    w1.updateNibble("Width1 (oct)", Nibble.TYPE_EQWIDTH);
	    nibbles.add( w1 );

	    f2 = parent.getNibble(97);
	    f2.updateNibble("Freq2 (Hz)", Nibble.TYPE_EQFREQ);
	    nibbles.add( f2 );

	    g2 = parent.getNibble(98);
	    g2.updateNibble("Gain2 (dB)", Nibble.TYPE_N12_12);
	    nibbles.add( g2 );

	    w2 = parent.getNibble(99);
	    w2.updateNibble("Width2 (oct)", Nibble.TYPE_EQWIDTH);
	    nibbles.add( w2 );

	    f3 = parent.getNibble(100);
	    f3.updateNibble("Freq3 (Hz)", Nibble.TYPE_EQFREQ);
	    nibbles.add( f3 );

	    g3 = parent.getNibble(101);
	    g3.updateNibble("Gain3 (dB)", Nibble.TYPE_N12_12);
	    nibbles.add( g3 );

	    w3 = parent.getNibble(102);
	    w3.updateNibble("Width3 (oct)", Nibble.TYPE_EQWIDTH);
	    nibbles.add( w3 );
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Drive Block");
	}

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();
	int i = 0;
	activeNibbles = new int[]{ 93, 94, 95, 96, 97, 98, 99, 100, 101, 102};

	box.add( new Row("EQ", nibbles.elementAt(0), i++));
	box.add( new Row("F1", nibbles.elementAt(1), i++));
	box.add( new Row("G1", nibbles.elementAt(2), i++));
	box.add( new Row("W1", nibbles.elementAt(3), i++));
	box.add( new EmptyRow(false, i++));
	box.add( new Row("F2", nibbles.elementAt(4), i++));
	box.add( new Row("G2", nibbles.elementAt(5), i++));
	box.add( new Row("W2", nibbles.elementAt(6), i++));
	box.add( new EmptyRow(false, i++));
	box.add( new Row("F3", nibbles.elementAt(7), i++));
	box.add( new Row("G3", nibbles.elementAt(8), i++));
	box.add( new Row("W3", nibbles.elementAt(9), i++));
	
	while (i<14) 	
	    box.add( new EmptyRow(true, i++));    
    }

    int getBlockType(Box b) {
	return EQ;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

