package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The EQGate Block
 * Extension of Block class to manage EQ and Gate parameters
 * in Variations
 */
public class EQGate extends CopyBlock {


    Nibble f1;
    Nibble g1;
    Nibble w1;
    Nibble f2;
    Nibble g2;
    Nibble w2;
    Nibble f3;
    Nibble g3;
    Nibble w3;
    Nibble onoff;

    Nibble gateType;
    Nibble gateThreshold;
    Nibble gateDamp;
    Nibble gateRelease;
    Nibble gateOnoff;

    int[] activeNibblesEQ;
    int[] activeNibblesGate;

    Box eqBox;
    Box gateBox;
    Box hb;

    public EQGate(Patch par) {	
	super("EQ", par, true);

	activeNibbles = new int[]{ 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 104,
				   89, 90, 91, 92};

	activeNibblesEQ = new int[]{ 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 104};

	activeNibblesGate = new int[]{ 89, 90, 91, 92};

	try {
	    onoff = parent.getNibble(93);
	    onoff.updateNibble("On",Nibble.TYPE_OFFON);
	    nibbles.add( onoff );

	    f1 = parent.getNibble(94);
	    f1.updateNibble("Freq1 (Hz)", Nibble.TYPE_EQFREQ);
	    nibbles.add( f1 );

	    g1 = parent.getNibble(95);
	    g1.updateNibble("Gain1 (dB)", Nibble.TYPE_N12_12);
	    nibbles.add( g1 );

	    w1 = parent.getNibble(96);
	    w1.updateNibble("Width1 (oct)", Nibble.TYPE_EQWIDTH);
	    nibbles.add( w1 );

	    f2 = parent.getNibble(97);
	    f2.updateNibble("Freq2 (Hz)", Nibble.TYPE_EQFREQ);
	    nibbles.add( f2 );

	    g2 = parent.getNibble(98);
	    g2.updateNibble("Gain2 (dB)", Nibble.TYPE_N12_12);
	    nibbles.add( g2 );

	    w2 = parent.getNibble(99);
	    w2.updateNibble("Width2 (oct)", Nibble.TYPE_EQWIDTH);
	    nibbles.add( w2 );

	    f3 = parent.getNibble(100);
	    f3.updateNibble("Freq3 (Hz)", Nibble.TYPE_EQFREQ);
	    nibbles.add( f3 );

	    g3 = parent.getNibble(101);
	    g3.updateNibble("Gain3 (dB)", Nibble.TYPE_N12_12);
	    nibbles.add( g3 );

	    w3 = parent.getNibble(102);
	    w3.updateNibble("Width3 (oct)", Nibble.TYPE_EQWIDTH);
	    nibbles.add( w3 );

	    gateOnoff = parent.getNibble(104);
	    gateOnoff.updateNibble("On",Nibble.TYPE_OFFON);
	    nibbles.add( gateOnoff );

	    gateType = parent.getNibble(89);
	    gateType.updateNibble("Type", Nibble.TYPE_GATE);
	    nibbles.add( gateType );

	    gateThreshold = parent.getNibble(90);
	    gateThreshold.updateNibble("Threshold (dB)", Nibble.TYPE_N60_0);
	    nibbles.add( gateThreshold );

	    gateDamp = parent.getNibble(91);
	    gateDamp.updateNibble("Damp (dB)", Nibble.TYPE_0_90);
	    nibbles.add( gateDamp );

	    gateRelease = parent.getNibble(92);
	    gateRelease.updateNibble("Speed (/s)",Nibble.TYPE_3_200);
	    nibbles.add( gateRelease );

	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load EQ/Gate Block");
	}

	box.removeMouseListener(this);
	hb = Box.createHorizontalBox();
	hb.setOpaque(true);
	hb.setBackground(Constants.grisvert);

	eqBox = Box.createVerticalBox();
	eqBox.addMouseListener(this);

	gateBox = Box.createVerticalBox();
	gateBox.addMouseListener(this);

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();

	int i = 0;
	eqBox.removeAll();
	eqBox.add( new Row("EQ", nibbles.elementAt(0), i++));
	eqBox.add( new Row("F1", nibbles.elementAt(1), i++));
	eqBox.add( new Row("G1", nibbles.elementAt(2), i++));
	eqBox.add( new Row("W1", nibbles.elementAt(3), i++));
	eqBox.add( new EmptyRow(false, i++));
	eqBox.add( new Row("F2", nibbles.elementAt(4), i++));
	eqBox.add( new Row("G2", nibbles.elementAt(5), i++));
	eqBox.add( new Row("W2", nibbles.elementAt(6), i++));
	eqBox.add( new EmptyRow(false, i++));
	eqBox.add( new Row("F3", nibbles.elementAt(7), i++));
	eqBox.add( new Row("G3", nibbles.elementAt(8), i++));
	eqBox.add( new Row("W3", nibbles.elementAt(9), i++));	
	while (i<12) 	
	    eqBox.add( new EmptyRow(true, i++));   

	i = 0;
	gateBox.removeAll();
	gateBox.add( new Row("Noise Gate", gateOnoff, i++));
	gateBox.add( new Row("Noise Gate", gateType, i++));
	gateBox.add( new Row("Noise Gate", gateThreshold, i++));
	gateBox.add( new Row("Noise Gate", gateDamp, i++));
	gateBox.add( new Row("Noise Gate", gateRelease, i++));
	while (i<12) gateBox.add( new EmptyRow(true, i++));  

	hb.add(eqBox);
	hb.add(Box.createHorizontalStrut(10));
	hb.add(gateBox);
	box.add(hb);
 
    }

    public String toHTML() {
	String s = "";
	s+="        <TD align=\"left\" valign=\"top\">\n";
	s+="          <B>EQ</B>\n";
	for (int i=0; i<10;i++) {
	    Nibble nib = nibbles.elementAt(i);
	    s+=nib.toHTML();
	}
	s+="        </TD>\n";
	s+="        <TD align=\"left\" valign=\"top\">\n";
	s+="          <B>Noise Gate</B>\n";
	for (int i=10; i<15;i++) {
	    Nibble nib = nibbles.elementAt(i);
	    s+=nib.toHTML();
	}
	s+="        </TD>\n";
	return s;
    }

    public String toXML() {
	String s = "";
	s+="<block name=\"EQ\">\n";
	for (int i=0; i<10;i++) {
	    Nibble nib = nibbles.elementAt(i);
	    s+=nib.toXML();
	}
	s+="</block>\n";
	s+="<block name=\"Noise Gate\">\n";
	for (int i=10; i<15;i++) {
	    Nibble nib = nibbles.elementAt(i);
	    s+=nib.toXML();
	}
	s+="</block>\n";
	return s;
    }

    int getBlockType(Box b) {
	if (b.equals(eqBox)) return EQ;
	else return GATE;
    }

    int[] getActiveNibbles(Box b) {
	if (b.equals(eqBox)) return activeNibblesEQ;
	else return activeNibblesGate;   
    }

    void setActiveNibbles(Box b, int[] an) {
	if (b.equals(eqBox))
	    activeNibblesEQ = an;
	else activeNibblesGate = an;
    }

}

