package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Gate Block
 * Extension of Block class to manage Gate parameters
 */
public class Gate extends CopyBlock {
    String name="";
    int bankN;
    int presN;

    JLabel bankLbl;
    JLabel presLbl;
    JTextField bankTf;
    JTextField presTf;
    JTextField nameTf;

    Nibble type;
    Nibble threshold;
    Nibble damp;
    Nibble release;
    Nibble onoff;

    public Gate(Patch par) {	
	super("Noise Gate", par, true);

	try {
	    type = parent.getNibble(89);
	    type.updateNibble("Type", Nibble.TYPE_GATE);
	    nibbles.add( type );

	    threshold = parent.getNibble(90);
	    threshold.updateNibble("Threshold (dB)", Nibble.TYPE_N60_0);
	    nibbles.add( threshold );

	    damp = parent.getNibble(91);
	    damp.updateNibble("Damp (dB)", Nibble.TYPE_0_90);
	    nibbles.add( damp );

	    release = parent.getNibble(92);
	    release.updateNibble("Speed (/s)",Nibble.TYPE_3_200);
	    nibbles.add( release );

	    onoff = parent.getNibble(104);
	    onoff.updateNibble("On",Nibble.TYPE_OFFON);
	    nibbles.add( onoff );

	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Gate Block");
	}

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();
	int i = 0;
	activeNibbles = new int[]{ 104, 89, 90, 91, 92};

	box.add( new Row("Noise Gate", onoff, i++));
	box.add( new Row("Noise Gate", type, i++));
	box.add( new Row("Noise Gate", threshold, i++));
	box.add( new Row("Noise Gate", damp, i++));
	box.add( new Row("Noise Gate", release, i++));

	while (i<9) box.add( new EmptyRow(true, i++));

    }

    int getBlockType(Box b) {
	return GATE;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

