package nova;

import javax.swing.*;
import java.awt.event.*;

/** Dialog called when a Preset has to be downloaded from NOVA
 * User can recall :
 -- current Preset (#0)
 -- factory Preset (F0-1 to F9-3 / #1 to #30)
 -- user Preset (00-1 to 19-3 / #31 to #90)
 */
public class GetPatchDialog extends JDialog implements ActionListener {

    NovaManager parent;
    JComboBox nova = new JComboBox();
    JComboBox fBanks = new JComboBox();
    JComboBox uBanks = new JComboBox();
    JComboBox fPresets = new JComboBox();
    JComboBox uPresets = new JComboBox();
	
    public GetPatchDialog(NovaManager nm) {
	super(nm, "Get Preset(s) from Nova System", true);
	setResizable(false);
	parent = nm;
	setVisible(false);

	// JComboBoxes
	for (int i=0; i<10; i++) {
	    String num = "F";
	    fBanks.addItem(num+i);
	}
	fBanks.setSelectedIndex(0);

	for (int i=0; i<20;i++) {
	    String num ="";
	    if (i<10) num+="0";
	    uBanks.addItem(num+i);
	}
	uBanks.setSelectedIndex(0);

	for (int i=1; i<=3; i++) {
	    uPresets.addItem(""+i);
	    fPresets.addItem(""+i);
	}
	uPresets.setSelectedIndex(0);
	fPresets.setSelectedIndex(0);

	// Labels
	JLabel novaLbl = new JLabel("Select your Nova (SysexId)");
	JLabel factoryLbl = new JLabel("Factory Presets");
	JLabel userLbl = new JLabel("User Presets");
	
	// JButtons
	JButton factoryBtn = new JButton("Get Factory Preset");
	factoryBtn.setActionCommand("factory");
	factoryBtn.addActionListener(this);

	JButton userBtn = new JButton("Get User Preset");
	userBtn.setActionCommand("user");
	userBtn.addActionListener(this);

	JButton currentBtn = new JButton("Get Current Preset");
	currentBtn.setActionCommand("current");
	currentBtn.addActionListener(this);

	JButton uBankBtn = new JButton("Get User Bank");
	uBankBtn.setActionCommand("userbank");
	uBankBtn.addActionListener(this);

	JButton fBankBtn = new JButton("Get Factory Bank");
	fBankBtn.setActionCommand("factorybank");
	fBankBtn.addActionListener(this);

	// Build Interface
	java.awt.Container p = this.getContentPane();
	GroupLayout layout = new GroupLayout( p );
	p.setLayout( layout );

	layout.setHorizontalGroup(
				  layout.createSequentialGroup()
				  .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					    .addComponent( factoryLbl )
					    .addComponent( userLbl))
				  .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					    .addComponent( fBanks )
					    .addComponent( uBanks))
				  .addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING)
					    .addComponent( fPresets )
					    .addComponent( uPresets))
				  .addGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
					    .addComponent(currentBtn)
					    .addComponent(factoryBtn)
					    .addComponent(fBankBtn)
					    .addComponent(userBtn)
					    .addComponent(uBankBtn))
				  );
	layout.setVerticalGroup(
				layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					  .addComponent(currentBtn))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					  .addComponent(factoryLbl)
					  .addComponent(fBanks)
					  .addComponent(fPresets)
					  .addComponent(factoryBtn)
					  )
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					  .addComponent(fBankBtn)
					  )
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					  .addComponent(userLbl)
					  .addComponent(uBanks)
					  .addComponent(uPresets)
					  .addComponent(userBtn)
					  )
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE)
					  .addComponent(uBankBtn)
					  )
				);	
    }

    void open() {
	nova.removeAllItems();
	/*
	java.util.List novas = parent.getNovas();
	for (int i=0; i<novas.size(); i++) {
	    nova.addItem( novas.get( i ));
	}
	if (nova.getItemCount()==0)
	    nova.addItem( new Integer(0));
	nova.setSelectedIndex(0);	
	*/

	pack();	
	setVisible(true);
    }

    void openAlone() {
	nova.removeAllItems();
	/*	
	nova.addItem( new Integer(0));
	nova.addItem( new Integer(1));
	nova.addItem( new Integer(2));
	nova.setSelectedIndex(0);
	*/

	pack();	
	setVisible(true);
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Integer id = parent.getCurrentSysExID();

	if (command.equals("factory")) {
	    int b = fBanks.getSelectedIndex();
	    int p = fPresets.getSelectedIndex();
	    parent.getPatch(id.byteValue(), (byte) (1+b*3+p) );
	    setVisible( false );
	} 
	else if (command.equals("user")) {
	    int b = uBanks.getSelectedIndex();
	    int p = uPresets.getSelectedIndex();
	    
	    parent.getPatch(id.byteValue(), (byte) (31+b*3+p));
	    setVisible( false );
	}
	else if (command.equals("current")) {	    
	    parent.getPatch(id.byteValue(), (byte) 0);
	    setVisible( false );
	}
	else if (command.equals("userbank")) {	    
	    parent.getBank(id.byteValue(), 31, 90);
	    setVisible( false );
	}
	else if (command.equals("factorybank")) {	    
	    parent.getBank(id.byteValue(), 1, 30);
	    setVisible( false );
	}
    }

    public static void main(String[] args) {
	NovaManager nm = new NovaManager();
	GetPatchDialog gpd = new GetPatchDialog(nm);
	gpd.openAlone();
    }
}