package nova;

import javax.swing.*;
import java.awt.event.*;

/** Dialog called when a Variation has to be downloaded from NOVA
 * User selects a Block and the variation number (1-4).
 * Variations are stored in Preset Memory from #90 to #117
 * BUG 1.13 : clash.
 * COMP 1 is stored at the same location as Preset 19-3
 */
public class GetVariationDialog extends JDialog implements ActionListener {

    NovaManager parent;
    JComboBox nova = new JComboBox();
    JComboBox blocks = new JComboBox();
    JComboBox variations = new JComboBox();

    // Blocks are ordered like on the NOVA, not like in NOVA memory
    String[] blockNames = {"All","Drive/Boost", "Compressor", "EQ/Gate", "Modulation", "Pitch", "Delay", "Reverb"};
    int[] blockId = {-1, 1, 0, 5, 2, 6, 3, 4};
	
    public GetVariationDialog( NovaManager nm) {
	super(nm, "Get Variations from Nova System", true);
	setResizable(false);
	parent = nm;
	setVisible(false);

	// JComboBoxes
	for (int i=0; i<blockNames.length; i++)
	    blocks.addItem(blockNames[i]);
	
	blocks.setSelectedIndex(0);

	variations.addItem("1-4");
	for (int i=1; i<=4;i++) 
	    variations.addItem(""+i);
	
	variations.setSelectedIndex(0);

	// Labels
	JLabel blkLbl = new JLabel("Effect Block");
	JLabel varLbl = new JLabel("Variation #");

	JButton get = new JButton("Get");
	get.setActionCommand("get");
	get.addActionListener(this);

	// Build Interface
	java.awt.Container p = this.getContentPane();
	p.setLayout( new java.awt.GridLayout(3,2));
	
	p.add(blkLbl);
	p.add(blocks);
	p.add(varLbl);
	p.add(variations);
	p.add(new JLabel());
	p.add(get);
    }

    void open() {
	/*
	nova.removeAllItems();
	java.util.List novas = parent.getNovas();
	for (int i=0; i<novas.size(); i++) {
	    nova.addItem( novas.get( i ));
	}
	if (nova.getItemCount()==0)
	    nova.addItem( new Integer(0));
	nova.setSelectedIndex(0);	
	*/
	pack();	
	setVisible(true);
    }

    // just for standalone testing (see main)
    private void openAlone() {
	/*
	nova.removeAllItems();
	
	nova.addItem( new Integer(0));
	nova.addItem( new Integer(1));
	nova.addItem( new Integer(2));
	nova.setSelectedIndex(0);
	*/
	pack();	
	setVisible(true);
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	Integer id = (Integer)parent.getCurrentSysExID();
	int b = blocks.getSelectedIndex();
	int fx = blockId[b];
	int v = variations.getSelectedIndex(); 


	if ( b == 0 ) {//ALL variations from ALL blocks
	    parent.getBank(id.byteValue(), 0x5A, 0x75);
	} else if (v == 0) {//ALL variations from ONE block
	    parent.getBank(id.byteValue(), 90 + 4*fx, 90 + 4*fx + 3 );
	} else {
	    parent.getPatch(id.byteValue(), (byte) (90 + 4*fx + v-1 ));
	}
	setVisible(false);
    }

    public static void main(String[] args) {
	NovaManager nm = new NovaManager();
	GetVariationDialog gvd = new GetVariationDialog(nm);
	gvd.openAlone();
    }
}