package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Global Block
 * Extension of Block class to manage Global parameters
 * (Name, Levels, etc.)
 * Also set the title of the tab
 */
public class Global extends CopyBlock {
    String name="";
    int bankN;
    int presN;
    int numcode;

    JLabel bankLbl;
    JLabel presLbl;
    JTextField bankTf;
    JTextField presTf;
    JTextField nameTf;

    JLabel titleLabel;

    public Global(Patch par) {	
	super("Global settings", par, false);
	activeNibbles = new int[]{1, 2, 3, 4, 5, 6, 7, 8};

	if (parent instanceof Variation) {
	    name = ((Variation)parent).getVariationName();
	}
	else {
	    name = parent.getOriginalName();
	    bankN = parent.getOriginalBankNumber();
	    presN = parent.getOriginalPresetNumber();
	    numcode = parent.getPresetCode();
	}

	titleLabel = new JLabel("", JLabel.CENTER);
	titleLabel.setFont( new Font("Verdana", Font.BOLD, 20));
	titleLabel.setForeground( Color.white );
	titleLabel.setPreferredSize(new Dimension(300, 60));

	if (parent.parent.isEditable())
	    titleLabel.addMouseListener(parent);

	JPanel title = parent.getTitlePane();
	title.removeAll();
	title.add( titleLabel, BorderLayout.CENTER);

	try {
	    Nibble temp1 = parent.getNibble(1);
	    temp1.updateNibble("Tempo", Nibble.TYPE_100_3000);
	    nibbles.add( temp1 );

	    Nibble temp2 = parent.getNibble(2);
	    temp2.updateNibble("Routing", Nibble.TYPE_ROUTING);
	    nibbles.add( temp2 );

	    Nibble temp3 = parent.getNibble(3);
	    temp3.updateNibble("LvlOut L (dB)", Nibble.TYPE_N100_0);
	    nibbles.add( temp3 );

	    Nibble temp4 = parent.getNibble(4);
	    temp4.updateNibble("LvlOut R (dB)",Nibble.TYPE_N100_0);
	    nibbles.add( temp4 );

	    Nibble temp5 = parent.getNibble(5);
	    temp5.updateNibble("Map Param",Nibble.TYPE_MAPPARAM);
	    //temp5.setValueList( parent.getExpressionPedalValueList());
	    //temp5.setTypeList( parent.exp_pedal_valueList );
	    //temp5.decode();
	    nibbles.add( temp5 );

	    Nibble temp6 = parent.getNibble(6);
	    temp6.updateNibble("Map Min (%)",Nibble.TYPE_0_100);
	    nibbles.add( temp6 );

	    Nibble temp7 = parent.getNibble(7);
	    temp7.updateNibble("Map Med (%)",Nibble.TYPE_0_100);
	    nibbles.add( temp7 );

	    Nibble temp8 = parent.getNibble(8);
	    temp8.updateNibble("Map Max (%)",Nibble.TYPE_0_100);
	    nibbles.add( temp8 );

	} catch (Exception e) {
	    e.printStackTrace();
	    System.err.println("Not enough Nibbles to load Global Block");
	}

	refresh();
    }

    void refreshTitle() {
	String s;
	if (parent instanceof Variation) {
	    name = ((Variation)parent).getVariationName();
	    s = name;
	} else {
	    numcode = parent.getPresetCode();
	    name = parent.getOriginalName();
	    s = "Preset #"+numcode+" ["+Constants.PRESET[numcode]+"]  \""+name+"\"";
	}
	titleLabel.setText(s);
    }

    /* reformat the widgets */
    void refresh() {
	refreshTitle();
	Nibble temp5 = parent.getNibble(5);
	temp5.setValueList( parent.getExpressionPedalValueList());
	temp5.setTypeList( parent.exp_pedal_valueList );

	box.removeAll();
	box.add( new Row("Global settings", null, 0));	
	activeNibbles = new int[]{1, 2, 3, 4, 5, 6, 7, 8 };
	for (int i=0; i< nibbles.size(); i++)
	    box.add( new Row("Global settings", nibbles.elementAt(i), i+1));	
    }

    String getPresetName() {
	return nameTf.getText();
    }

    int getBankNumber() {
	int v = 0;
	try {
	    v = Integer.parseInt(bankTf.getText());
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return v;
    }

    int getPresetNumber() {
	int v = 0;
	try {
	    v = Integer.parseInt(presTf.getText());
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return v;
    }

    int getBlockType(Box b) {
	return GLOBAL;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

