package nova;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** A RowPopup is a Popup displaying allowed values for it 
    TODO : close on exit
*/
public class LongRowPopup extends JDialog implements ActionListener, TextListener, MouseListener {
    final static int maxheight = 300;

    NovaManager parent;
    Row r;
    Nibble n;
    int x,y;

    String value = "";
    Box b;
    TextField tf;
    String[] typeList;
    JLabel current;
    private boolean tempo = false;

    GridBagConstraints gc;

    Container root;


    public LongRowPopup(NovaManager parent, Row r, int x, int y) {
	super(parent, true);
	setUndecorated(false);//need toolbar to move
	setResizable(false);//no resize: ugly
	this.setDefaultCloseOperation(javax.swing.WindowConstants.HIDE_ON_CLOSE);

	this.parent = parent;
	this.r = r;
	this.x = x;
	this.y = y;
	this.n = r.getNibble();
	typeList = n.getTypeList();
	if (n.getType()==Nibble.TYPE_100_3000) tempo = true;
	
	value = n.getStringValue();

	setLayout( new GridBagLayout());
	gc = new GridBagConstraints();
	gc.insets = new Insets(2,5,2,5);
	root = getContentPane();
	setContent();
	setLocation(x,y);
    }

    void setContent() {
	root = getContentPane();
	root.removeAll();

	if (typeList == null) {
	    parent.log("Popup : typeList is null");
	    parent.log("Nibble type is "+n.getType());
	} else {
	    
	    gc.gridwidth=4;
	    gc.gridx = 0;
	    gc.gridy = 0;
	    gc.weightx = 1.0;
	    gc.weighty = 1.0;
	    gc.fill = GridBagConstraints.BOTH;
	    b = Box.createVerticalBox();
	    filter("");

	    JScrollPane js = new JScrollPane(b);
	    root.add(js, gc);
	    
	    JLabel lab = new JLabel("Filter");
	    gc.gridwidth=1;
	    gc.gridy=1;
	    gc.gridx=0;
	    gc.weightx=0.0;
	    gc.weighty=0.0;
	    root.add(lab, gc);
	    
	    tf = new TextField(10);
	    //tf.addInputMethodListener(this);
	    tf.addTextListener(this);
	    gc.gridx++;
	    gc.weightx=1.0;
	    root.add(tf, gc);

	    JButton cancel = new JButton("Cancel");
	    cancel.setActionCommand("cancel");
	    cancel.addActionListener(this);
	    gc.gridx++;
	    gc.weightx=0;
	    root.add(cancel, gc);
	    
	    JButton ok = new JButton("OK");
	    ok.setActionCommand("ok");
	    ok.addActionListener(this);
	    gc.gridx++;
	    gc.weightx=0;
	    root.add(ok, gc);
	}
    }
    
    /* call the parent component to set the new Nibble value and refresh */
    void update(String s) {
	n.setValue(s);
	r.refresh();
    }

    private void filter(String txt) {
	b.removeAll();

	if (!tempo) {
	    for (int i=0; i<typeList.length; i++) {
		String val = typeList[i];
		if ((txt.length()==0) ||(val.indexOf(txt)>=0)) {
		    JLabel l = new JLabel(val);
		    if (val.equals(value)) {
			l.setFont(RowPopup.boldFont);
			current = l;
		    }
		    else l.setFont(RowPopup.normalFont);
		    l.addMouseListener(this);
		    b.add(l);
		}
	    }
	} else 
	    for (int i=0; i<typeList.length; i++) {
		String val = typeList[i];
		int ms = Integer.parseInt(val);
		int bpm = Nibble.toBPM(ms);

		if ((txt.length()==0) ||(val.indexOf(txt)>=0)) {
		    JLabel lms = new JLabel(val);
		    JLabel lbpm = new JLabel("("+bpm+" bpm)");
		    if (val.equals(value)) {
			lms.setFont(RowPopup.boldFont);
			current = lms;
		    }
		    else {
			lms.setFont(RowPopup.normalFont);
		    }
		    lms.addMouseListener(this);
		    Box hb = Box.createHorizontalBox();
		    hb.add(lms);
		    hb.add(Box.createHorizontalStrut(50));
		    hb.add(lbpm);
		    b.add(hb);
		}
	    }

	validate();
	pack();
    }

    public void pack() {
	super.pack();
	if (getHeight()> maxheight) 
	    setSize( new Dimension(getWidth(),maxheight));
    }

    // TEXT LISTENER
    public void textValueChanged(TextEvent e) {
	filter( tf.getText() );
    }

    // ACTION LISTENER INTERFACE
    public void actionPerformed(ActionEvent e) {
	String s = e.getActionCommand();
	if (s.equals("cancel")) setVisible(false);
	else if (s.equals("ok")) {
	    if (value != null) update(value);
	    setVisible(false);
	}	    
    }


    // == MOUSE LISTENER INTERFACE //
    public void mouseClicked(MouseEvent e) {
	String s ="";
	if (current != null) current.setFont(RowPopup.normalFont);
	current = (JLabel) e.getSource();
	value = current.getText();
	current.setFont(RowPopup.boldFont);
	validate();
    }

    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
 
}