package nova;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiDevice;
import javax.sound.midi.Sequencer;
import javax.sound.midi.Synthesizer;
import javax.sound.midi.MidiDevice.Info;
import javax.sound.midi.Receiver;
import javax.sound.midi.Transmitter;
import javax.sound.midi.MidiUnavailableException;

import javax.sound.midi.MidiMessage;
import javax.sound.midi.SysexMessage;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.InvalidMidiDataException;


/* Class doing the interface with Nova through javax.sound.midi interface
   Subclass of MidiInterface
*/
public class MidiDefaultInterface extends MidiInterface {

    MidiDevice.Info[] ins;
    MidiDevice.Info[] outs;

    MidiInputBuffer buffer;
    Receiver receiver;

    MidiDevice currentInPort;
    MidiDevice currentOutPort;

    public MidiDefaultInterface() {
	super();
    }

    public MidiDefaultInterface(NovaManager np) {
	super(np);
    }

    protected void init() {
	buffer = new MidiInputBuffer( this );
	parent.logSilent("Using Javax.sound API");
    }

    void setComboListeners() {

	ib.addActionListener(new ActionListener(){	
		/* Need a MidiInputBuffer BEFORE the buildInterface */
		public void actionPerformed(ActionEvent e) {
		    if (ib.getSelectedIndex() <0) return;
		    try{
			if (currentInPort != null) currentInPort.close();
			currentInPort= javax.sound.midi.MidiSystem.getMidiDevice(ins[ib.getSelectedIndex()]);
			currentInPort.open();
			Transmitter t = currentInPort.getTransmitter();
			t.setReceiver( buffer );
		    }catch (MidiUnavailableException mue){
			parent.logSilent("can't open MidiIn : " +mue.toString());
		    }

		    if (currentOutPort != null) lookUp();	    
		}		    		
	    });
	
	ob.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    if (ob.getSelectedIndex() <0) return;		    		    
		    try{
			if (currentOutPort!= null) currentOutPort.close();
			currentOutPort= javax.sound.midi.MidiSystem.getMidiDevice(outs[ob.getSelectedIndex()]);
			currentOutPort.open();
			receiver = currentOutPort.getReceiver();
			lookUp();	    
		    }catch (MidiUnavailableException mue){
			parent.logSilent("can't open MidiOut : " +mue.toString());
		    }
		}
	    });
    }

    void rebuildCombos() {

	ib.removeAllItems();
	ob.removeAllItems();

	int numIns = 0;
	int numOuts = 0;
	
	try{
	    MidiDevice.Info[] dev = javax.sound.midi.MidiSystem.getMidiDeviceInfo();
	    for (int i = 0; i < dev.length; i++) {
		try{
		    if (javax.sound.midi.MidiSystem.getMidiDevice(dev[i]).getMaxTransmitters() != 0) 
			numIns++;
		    
		    if (javax.sound.midi.MidiSystem.getMidiDevice(dev[i]).getMaxReceivers() != 0)
			numOuts++;

		} catch (Exception me){
		    me.printStackTrace();
		}
	    }
		
	    ins = new MidiDevice.Info[numIns];
	    outs = new MidiDevice.Info[numOuts];
	    numIns = 0;
	    numOuts = 0;

	    for (int i = 0; i < dev.length; i++) {
		try{
		    MidiDevice temp = javax.sound.midi.MidiSystem.getMidiDevice(dev[i]);
		    if ((temp.getMaxTransmitters() != 0)
			&& !(temp instanceof Sequencer)
			&& !(temp instanceof Synthesizer))
			ins[numIns++] = dev[i];
		    if ((temp.getMaxReceivers() != 0)
			&& !(temp instanceof Sequencer)
			&& !(temp instanceof Synthesizer)) 
			outs[numOuts++] = dev[i];
		} catch (Exception me){		    
		    me.printStackTrace();
		}
	    }	    
	    

	    for (int i=0; i<numIns; i++) ib.addItem(ins[i]);
	    for (int i=0; i<numOuts; i++) ob.addItem(outs[i]);
	    
	} catch (Exception e){
	    e.printStackTrace();
	}
    }

    class MidiInputBuffer implements Receiver {
	MidiDefaultInterface parent;
	
	private MidiInputBuffer(MidiDefaultInterface mi) {
	    this.parent = mi;
	}

	/* Receiver interface */
	public void close() {}

	public void send(MidiMessage msg, long timestamp) {
	    final byte[] d = msg.getMessage();
	    parent.parent.logSilent("Received data ("+msg.getLength()+" bytes)");

	    Thread t = new Thread() {
		    public void run() {
			parent.processInputData( d );
		    }
		};
	    t.start();
	}
    }


    void send(byte[] data) {
	MMidiMessage msg = new MMidiMessage(data);
	parent.logSilent("Sending Message :");
	parent.logSilent(msg.toString());

	if (receiver != null) {
	    parent.logSilent("Receiver found : transmitting message");
	    receiver.send( msg, -1);
	} else { 
	    parent.logSilent("No output device : message not sent");
	}
	parent.logSilent();
    }

}
			
