//  MidiMacInterface
//  Adapted from
//    Simple mmj demo
//    Created by Nils Peters on 1/13/07.
//    Copyright (c) 2007 __humatic__. All rights reserved.
//

package nova;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.File;

import javax.sound.midi.*;
import java.util.Vector;

   

/* Class doing the interface with Nova through javax.sound.midi interface or mmj
   Subclass of MidiInterface
   Without mmj, it appears sometimes that SysEx are not sent to output.
*/
public class MidiMacInterface extends MidiInterface implements de.humatic.mmj.MidiSystemListener {
        
    private de.humatic.mmj.MidiOutput mo;
    private Receiver receiver;    
    
    javax.sound.midi.MidiDevice.Info[] ins;
    javax.sound.midi.MidiDevice.Info[] outs;
    MidiDevice currentInPort;
    MidiDevice currentOutPort;
    
    private MidiInputBuffer buffer;


    public MidiMacInterface() {
	super();
    }

    public MidiMacInterface(NovaManager np) {
	super(np);
    }

    void init() {
	enableMMJ = true;
	useMMJ = false;
	parent.logSilent("Using Javax.sound API");
	buffer = new MidiInputBuffer( this );
    }

    void setComboListeners() {
	ib.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    if (ib.getItemCount() == 0) return;
		    if (ib.getSelectedIndex() < 0) return;
		    if (useMMJ) {
			de.humatic.mmj.MidiInput mi = de.humatic.mmj.MidiSystem.openMidiInput(ib.getSelectedIndex());
			mi.addMidiListener( buffer );
		    } else {
			try{
			    if (currentInPort != null) currentInPort.close();
			    currentInPort= javax.sound.midi.MidiSystem.getMidiDevice(ins[ib.getSelectedIndex()]);
			    currentInPort.open();
			    Transmitter t = currentInPort.getTransmitter();
			    t.setReceiver( buffer );
			}catch (MidiUnavailableException mue){
			    parent.log("can't open MidiIn : " +mue.toString());
			}
		    }
		    lookUp();
		}
	    });

	ob.addActionListener(new ActionListener(){
		public void actionPerformed(ActionEvent e) {
		    if (ob.getItemCount() == 0) return;
		    if (ob.getSelectedIndex() < 0) return;
		    if (useMMJ) {
			mo = de.humatic.mmj.MidiSystem.openMidiOutput(ob.getSelectedIndex());
		    } else {
			try{
			    if (currentOutPort!= null) currentOutPort.close();
			    currentOutPort= javax.sound.midi.MidiSystem.getMidiDevice(outs[ob.getSelectedIndex()]);
			    currentOutPort.open();
			    receiver = currentOutPort.getReceiver();
			}catch (MidiUnavailableException mue){
			    parent.log("can't open MidiOut : " +mue.toString());
			}
		    }
		    lookUp();
		}
	    });

    }

    void rebuildCombos() {
	novas.removeAllElements();
	ib.removeAllItems();
	ob.removeAllItems();

	int numIns = 0;
	int numOuts = 0;
	
	try{
	    if (useMMJ){ 	   	    
		de.humatic.mmj.MidiSystem.initMidiSystem("mmj src", "mmj dest");	    
		de.humatic.mmj.MidiSystem.addSystemListener(this);	    
	    } else {	    	    
		javax.sound.midi.MidiDevice.Info[] dev = javax.sound.midi.MidiSystem.getMidiDeviceInfo();

		for (int i = 0; i < dev.length; i++) {
		    try{
			if (javax.sound.midi.MidiSystem.getMidiDevice(dev[i]).getMaxTransmitters() != 0) numIns++;
			if (javax.sound.midi.MidiSystem.getMidiDevice(dev[i]).getMaxReceivers() != 0) numOuts++;
		    } catch (Exception me){
			me.printStackTrace();
		    }
		}
		
		ins = new javax.sound.midi.MidiDevice.Info[numIns];
		outs = new javax.sound.midi.MidiDevice.Info[numOuts];
		numIns = 0;
		numOuts = 0;
		for (int i = 0; i < dev.length; i++) {
		    try{
			MidiDevice temp = javax.sound.midi.MidiSystem.getMidiDevice(dev[i]);
			if ((temp.getMaxTransmitters() != 0)
			    && !(temp instanceof Sequencer)
			    && !(temp instanceof Synthesizer))
			    ins[numIns++] = dev[i];
			if ((temp.getMaxReceivers() != 0)
			    && !(temp instanceof Sequencer)
			    && !(temp instanceof Synthesizer)) 
			    outs[numOuts++] = dev[i];
		    } catch (Exception me){		    
			me.printStackTrace();
		    }
		}	    
	    }

	    if (useMMJ) {
		for (int i=0; i< de.humatic.mmj.MidiSystem.getInputs().length; i++)
		    ib.addItem( de.humatic.mmj.MidiSystem.getInputs()[i] );
		for (int i=0; i< de.humatic.mmj.MidiSystem.getOutputs().length; i++)
		    ob.addItem( de.humatic.mmj.MidiSystem.getOutputs()[i] );
	    } else {
		for (int i=0; i<numIns; i++) ib.addItem(ins[i]);
		for (int i=0; i<numOuts; i++) ob.addItem(outs[i]);
	    }

	    //System.out.print("ib select > ");
	    //ib.setSelectedIndex(ib.getItemCount()-1);

	    //System.out.print("ob select > ");
	    //ob.setSelectedIndex(ob.getItemCount()-1);

	    // Don't need to look up again : setSelectedIndex() already did it
	    // lookUp();
	    
	} catch (Exception e){
	    e.printStackTrace();
	}
    }
     
    /* MMJ MidiSystemListener method */
    public void systemChanged() {
	super.systemChanged();
    }// End of systemChanged

	
    
    /* ItemChange Interface */
    public void itemStateChanged(ItemEvent e) {
	if (e.getStateChange()==ItemEvent.SELECTED) {
	    useMMJ=true;
	    parent.logSilent("Using MMJ");
	}
	else {
	    useMMJ=false;
	    parent.logSilent("Using Javax.sound API");
	}

	systemChanged();
    }
    
    /* InputBuffer implementing the input interfaces of both Midi implementation (mmj and javax)
     * A way to unify input processing 
     * without clash with JFrame (that also use the close method)
     */
    private class MidiInputBuffer implements de.humatic.mmj.MidiListener, Receiver {
	MidiInterface parent;
	
	private MidiInputBuffer(MidiInterface mi) {
	    this.parent = mi;
	}

	/* MidiListener interface */
	public void midiInput(byte[] data) {
	    final byte[] d = data;
	    parent.parent.logSilent("Received data ("+data.length+" bytes)");
	    Thread t = new Thread() {
		    public void run() {
			parent.processInputData( d );
		    }
		};
	    t.start();
	}

	/* Receiver interface */
	public void close() {}

	public void send(MidiMessage msg, long timestamp) {
	    final byte[] d = msg.getMessage();
	    parent.parent.logSilent("Received data ("+msg.getLength()+" bytes)");
	    Thread t = new Thread() {
		    public void run() {
			parent.processInputData( d );
		    }
		};
	    t.start();
	}
    }

    void send(byte[] data) {
	MMidiMessage msg = new MMidiMessage(data);
	parent.logSilent("Sending Message :");
	parent.logSilent(msg.toString());

	if (useMMJ) { 
	    if (mo != null) {
		parent.logSilent("Receiver found : transmitting message");
		mo.sendMidi( data );
	    } else { 
		parent.logSilent("No output device : message not sent");
	    }
	}
	else {
	    if (receiver != null) {
		parent.logSilent("Receiver found : transmitting message");
		receiver.send( msg, -1);
	    } else { 
		parent.logSilent("No output device : message not sent");
	    }
	}
    }

}
			

