package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The MidiMapIn Block
 * Extension of Block class to manage MidiMapIn parameters
 * in SystemDump
 */
public class MidiMapIn extends Block {
    String name="";
    JPanel p;

    public MidiMapIn(SystemDump par) {	
	super("MIDI Map In", par);

	try {
	    for (int i=0; i<=42; i++) {
		Nibble n = system.getNibble(64+i);
		if (i<42) 
		    n.updateNibble(""+(3*i+1)+"-"+(3*i+3), Nibble.TYPE_MIDI_MAP);
		else 
		    n.updateNibble(""+(3*i+1), Nibble.TYPE_MIDI_MAP);
		nibbles.add( n );
	    }
	    
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load MIDI MAP IN Block");
	    System.err.println(system.getNibbleCount());
	}

	setBorder( border );
	refresh();	
    }

    /* reformat the widgets */
    void refresh() {
	removeAll();
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0,0,0,0);
	c.weightx = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.gridy = 0;
	add( new Row("MIDI MAP IN", null, 0), c);
	c.gridx++;
	for (int i=0; i<4; i++) {
	    if (i == 4) c.weightx = 1.0;
	    add( new Row("",null, 0), c);
	    c.gridx++;
	}
	
	for (int i=0; i<=nibbles.size()/5; i++) {
	    c.gridx = 0;
	    c.gridy++;
	    c.weightx = 0.0;
	    for (int j = 0; j< 5; j++) {
		if (j == 4) c.weightx = 1.0;
		if (5*i + j >= nibbles.size()) 
		    add( new EmptyRow(false, 1), c);
		else
		    add( new MidiMapInRow( 5*i+j, nibbles.elementAt(5*i+j), i+1), c );
		c.gridx++;
	    }
	    
	}
    }

    public String toHTML() {
	String s = "";
	s += "<H2>MIDI Map In</H2>\n";
	s += "<TABLE class=MIDI border=0 cols=24 cellspacing=5 width=\"1000\">\n";
	for (int i=0; i<=nibbles.size()/5; i++) {
	    s +="<TR>\n";
	    for (int j = 0; j<5; j++) {
		int k = 5*i +j ;
		if (k < nibbles.size()) {
		    Nibble n = nibbles.elementAt( k );
		    int val = n.getMapMidiValue1();
		    String preset;
		    if ((val > 0) && (val < 91)) preset = Constants.PRESET[val];
		    else preset = "None";
		    s += "<TD align=center>\n";
		    s += "&nbsp;"+(3*k+1) + "&nbsp;>&nbsp;"+preset+"&nbsp;";
		    s += "</TD>\n";
		
		    s += "<TD align=center>\n";
		    if (3*k+2 < 128) {
			val = n.getMapMidiValue2();
			if ((val > 0) && (val < 91)) preset = Constants.PRESET[val];
			else preset = "None";
			s += "&nbsp;"+(3*k+2) + "&nbsp;>&nbsp;"+preset+"&nbsp;";
		    }
		    s += "</TD>\n";
	    
		    s += "<TD align=center>\n";
		    if (3*k+3 < 128) {
			val = n.getMapMidiValue3();
			if ((val > 0) && (val < 91)) preset = Constants.PRESET[val];
			else preset = "None";
			s += "&nbsp;"+(3*k+3) + "&nbsp;>&nbsp;"+preset+"&nbsp;";
		    }
		    s += "</TD>\n";
		}
	    }
	    s+="</TR>\n";
	}
	
	s += "</TABLE>\n";
	return s;
    }

    public String toXML() {
	String s = "";
	s += "<block name=\"MIDI Map In\">\n";
	for (int k=0; k<nibbles.size(); k++) {
	    Nibble n = nibbles.elementAt( k );
	    s+=n.toXMLOpen();

	    int val = n.getMapMidiValue1();
	    String preset;
	    if ((val > 0) && (val < 91)) preset = Constants.PRESET[val];
	    else preset = "None";
	    s += preset;

	    if (3*k+2 < 128) {
		val = n.getMapMidiValue2();
		if ((val > 0) && (val < 91)) preset = Constants.PRESET[val];
		else preset = "None";
		s += ", "+preset;
	    }
	    
	    if (3*k+3 < 128) {
		val = n.getMapMidiValue3();
		if ((val > 0) && (val < 91)) preset = Constants.PRESET[val];
		else preset = "None";
		s += ", "+preset;		
	    }
	    s+=n.toXMLClose();
	}
	
	s += "</block>\n";
	return s;
    }
}

