package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The MidiMapOut Block
 * Extension of Block class to manage MidiMapOut parameters
 * in SystemDump
 */
public class MidiMapOut extends Block {
    String name="";
    JPanel p;

    public MidiMapOut(SystemDump par) {	
	super("MIDI Map Out", par);
	// Load values at specific memory locations
	// Slots 107 to 126 in SystemDump Sysex
	try {
	    for (int i=0; i<20; i++) {
		Nibble n = system.getNibble(107+i);
		n.updateNibble(""+i, Nibble.TYPE_MIDI_MAP);
		nibbles.add( n );
	    }
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load MIDI MAP OUT Block");
	    System.err.println(system.getNibbleCount());
	}
	setBorder( border );
	refresh();	
    }

    /* reformat the widgets */
    void refresh() {
	removeAll();
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0,0,0,0);
	c.weightx = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridx = 0;
	c.gridy = 0;
	add( new Row("MIDI MAP OUT", null, 0), c);
	c.gridx++;
	for (int i=0; i<4; i++) {
	    if (i == 4) c.weightx = 1.0;
	    add( new Row("",null, 0), c);
	    c.gridx++;
	}
	
	for (int i=0; i<nibbles.size()/5; i++) {
	    c.gridx = 0;
	    c.gridy++;
	    c.weightx = 0.0;
	    for (int j = 0; j< 5; j++) {
		if (j == 4) c.weightx = 1.0;
		add( new MidiMapOutRow( 5*i+j, nibbles.elementAt(5*i+j), i+1), 
		     c );
		c.gridx++;
	    }
	}
    }

    public String toHTML() {
	String s = "";
	s += "<H2>MIDI Map Out</H2>\n";
	s += "<TABLE class=MIDI border=0 cols=24 cellspacing=5 width=\"1000\">\n";
	for (int i=0; i<=nibbles.size()/4; i++) {
	    s +="<TR>\n";
	    for (int j = 0; j<4; j++) {
		int k = 4*i +j ;
		if (k < nibbles.size()) {
		    Nibble n = nibbles.elementAt( k );
		    int val = n.getMapMidiValue1();
		    String preset;
		    preset = Constants.PRESET[31+3*k];
		    s += "<TD align=center>\n";
		    s += "&nbsp;"+ preset + "&nbsp;>&nbsp;"+val+"&nbsp;";
		    s += "</TD>\n";
		
		    s += "<TD align=center>\n";
		    if (3*k+2 < 128) {
			val = n.getMapMidiValue2();
			preset = Constants.PRESET[32+3*k];
			s += "&nbsp;"+ preset + "&nbsp;>&nbsp;"+val+"&nbsp;";
		    }
		    s += "</TD>\n";
	    
		    s += "<TD align=center>\n";
		    if (3*k+3 < 128) {
			val = n.getMapMidiValue3();
			preset = Constants.PRESET[33+3*k];
			s += "&nbsp;"+ preset + "&nbsp;>&nbsp;"+val+"&nbsp;";
		    }
		    s += "</TD>\n";
		}
	    }
	    s+="</TR>\n";
	}
	
	s += "</TABLE>\n";
	return s;
    }

    public String toXML() {
	
	String s = "";
	s += "<block name=\"MIDI Map Out\">\n";
	for (int k=0; k<nibbles.size(); k++) {
	    
	    Nibble n = nibbles.elementAt( k );
	    s+=n.toXMLOpen();
	    
	    int val = n.getMapMidiValue1();
	    s+= val;

	    if (3*k+2 < 128) {
		val = n.getMapMidiValue2();
		s+=", "+val;
	    }
	
	    if (3*k+3 < 128) {
		val = n.getMapMidiValue3();
		s+=", "+val;
	    }
	    s += n.toXMLClose();
	}
	
	s += "</block>\n";
	return s;
    }
}

