package nova;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

/** A RowPopup is a Popup displaying allowed values for it 
*/
public class MidiRowPopup extends Window implements MouseListener, ItemListener, ActionListener {

    private NovaManager parent;
    private Row r;
    private int x;
    private int y;
    private GridBagConstraints gc;
    private boolean in;

    private JTextField[] tf = new JTextField[3];
    private JComboBox[] bk = new JComboBox[3];
    private JComboBox[] n = new JComboBox[3];
    private JCheckBox[] cb = new JCheckBox[3];

    private JButton cancel = new JButton("Cancel");
    private JButton ok = new JButton("Save");

    private Nibble nib ;
    private int number=0;
    
    public MidiRowPopup(NovaManager parent, Row r, int x, int y) {
	super(parent);
	this.parent = parent;
	this.r = r;
	this.x = x;
	this.y = y;

	nib = r.getNibble();
	
	if (r instanceof MidiMapInRow) {
	    in = true;
	    number = ((MidiMapInRow) r).getN();
	}
	else {
	    in = false;
	    number = ((MidiMapOutRow) r).getN();
	}

	for (int i=0; i<3; i++) {
	    tf[i] = new JTextField(3);
	    bk[i] = new JComboBox( Constants.ALLBANKS );
	    n[i] = new JComboBox( Constants.NUM );
	    cb[i] = new JCheckBox("None");
	    cb[i].addItemListener(this);

	    int j = nib.getMapMidiValue(i);
	    if (j>91) j=0;

	    if (!in) 
		tf[i].setText(""+j);
	    else {
		if (j==0) {
		    cb[i].setSelected(true);
		    setCb(i, false);
		} else {
		    cb[i].setSelected(false);
		    setCb(i, true);
		    bk[i].setSelectedIndex( (j-1) /3 );
		    n[i].setSelectedIndex( (j-1) %3 );
		}
	    }
	}

	cancel.setActionCommand("cancel");
	cancel.addActionListener(this);
	ok.setActionCommand("ok");
	ok.addActionListener(this);

	setLayout( new GridBagLayout());
	gc = new GridBagConstraints();
	gc.insets = new Insets(2,5,2,5);
	setContent();
	setLocation(x,y);

    }

    void setContent() {
	removeAll();

	if (in) {
	    gc.gridwidth=2;
	    gc.gridy = 0;
	    gc.gridx = 0;
	    add( new JLabel(""+(3*number+1), JLabel.CENTER), gc);
	    gc.gridy++;
	    add( cb[0], gc);
	    gc.gridy++;
	    gc.gridwidth=1;
	    add( bk[0], gc);
	    gc.gridx += 1;
	    add( n[0], gc);   

	    if (number<42) {
		gc.gridwidth=2;
		gc.gridy = 0;
		gc.gridx = 2;
		add( new JLabel(""+(3*number+2), JLabel.CENTER), gc);
		gc.gridy++;
		add( cb[1], gc);
		gc.gridy++;
		gc.gridwidth=1;
		add( bk[1], gc);
		gc.gridx += 1;
		add( n[1], gc);  

		gc.gridwidth=2;
		gc.gridy = 0;
		gc.gridx = 4;
		add( new JLabel(""+(3*number+3), JLabel.CENTER), gc);
		gc.gridy++;
		add( cb[2], gc);
		gc.gridy++;
		gc.gridwidth=1;
		add( bk[2], gc);
		gc.gridx += 1;
		add( n[2], gc);  
 		
		gc.gridwidth=3;
		gc.gridx = 0;
		gc.gridy = 3;
		add( cancel, gc);
		gc.gridx +=3;
		add( ok, gc);
	    } else {
		gc.gridwidth=1;
		gc.gridx = 0;
		gc.gridy = 3;
		add( cancel, gc);
		gc.gridx++;
		add( ok, gc);	
	    } 
	} else {
	    gc.gridx=0;
	    gc.gridy=0;
	    add( new JLabel(""+Constants.BANKS[number]+"-1",JLabel.CENTER), gc);
	    gc.gridy++;
	    add( tf[0], gc);

	    gc.gridx=1;
	    gc.gridy=0;
	    add( new JLabel(""+Constants.BANKS[number]+"-2",JLabel.CENTER), gc);
	    gc.gridy++;
	    add( tf[1], gc);

	    gc.gridx=2;
	    gc.gridy=0;
	    add( new JLabel(""+Constants.BANKS[number]+"-3",JLabel.CENTER), gc);
	    gc.gridy++;
	    add( tf[2], gc);

	    gc.gridx = 0;
	    gc.gridy = 3;
	    add( cancel, gc);
	    gc.gridx +=2;
	    add( ok, gc);	

	}
    }

    private void setCb( int i, boolean state ) {
	bk[i].setEnabled( state );
	n[i].setEnabled( state );
    }

    
    /* call the parent component to set the new Nibble value and refresh */
    void update(int[] val) {
	Nibble n = r.getNibble();
	n.setMidiValue(val);
	if (in) {
	    MidiMapInRow m = (MidiMapInRow) r;
	    m.doit();
	} else {
	    MidiMapOutRow m = (MidiMapOutRow) r;
	    m.doit();
	}
	r.validate();
    }

    public void setVisible(boolean state) {
	super.setVisible( state && parent.isEditable() );
    }

    private void save() {
	String[] s = new String[3];
	int[] val = new int[3];

	if (!in) {
	    for (int i=0; i<3; i++) {
		s[i] = tf[i].getText();
		try { 
		    val[i] = Integer.parseInt(s[i]);
		    if (val[i]<0) val[i]=0;
		    if (val[i]>127) val[i]=0;
		} catch (Exception pe) { val[i] = 0; }
	    }
	}
	else {
	    for (int i=0; i<3; i++) {
		int bank = bk[i].getSelectedIndex();
		int num = n[i].getSelectedIndex();
		if (cb[i].isSelected()) val[i] = 0;
		else val[i] = 1 + 3*bank + num;
	    }
	}

	update(val);
	setVisible(false);
    }

    // == MOUSE LISTENER INTERFACE //
    public void mouseClicked(MouseEvent e) {
    }
    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }

    public void itemStateChanged(ItemEvent ie) {
       JCheckBox c = (JCheckBox) ie.getSource();
       int i = -1;
       if (c.equals(cb[0])) i = 0;
       if (c.equals(cb[1])) i = 1;
       else if (c.equals(cb[2])) i = 2;
       
       if (i>=0) {
	   boolean state = c.isSelected();
	   setCb(i, !state);
       }
       
    } 

   public void actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
        if (action.equals("cancel")) setVisible(false);
        else if (action.equals("ok")) save();
   }


}