package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Modulation Block
 * Extension of Block class to manage Modulation parameters
 * in Presets and Variations
 */
public class Modulation extends CopyBlock {

    Nibble type;
    Nibble speed;
    Nibble depth;
    Nibble tempo;
    Nibble hicut;
    Nibble feedback;
    Nibble fb_hicut;
    Nibble delay;
    Nibble width;
    Nibble typeT;
    Nibble range;
    Nibble mix;
    Nibble onoff;

    public Modulation(Patch par) {	
	super("Modulation", par, true);
	activeNibbles = new int[] { 56, 41, 42, 43, 44, 45, 46, 47, 48, 
				    51, 52, 53, 54 } ;

	try {
	    for (int i=0; i<activeNibbles.length; i++) 
		nibbles.add( parent.getNibble( activeNibbles[i]));

	    onoff = parent.getNibble(56);
	    type = parent.getNibble(41);
	    speed = parent.getNibble(42);
	    depth = parent.getNibble(43);
	    tempo = parent.getNibble(44);
	    hicut = parent.getNibble(45);
	    feedback = parent.getNibble(46);
	    fb_hicut = parent.getNibble(47);
	    delay = parent.getNibble(48);

	    width = parent.getNibble(51);
	    typeT = parent.getNibble(52);
	    range = parent.getNibble(53);	    
	    mix = parent.getNibble(54);

	    type.updateNibble("Type", Nibble.TYPE_MOD);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Modulation Block");
	}

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	int mod = type.getValue();

	int i = 0;
	Row r;

	box.removeAll();
	for (int j=2; j<activeNibbles.length; j++) {
	    Nibble n = parent.getNibble( activeNibbles[j]);
	    n.setType(Nibble.TYPE_INT);
	}
	box.add( new Row("Modulation", onoff, i++));
	box.add( new Row("Modulation", type, i++));

	switch (mod) {
	case 0://CHORUS
	    activeNibbles = new int[]{ 56, 41, 42, 44, 43,  45, 48, 54};

	    speed.updateNibble("Speed (Hz)", Nibble.TYPE_SPEED);
	    tempo.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    depth.updateNibble("Depth (%)", Nibble.TYPE_0_100);

	    hicut.updateNibble("Hi Cut (Hz)", Nibble.TYPE_HICUT);
	    delay.updateNibble("Delay (ms)", Nibble.TYPE_DECI_01_50);
	    mix.updateNibble("Mix (%)", Nibble.TYPE_0_100);

	    box.add( new Row("", speed, i++));
	    box.add( new Row("", tempo, i++));
	    box.add( new Row("", depth, i++));

	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", delay, i++));
	    box.add( new Row("", mix, i++));
	    break;
	case 1://FLANGER
	    activeNibbles = new int[]{ 56, 41,  42, 44, 43,  45, 46, 47, 48, 54};

	    speed.updateNibble("Speed (Hz)", Nibble.TYPE_SPEED);
	    tempo.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    depth.updateNibble("Depth (%)", Nibble.TYPE_0_100);

	    hicut.updateNibble("Hi Cut (Hz)", Nibble.TYPE_HICUT);
	    feedback.updateNibble("Feedback (%)", Nibble.TYPE_N100_100);
	    fb_hicut.updateNibble("Feedback HiCut (Hz)", Nibble.TYPE_HICUT);

	    delay.updateNibble("Delay (ms)", Nibble.TYPE_DECI_01_50);
	    mix.updateNibble("Mix (%)", Nibble.TYPE_0_100);

	    box.add( new Row("", speed, i++));
	    box.add( new Row("", tempo, i++));
	    box.add( new Row("", depth, i++));

	    box.add( new Row("", hicut, i++));
	    box.add( new Row("", feedback, i++));
	    box.add( new Row("", fb_hicut, i++));

	    box.add( new Row("", delay, i++));
	    box.add( new Row("", mix, i++));
	    break;
	case 2://VIBRATO
	    activeNibbles = new int[]{ 56, 41,  42, 44, 43,  45};

	    speed.updateNibble("Speed (Hz)", Nibble.TYPE_SPEED);
	    tempo.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    depth.updateNibble("Depth (%)", Nibble.TYPE_0_100);
	    hicut.updateNibble("Hi Cut (Hz)", Nibble.TYPE_HICUT);

	    box.add( new Row("", speed, i++));
	    box.add( new Row("", tempo, i++));
	    box.add( new Row("", depth, i++));
	    box.add( new Row("", hicut, i++));
	    break;
	case 3://PHASER
	    activeNibbles = new int[]{ 56, 41,  42, 44, 43,  53, 46, 54};

	    speed.updateNibble("Speed (Hz)", Nibble.TYPE_SPEED);
	    tempo.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    depth.updateNibble("Depth (%)", Nibble.TYPE_0_100);

	    range.updateNibble("Range", Nibble.TYPE_LOHI);
	    feedback.updateNibble("Feedback (%)", Nibble.TYPE_N100_100);
	    mix.updateNibble("Mix (%)", Nibble.TYPE_0_100);

	    box.add( new Row("", speed, i++));
	    box.add( new Row("", tempo, i++));
	    box.add( new Row("", depth, i++));

	    box.add( new Row("", range, i++));
	    box.add( new Row("", feedback, i++));
	    box.add( new Row("", mix, i++));
	    break;
	case 4: //TREMOLO
	    activeNibbles = new int[]{ 56, 41,  42, 44, 43,  52, 51, 45};

	    speed.updateNibble("Speed (Hz)", Nibble.TYPE_SPEED);
	    tempo.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    depth.updateNibble("Depth (%)", Nibble.TYPE_0_100);

	    typeT.updateNibble("Type", Nibble.TYPE_SOFTHARD);
	    width.updateNibble("Width (%)", Nibble.TYPE_0_100);
	    hicut.updateNibble("Hi Cut (Hz)", Nibble.TYPE_HICUT);

	    box.add( new Row("", speed, i++));
	    box.add( new Row("", tempo, i++));
	    box.add( new Row("", depth, i++));

	    box.add( new Row("", typeT, i++));
	    box.add( new Row("", width, i++));
	    box.add( new Row("", hicut, i++));
	    break;  
	case 5://PANNER
	    activeNibbles = new int[]{ 56, 41,  42, 44, 43};

	    speed.updateNibble("Speed (Hz)", Nibble.TYPE_SPEED);
	    tempo.updateNibble("Tempo", Nibble.TYPE_TEMPO);
	    depth.updateNibble("Depth (%)", Nibble.TYPE_0_100);

	    box.add( new Row("", speed, i++));
	    box.add( new Row("", tempo, i++));
	    box.add( new Row("", depth, i++));
	    break;
	}

	while (i<14)	   
	    box.add( new EmptyRow(true, i++));    	    
    }

    int getBlockType(Box b) {
	return MOD;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

