package nova;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.util.List;
import java.util.Arrays;

/** A Nibble is a MIDI value encoded on 4 bytes 
    The MSB of each byte must be set to 0.
    Value can be negative.
    In NOVA System, Min is -2400 ; Max is +2400 (shifter Voice1).
    Nibble is also a Component with a Label, a type and a field value

    In the futur, Nibble will be editable
*/
public class Nibble extends JPanel {
    
    private byte[] bytes;
    private int value;
    private String stringValue ="unknown" ;
    private int offset = 0;
    private boolean editable = true;
    private PrintablePanel parent = null; // for Log

    final static String[] test={"Test", "Test"};

    private final static int MINVALUE = -16384; // x00 00 7F 07 = -16384
    private final static int MAXVALUE = 16383;  // x7F 7F 00 00 = 16383
    
    final static int TYPE_DECI=1; // 1 <=> 0.1
    final static int TYPE_ONOFF=2; // 0 = ON ; 1 = OFF
    final static int TYPE_ROUTING=3; // SemiPar, Serial, Parallel
    final static int TYPE_MAPPARAM=4; // Off, Drive... = le parametre qu'on controle avec la pedale d'expression
    final static int TYPE_OFFON=5; // 1 = ON ; 0 = OFF
    final static int TYPE_PRESET=6;
    final static int TYPE_CPRESET=7;

    final static int TYPE_ATTACK=10;
    final static int TYPE_RATIO=11;
    final static int TYPE_RELEASE=12;
    final static int TYPE_EQFREQ=13;
    final static int TYPE_EQWIDTH=14;

    final static int TYPE_SPEED=20;
    final static int TYPE_TEMPO=21;
    final static int TYPE_HICUT=22;
    final static int TYPE_LOCUT=24;
    final static int TYPE_HILO=25;
    final static int TYPE_HARDSOFT=26;
    final static int TYPE_SOFTHARD=27;
    final static int TYPE_LOHI=28;
    //final static int TYPE_LOCUT=27;

    final static int TYPE_HICOL=30;
    final static int TYPE_LOCOL=31;
    final static int TYPE_SHAPE=32;
    final static int TYPE_SIZE=33;

    final static int TYPE_KEY=40;
    final static int TYPE_SCALE=41;
    final static int TYPE_DEGREES=42;
    final static int TYPE_UPDOWN=43;
    final static int TYPE_DOWNUP=44;

    final static int TYPE_COMP=50;
    final static int TYPE_DRIVE=51;
    final static int TYPE_DELAY=52;
    final static int TYPE_REVERB=53;
    final static int TYPE_MOD=54;
    final static int TYPE_PITCH=55;
    final static int TYPE_GATE=56;

    final static int TYPE_PEDAL=60;
    final static int TYPE_PEDALMASTER=61;//Priority Preset vs Pedal
    final static int TYPE_TAPMASTER=62; //Preset vs. Tap
    final static int TYPE_FOOTSWITCH=63;
    final static int TYPE_INPUTSRC=64;
    final static int TYPE_DCLOCK=65;
    final static int TYPE_DITHER=66;
    final static int TYPE_OUTPUTRANGE=67;
    final static int TYPE_VOLUMEPOS=68;
    final static int TYPE_TUNEROUT=69;

    final static int TYPE_MIDI_CC=70;
    final static int TYPE_MIDI_CHANNEL=71;
    final static int TYPE_MIDI_SYSEX=72;
    final static int TYPE_MIDI_MAP=73;
    final static int TYPE_TUNERMODE=74;
    final static int TYPE_TUNERRANGE=75;
    final static int TYPE_IMPEDANCE=76;

    final static int TYPE_DECI_01_50 = 90; // 1 <=> 0.1
    final static int TYPE_DECI_01_20 = 91;
    final static int TYPE_N100_OFF_0 = 92; // Must be < 100

    //TYPES over 100 are INT values
    final static int TYPE_INT=100; // 1 <=> 1
    final static int TYPE_N200_200 = 101;
    final static int TYPE_N100_0 = 102;
    final static int TYPE_N100_100 = 103;
    final static int TYPE_N99_15 = 104;
    final static int TYPE_N60_0 = 105;
    final static int TYPE_N50_0 = 106;
    final static int TYPE_N50_50 = 107;
    final static int TYPE_N30_0 = 108;
    final static int TYPE_N25_25 = 109;
    final static int TYPE_N12_12 = 110;
    final static int TYPE_0_10 = 111;
    final static int TYPE_0_24 = 112;
    final static int TYPE_0_30 = 113;
    final static int TYPE_N40_0 = 114;
    final static int TYPE_0_90 = 115;
    final static int TYPE_0_100 = 116;
    final static int TYPE_0_120 = 117;
    final static int TYPE_0_200 = 118;
    final static int TYPE_0_1800 = 119;
    final static int TYPE_1_2 = 120;
    final static int TYPE_1_10 = 121;
    final static int TYPE_1_20 = 122;
    final static int TYPE_100_3000 = 123;
    final static int TYPE_0_18 = 124;
    final static int TYPE_3_200 = 125;
    final static int TYPE_0_350 = 126;
    final static int TYPE_N2400_2400 = 127;
    final static int TYPE_0_50 = 128;
    final static int TYPE_N100_6 = 129;
    final static int TYPE_420_460 = 130;
    final static int TYPE_N6_18 = 131;
    final static int TYPE_N99_0 = 132;
    final static int TYPE_N99_12 = 133;

    private JLabel label=new JLabel();
    private String name="";
    private int type = TYPE_INT;
    private Box vbox = Box.createVerticalBox();
    private String[] typeList = Constants.ONOFF;
    private List<String> valueList;
   
    public Nibble(PrintablePanel p, byte[] bArray) throws NibbleException {
	super();
	this.parent = p;
	//check(bArray);	
	bytes = new byte[4];
	for (int i=0; i<4; i++) bytes[i]=bArray[i];
	value = toInteger( bytes );
	setLayout( new BorderLayout());
	setType(TYPE_ONOFF);
    }
    
    public Nibble(PrintablePanel p, int val)  throws NibbleException  {
	super();
	this.parent = p;
	value = val;
	bytes = toBytes( value );
	setLayout( new BorderLayout());
	setType(TYPE_ONOFF);
    }

    public Nibble(PrintablePanel p, byte[] bArray, int offset) throws NibbleException {
	super();
	//check(bArray);	
	this.parent = p;
	bytes = new byte[4];
	for (int i=0; i<4; i++) bytes[i]=bArray[i];
	value = toInteger( bytes );
	this.offset = offset;
	setLayout( new BorderLayout());
	setType(TYPE_INT);
    }
    
    public static int toInteger(byte[] bArray) throws NibbleException {
	int val=0;
	//check( bArray );	
	
	if (bArray[3]==0x00) {
	    // positive
	    val += bArray[0];
	    val += bArray[1]*128;
	} else {
	    // negative
	    val += bArray[0] - 128;
	    val += (bArray[1] - 127)*128;
	}
	return val;
    }
    
    public static byte[] toBytes(int val) throws NibbleException {
	byte[] b = new byte[4];
	//check( val );	
	if (val >= 0) {
	    b[0] = (byte) (val % 128);
	    b[1] = (byte) (val / 128 );
	    b[2] = 0x00;
	    b[3] = 0x00;
	} else {
	    b[0] = (byte) ( 0x80 - (-val % 128 )) ;
	    b[1] = (byte) (val / 128 + 127);
	    b[2] = 0x7F;
	    b[3] = 0x07;
	}
	return b;
    }
    
    public byte[] getBytes() {
	return bytes;
    }

    public int getValue() {
	return value;
    }

    public String getStringValue() {
	return stringValue;
    }

    public char getASCII() {
	return (char) (value % 128);
    }

    /** set the integer value and update the bytes code accordingly */
    public void setValue(int i) throws NibbleException {
	value = i;
	bytes = toBytes( value );	
	if (type != TYPE_MIDI_MAP) decode();
    }

    /** set the value with String argument */
    public void setValue(String s) {
	int finalVal = 0;
	
	try {
	    finalVal = encode(s);
	} catch (OutOfBoundsException e) {
	    finalVal = getDefaultValue();
	    log(e.getMessage());
	} 
	finally {
	    try {
		setValue(finalVal);
	    } catch (NibbleException x) {
		x.printStackTrace();
	    }
	}
    }

    private int encode(String s) throws OutOfBoundsException {
	int val = valueList.indexOf( s );
	if (val >=0 ) {
	    switch (type) {
		//there may be an offset to add
	    case TYPE_MAPPARAM:
		List l = Arrays.asList(typeList);
		value = l.indexOf(s) - 1;
		break;
	    case TYPE_CPRESET:
		value = val+1;
		break;
	    case TYPE_RELEASE:
		value = val+3;
		break;
	    case TYPE_EQFREQ:
		value = val+25;
		break;
	    case TYPE_EQWIDTH:
		value = val+3;
		break;
	    case TYPE_DEGREES:
		value = val-12;
		break;
	    case TYPE_N200_200:
		value = val-200;
		break;
	    case TYPE_N100_0:
		value = val-100;
		break;
	    case TYPE_N100_OFF_0:
		value = val-100;
		break;
	    case TYPE_N100_100:
		value = val-100;
		break;
	    case TYPE_N99_15:
		value = val-99;
		break;
	    case TYPE_N99_0:
		value = val-99;
		break;
	    case TYPE_N99_12:
		value = val-99;
		break;
	    case TYPE_N60_0:
		value = val-60;
		break;
	    case TYPE_N50_0:
		value = val-50;
		break;
	    case TYPE_N50_50:
		value = val-50;
		break;
	    case TYPE_N40_0:
		value = val-40;
		break;
	    case TYPE_N30_0:
		value = val-30;
		break;
	    case TYPE_N6_18:
		value = val-6;
		break;
	    case TYPE_N25_25:
		value = val-25;
		break;
	    case TYPE_N12_12:
		value = val-12;
		break;
	    case TYPE_1_2:
		value = val+1;
		break;
	    case TYPE_1_10:
		value = val+1;
		break;
	    case TYPE_1_20:
		value = val+1;
		break;
	    case TYPE_100_3000:
		value = val+100;
		break;
	    case TYPE_3_200:
		value = val+3;
		break;
	    case TYPE_N2400_2400:
		value = val-2400;
		break;
	    case TYPE_N100_6:
		value = val-100;
		break;
	    case TYPE_420_460:
		value = val+420;
		break;
	    case TYPE_DECI_01_20:
		value = val + 1; // 0.1 <=> val = 0 => value = 1
		break;
	    default:
		value = val;
	    }
	} else {
	    throw new OutOfBoundsException(s,this);
	}
	return value;
    }


    /* spread values across 4 bytes for MidiMap Encoding */
    void setMidiValue(int[] val) {
	int o1 = val[2];
	int o2 = (val[1] % 64) * 2;
	int o3 = (val[0] % 32) * 4 + (val[1] / 64 );
	int o4 = val[0] / 32 ;
	
	bytes[0] = (byte) o1;
	bytes[1] = (byte) o2;
	bytes[2] = (byte) o3;
	bytes[3] = (byte) o4;
    }

    /** set the byte codes and update the values accordingly */
    public void setBytes(byte[] bArray) throws NibbleException {
	for (int i=0; i<4; i++) bytes[i]=bArray[i];
	value = toInteger( bytes );
	decode();
    }

    
    private static void check( byte[] ba ) throws NibbleException {
	boolean b = false;

	if (ba == null) throw new NibbleException(0, ba);

	if (ba.length!=4) throw new NibbleException(1, ba);

	if ((ba[3]!=0x00) && (ba[3]!=0x07)) 
	    throw new NibbleException(2, ba);

	if ((ba[3]==0x00) && (ba[2]!=0x00)) 
	    throw new NibbleException(3, ba);

	if ((ba[3]==0x07) && (ba[2]!=0x7F)) 
	    throw new NibbleException(3, ba);

	if ( ((ba[0] & 0x80)==0x80) || 
	     ((ba[1] & 0x80)==0x80) )
	    throw new NibbleException(4, ba);	
    }

    private static void check( int val ) throws NibbleException {
	if ( val < MINVALUE ) throw new NibbleException( 5, val);
	if ( val > MAXVALUE ) throw new NibbleException( 6, val);
    }

    public String toString() {
	String s = ""+value+" =";
	for (int i=0; i<4; i++) 
	    s+=" "+ Integer.toString( ( bytes[i] & 0xff ) + 0x100, 16).substring( 1 );
	return s.toUpperCase();
    }

    public String toHexa() {
	String s = "";
	for (int i=0; i<4; i++) 
	    s+=Integer.toString( ( bytes[i] & 0xff ) + 0x100, 16).substring( 1 );
	return s.toUpperCase();
    }

    public String toDecimal() {
	String s = ""+value;
	return s;
    }

    public String toAscii() {
	String s = ""+(char) bytes[0];
	return s;
    }

    public String toXML() {
	String vals ="";
	String s;

	s = this.toXMLOpen();
	s += stringValue;
	s += this.toXMLClose();
    
	return s;
    }

   public String toXMLOpen() {
	String s = "<param ";
	s+="name=\""+ name + "\" ";
	s+="value=\""+ toHexa() + "\" ";
	s+="type=\""+ type + "\"";
	s+=">";
	return s;
    }

   public String toXMLClose() {
	String s = "</param>\n";
	return s;
    }

    public String toHTML() {
	String vals ="";
	String s;

	s = "          <BR/>"+name + ": <I>"+stringValue+"</I>\n";
	
	return s;
    }


    int getType() {
	return type;
    }

    String[] getTypeList() {
	return typeList;
    }

    void setTypeList( String[] array) {
	this.typeList = array;
	if (type != TYPE_MIDI_MAP) decode();
    }

    public void save(byte[] bArray) {
	for (int i=0; i<4; i++) bArray[offset+i] = bytes[i];
    }

    /** convert (byte) value into human readable value (string) */
    public void decode() {
	int old = value;
	String vals="";

	if (type == TYPE_INT) {
	    stringValue = vals;
	} else if (type > TYPE_INT) {
	    stringValue = ""+value;
	    if (valueList.indexOf(stringValue)<0) {
		value = getDefaultValue();
		stringValue = typeList[0];
		OutOfBoundsException e = new OutOfBoundsException(old, this);
		log(e.getMessage());
	    }
	}
	else {
	    try { 
		switch (type) {
		case TYPE_DECI_01_50:
		    vals= ""+(value/10)+"."+(value%10);
		    break;
		case TYPE_DECI_01_20:
		    vals= ""+(value/10)+"."+(value%10);
		    break;
		case TYPE_N100_OFF_0:
		    vals= typeList[value+100];
		    break;
		case TYPE_DEGREES:
		    vals= typeList[value+12];
		    break;
		case TYPE_RELEASE:
		    vals= typeList[value-3];
		    break;
		case TYPE_EQFREQ:
		    vals= typeList[value-25];
		    break;
		case TYPE_EQWIDTH:
		    vals= typeList[value-3];
		    break;
		case TYPE_CPRESET:
		    vals = typeList[value-1];
		    break;
		case TYPE_MAPPARAM:
		    vals = typeList[value+1];
		    break;
		default: vals = typeList[value];
		}
		stringValue = vals;

	    } catch (ArrayIndexOutOfBoundsException err) {
		value = getDefaultValue();
		stringValue = typeList[0];
		OutOfBoundsException e = new OutOfBoundsException(old, this);
		log(e.getMessage());
	    }
	}
    }

    protected void setValueList( List<String> l) {
	this.valueList = l;
    }

    public String getName() {
	return name;
    }

    int getMapMidiValue(int i) {
	int ret = 0;
	switch (i) {
	case 0:
	    ret = getMapMidiValue1();
	    break;
	case 1:
	    ret = getMapMidiValue2();
	    break;
	case 2:
	    ret = getMapMidiValue3();
	    break;
	default: 
	    ret = 0;
	}
	return ret;
    }

    int getMapMidiValue1() {
	int v1 = bytes[2] & 0xff;
	int v2 = bytes[3] & 0xff;
	int v = (v1 /4) + 32*( v2 % 8);
	return v;
    }

    int getMapMidiValue2() {
	int v1 = bytes[1] & 0xff;
	int v2 = bytes[2] & 0xff;
	int v = (v1 /2) + 64*( v2 % 4);
	return v;
    }

    int getMapMidiValue3() {
	int v1 = bytes[0] & 0xff;
	int v2 = bytes[1] & 0xff;
	int v = v1 + 128*( v2 % 2);	
	return v;
    }

    void updateNibble(String n, int t) {
	this.name = n;

	setType(t);
    }

    void setType(int t) {
	this.type = t;

	switch (type) {
	case Nibble.TYPE_INT:
	    typeList = null;
	    break;
	case Nibble.TYPE_ONOFF:
	    typeList= Constants.ONOFF;
	    break;
	case Nibble.TYPE_OFFON:
	    typeList= Constants.OFFON;
	    break;
	case Nibble.TYPE_ROUTING: 
	    typeList= Constants.ROUTING;
	    break;
	case Nibble.TYPE_MAPPARAM: 
	    Patch p = (Patch) parent;
	    setValueList(p.getExpressionPedalValueList());
	    typeList= p.exp_pedal_valueList;
	    break;

	case Nibble.TYPE_ATTACK:
	    typeList= Constants.ATTACK;
	    break;
	case Nibble.TYPE_RATIO:
	    typeList= Constants.RATIO;
	    break;
	case Nibble.TYPE_RELEASE:
	    typeList= Constants.RELEASE;
	    break;
	case Nibble.TYPE_EQFREQ:
	    typeList= Constants.EQFREQ;
	    break;
	case Nibble.TYPE_EQWIDTH:
	    typeList= Constants.EQWIDTH;
	    break;


	case Nibble.TYPE_SPEED:
	    typeList= Constants.SPEED;
	    break;
	case Nibble.TYPE_TEMPO:
	    typeList= Constants.TEMPO;
	    break;
	case Nibble.TYPE_HICUT:
	    typeList= Constants.HICUT;
	    break;
	case Nibble.TYPE_LOCUT:
	    typeList= Constants.LOCUT;
	    break;
	case Nibble.TYPE_HILO:
	    typeList= Constants.HILO;
	    break;
	case Nibble.TYPE_LOHI:
	    typeList= Constants.LOHI;
	    break;
	case Nibble.TYPE_SOFTHARD:
	    typeList= Constants.SOFTHARD;
	    break;
	case Nibble.TYPE_HARDSOFT:
	    typeList= Constants.HARDSOFT;
	    break;
	case Nibble.TYPE_HICOL:
	    typeList= Constants.HICOL;
	    break;
	case Nibble.TYPE_LOCOL:
	    typeList= Constants.LOCOL;
	    break;
	case Nibble.TYPE_SHAPE:
	    typeList= Constants.SHAPE;
	    break;
	case Nibble.TYPE_SIZE:
	    typeList= Constants.SIZE;
	    break;

	case Nibble.TYPE_KEY:
	    typeList= Constants.KEY;
	    break;
	case Nibble.TYPE_SCALE:
	    typeList= Constants.SCALE;
	    break;
	case Nibble.TYPE_DEGREES:
	    typeList= Constants.DEGREES;
	    break;
	case Nibble.TYPE_UPDOWN:
	    typeList= Constants.UPDOWN;
	    break;
	case Nibble.TYPE_DOWNUP:
	    typeList= Constants.DOWNUP;
	    break;

	case Nibble.TYPE_COMP:
	    typeList= Constants.COMP;
	    break;
	case Nibble.TYPE_DRIVE:
	    typeList= Constants.DRIVE;
	    break;
	case Nibble.TYPE_DELAY:
	    typeList=Constants.DELAY;
	    break;
	case Nibble.TYPE_REVERB:
	    typeList=Constants.REVERB;
	    break;
	case Nibble.TYPE_MOD:
	    typeList=Constants.MOD;
	    break;
	case Nibble.TYPE_PITCH:
	    typeList=Constants.PITCH;
	    break;
	case Nibble.TYPE_GATE:
	    typeList=Constants.SOFTHARD;
	    break;
	case Nibble.TYPE_PEDAL:
	    typeList = Constants.PEDAL;
	    break;
	case Nibble.TYPE_PEDALMASTER:
	    typeList = Constants.PEDALMASTER;
	    break;
	case Nibble.TYPE_TAPMASTER:
	    typeList = Constants.TAPMASTER;
	    break;
	case Nibble.TYPE_FOOTSWITCH:
	    typeList = Constants.FOOTSWITCH;
	    break;
	case Nibble.TYPE_INPUTSRC:
	    typeList = Constants.INPUTSRC;
	    break;
	case Nibble.TYPE_DCLOCK:
	    typeList = Constants.DCLOCK;
	    break;
	case Nibble.TYPE_DITHER:
	    typeList = Constants.DITHER;
	    break;
	case Nibble.TYPE_OUTPUTRANGE:
	    typeList = Constants.OUTPUTRANGE;
	    break;
	case Nibble.TYPE_VOLUMEPOS:
	    typeList = Constants.VOLUMEPOS;
	    break;
	case Nibble.TYPE_TUNEROUT:
	    typeList = Constants.TUNEROUT;
	    break;
	case Nibble.TYPE_TUNERMODE:
	    typeList = Constants.TUNERMODE;
	    break;
	case Nibble.TYPE_TUNERRANGE:
	    typeList = Constants.TUNERRANGE;
	    break;
	case Nibble.TYPE_IMPEDANCE:
	    typeList = Constants.IMPEDANCE;
	    break;
	case Nibble.TYPE_MIDI_CC:
	    typeList = Constants.MIDI_CC;
	    break;
	case Nibble.TYPE_MIDI_CHANNEL:
	    typeList = Constants.MIDI_CHANNEL;
	    break;
	case Nibble.TYPE_MIDI_SYSEX:
	    typeList = Constants.MIDI_SYSEX;
	    break;
	case Nibble.TYPE_PRESET:
	    typeList = Constants.PRESET;
	    break;
	case Nibble.TYPE_CPRESET:
	    typeList = Constants.CPRESET;
	    break;

	case TYPE_N200_200:
	    typeList = Constants.TYPE_N200_200;
	    break;
	case TYPE_N100_0:
	    typeList = Constants.TYPE_N100_0;
	    break;
	case TYPE_N100_OFF_0:
	    typeList = Constants.TYPE_N100_OFF_0;
	    break;
	case TYPE_N100_100:
	    typeList = Constants.TYPE_N100_100;
	    break;
	case TYPE_N99_15:
	    typeList = Constants.TYPE_N99_15;
	    break;
	case TYPE_N99_12:
	    typeList = Constants.TYPE_N99_12;
	    break;
	case TYPE_N99_0:
	    typeList = Constants.TYPE_N99_0;
	    break;
	case TYPE_N60_0:
	    typeList = Constants.TYPE_N60_0;
	    break;
	case TYPE_N50_0:
	    typeList = Constants.TYPE_N50_0;
	    break;
	case TYPE_N50_50:
	    typeList = Constants.TYPE_N50_50;
	    break;
	case TYPE_N40_0:
	    typeList = Constants.TYPE_N40_0;
	    break;
	case TYPE_N30_0:
	    typeList = Constants.TYPE_N30_0;
	    break;
	case TYPE_N25_25:
	    typeList = Constants.TYPE_N25_25;
	    break;
	case TYPE_N12_12:
	    typeList = Constants.TYPE_N12_12;
	    break;
	case TYPE_0_10:
	    typeList = Constants.TYPE_0_10;
	    break;
	case TYPE_0_24:
	    typeList = Constants.TYPE_0_24;
	    break;
	case TYPE_0_30:
	    typeList = Constants.TYPE_0_30;
	    break;
	case TYPE_DECI_01_50:
	    typeList = Constants.TYPE_DECI_01_50;
	    break;
	case TYPE_0_90:
	    typeList = Constants.TYPE_0_90;
	    break;
	case TYPE_0_100:
	    typeList = Constants.TYPE_0_100;
	    break;
	case TYPE_0_120:
	    typeList = Constants.TYPE_0_120;
	    break;
	case TYPE_0_200:
	    typeList = Constants.TYPE_0_200;
	    break;
	case TYPE_0_1800:
	    typeList = Constants.TYPE_0_1800;
	    break;
	case TYPE_1_2:
	    typeList = Constants.TYPE_1_2;
	    break;
	case TYPE_1_10:
	    typeList = Constants.TYPE_1_10;
	    break;
	case TYPE_1_20:
	    typeList = Constants.TYPE_1_20;
	    break;
	case TYPE_100_3000:
	    typeList = Constants.TYPE_100_3000;
	    break;
	case TYPE_DECI_01_20:
	    typeList = Constants.TYPE_DECI_01_20;
	    break;
	case TYPE_0_18:
	    typeList = Constants.TYPE_0_18;
	    break;
	case TYPE_3_200:
	    typeList = Constants.TYPE_3_200;
	    break;
	case TYPE_N2400_2400:
	    typeList = Constants.TYPE_N2400_2400;
	    break;
	case TYPE_0_350:
	    typeList = Constants.TYPE_0_350;
	    break;
	case TYPE_0_50:
	    typeList = Constants.TYPE_0_50;
	    break;
	case TYPE_N100_6:
	    typeList = Constants.TYPE_N100_6;
	    break;
	case TYPE_420_460:
	    typeList = Constants.TYPE_420_460;
	    break;
	case TYPE_N6_18:
	    typeList = Constants.TYPE_N6_18;
	    break;
	default: typeList = null;
	}

	if ((typeList != null) && (type != TYPE_MAPPARAM))
	    valueList = Arrays.asList(typeList);

	if (type != TYPE_MIDI_MAP) decode();
    }

    public void setEditable(boolean state) {
	editable = state;
    }

    public boolean isEditable() {
	return editable;
    }

    public static void main( String[] args) {
	try {
	    int v = Integer.parseInt( args[0]);
	    Nibble n = new Nibble(null, v );
	    System.out.println( n.toString() );
	    byte[] b = n.getBytes();
	    Nibble n2 = new Nibble(null, b );
	    System.out.println( n2.toString() );
	    System.out.println( n2.toAscii());
	    System.out.println( n2.toDecimal());
	    System.out.println( n2.toHexa());
	    

	} catch (Exception e) { 
	    e.printStackTrace();
	}

	
    }
	
    static int toBPM(int ms) {
	// 60 000ms = 1 bpm
	// 1 000ms = 60bpm
	// 1 ms = 60 000 bpm
	// 100 < ms < 3000
	// 600 > bpm > 1
	return (60000 / ms);
    }

    static int toMillis(int bpm) {
	// 1 bpm = 60 000 ms
	// 60 bpm = 1 000ms
	return (60000 / bpm);
    }
    
    protected void log(String s) {	
	if (parent != null) parent.log(s);
    }

    public void resetValue() {
	this.value = getDefaultValue();
    }

    protected int getDefaultValue() {
	int def ;
	switch (type) {
	    //there may be an offset to add
	case TYPE_MAPPARAM:
	    def = -1;
	    break;
	case TYPE_CPRESET:
	    def = 1;
	    break;
	case TYPE_RELEASE:
	    def = 3;
	    break;
	case TYPE_EQFREQ:
	    def = 25;
	    break;
	case TYPE_EQWIDTH:
	    def = 3;
	    break;
	case TYPE_DEGREES:
	    def = -12;
	    break;
	case TYPE_N200_200:
	    def = -200;
	    break;
	case TYPE_N100_0:
	    def = -100;
	    break;
	case TYPE_N100_OFF_0:
	    def = -100;
	    break;
	case TYPE_N100_100:
	    def = -100;
	    break;
	case TYPE_N99_15:
	    def = -99;
	    break;
	case TYPE_N99_12:
	    def = -99;
	    break;
	case TYPE_N99_0:
	    def = -99;
	    break;
	case TYPE_N60_0:
	    def = -60;
	    break;
	case TYPE_N50_0:
	    def = -50;
	    break;
	case TYPE_N50_50:
	    def = -50;
	    break;
	case TYPE_N40_0:
	    def = -40;
	    break;
	case TYPE_N30_0:
	    def = -30;
	    break;
	case TYPE_N6_18:
	    def = -6;
	    break;
	case TYPE_N25_25:
	    def = -25;
	    break;
	case TYPE_N12_12:
	    def = -12;
	    break;
	case TYPE_1_2:
	    def = 1;
	    break;
	case TYPE_1_10:
	    def = 1;
	    break;
	case TYPE_1_20:
	    def = 1;
	    break;
	case TYPE_100_3000:
	    def = 100;
	    break;
	case TYPE_3_200:
	    def = 3;
	    break;
	case TYPE_N2400_2400:
	    def = -2400;
	    break;
	case TYPE_N100_6:
	    def = -100;
	    break;
	case TYPE_420_460:
	    def = 420;
	    break;
	default:
	    def = 0;
	}
	return def;
    }
}