package nova;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.List;
import java.io.*;

/* Main Application Window */
public class NovaManager extends JFrame implements ActionListener, ChangeListener {
        
    private final static String version="1.20.2";

    private boolean editable;
    boolean standalone = true; //initial state: not connected (true)

    JMenu getMenu = new JMenu("Get");
    JMenu sendMenu = new JMenu("Send");

    NovaLog log = new NovaLog();
    ErrorDialog ed;
    GetPatchDialog gpd;
    GetSystemDumpDialog gsdd;
    GetVariationDialog gvd;
    JFileChooser fc;
    EditName en;

    SendCC scc;
    SendPC spc;

    JTabbedPane tab;

    List novas; // List of accessible Novas
    MidiInterface mi;

    PageFormat mPageFormat;

    private Window popup;

    public NovaManager() {
	super("NovaManager");
	this.setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
	this.setResizable(false);
	logSilent("NovaManager Version = "+version);

	// Choose the right behavior
	setEditable(true); // firmware > 1.13

	mPageFormat = new PageFormat();
	mPageFormat.setOrientation( PageFormat.LANDSCAPE );

	ed = new ErrorDialog(this);
	gpd = new GetPatchDialog(this);
	gsdd = new GetSystemDumpDialog(this);
	gvd = new GetVariationDialog(this);
	fc = new JFileChooser(System.getProperty("user.dir"));
	en = new EditName(this);

	spc = new SendPC(this);
	scc = new SendCC(this);

	tab = new JTabbedPane();
	tab.setTabPlacement(JTabbedPane.TOP);
	tab.setTabLayoutPolicy(JTabbedPane.WRAP_TAB_LAYOUT);
	tab.addChangeListener(this);
	tab.setOpaque(true);

	JMenuBar myMenuBar = new JMenuBar();
        JMenu myMenu = new JMenu("Open");
	myMenu.setEnabled(true);

        JMenuItem menuItem = new JMenuItem("Preset");
        menuItem.setActionCommand("openpreset");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);

	menuItem = new JMenuItem("Bank");
        menuItem.setActionCommand("openbank");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);

        menuItem = new JMenuItem("Variation");
        menuItem.setActionCommand("openvar");
        menuItem.addActionListener(this);
	myMenu.add(menuItem);

	menuItem = new JMenuItem("System Dump");
        menuItem.setActionCommand("opensystem");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);
	myMenuBar.add(myMenu);



	getMenu = new JMenu("Get");
	getMenu.setEnabled(true);

        menuItem = new JMenuItem("Preset");
        menuItem.setActionCommand("getpreset");
        menuItem.addActionListener(this);
 	menuItem.setEnabled( false );
	getMenu.add(menuItem);

        menuItem = new JMenuItem("Variation");
        menuItem.setActionCommand("getvar");
        menuItem.addActionListener(this);
	menuItem.setEnabled( false );
	getMenu.add(menuItem);

	menuItem = new JMenuItem("System Dump");
        menuItem.setActionCommand("getsystem");
        menuItem.addActionListener(this);
	menuItem.setEnabled( false );
        getMenu.add(menuItem);
	myMenuBar.add( getMenu );


	sendMenu = new JMenu("Send");
	sendMenu.setEnabled(true);

	menuItem = new JMenuItem("Program Change");
	menuItem.setActionCommand("PC");
	menuItem.addActionListener(this);
	menuItem.setEnabled(true);
	sendMenu.add(menuItem);

	menuItem = new JMenuItem("Control Change");
	menuItem.setActionCommand("CC");
	menuItem.addActionListener(this);
	menuItem.setEnabled(true);
	sendMenu.add(menuItem);
  	myMenuBar.add(sendMenu);
     


	myMenu = new JMenu("Midi");
	myMenu.setEnabled(true);

	menuItem = new JMenuItem("Settings");
        menuItem.setActionCommand("editMidi");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);
	myMenuBar.add( myMenu );

	menuItem = new JMenuItem("Console");
        menuItem.setActionCommand("console");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);
	myMenuBar.add( myMenu );


	myMenu = new JMenu("Print");
	myMenu.setEnabled(true);

	menuItem = new JMenuItem("Print Active Tab");
        menuItem.setActionCommand("print");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);
	menuItem = new JMenuItem("Print all Tabs");
        menuItem.setActionCommand("printAll");
        menuItem.addActionListener(this);
	menuItem.setEnabled( true );
        myMenu.add(menuItem);
	myMenuBar.add( myMenu );

	this.setJMenuBar(myMenuBar);
	setSize(new Dimension(400,300));
	this.getContentPane().add(tab);

	setStandalone( standalone );
	init();
	this.setVisible(true);
    }

    void init() {
	mi = new MidiDefaultInterface2(this);
	Thread a = new Thread(mi);
	a.start();
    }

    public void actionPerformed(ActionEvent e) {
        String action = e.getActionCommand();
	if (action.equals("openpreset")) openPatch(); 
	else if (action.equals("openbank")) openBank(); 
        else if (action.equals("openvar")) openVar();
        else if (action.equals("opensystem")) openSystem();
        else if (action.equals("getpreset")) getPatch();
        else if (action.equals("getvar")) getVar();
        else if (action.equals("getsystem")) getSystem();
        else if (action.equals("editMidi")) mi.setVisible( true );
        else if (action.equals("console")) log.setVisible( true );
        else if (action.equals("print")) print();
        else if (action.equals("printAll")) printAll();
        else if (action.equals("PC")) programChange();
        else if (action.equals("CC")) controlChange();
    }

    private void programChange() {
	spc.open();
    }

    private void controlChange() {
	scc.open();
    }


    /* create a Patch from scratch and open it */
    void newPatch() {
	System.out.println("New Patch");
    }

    /* open a Patch from a file */
    void openPatch() {
	//System.out.println("Open a Patch");
	int returnVal = fc.showOpenDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		byte[] data = getBytesFromFile(f);
		if (Patch.check(data)) 
		    this.add( new Patch(this, data) );
		else
		    ed.setMessage("<html><center><b>Preset Format Error</b><br><br>"+f.getName()+"<br>Incorrect SysEx file</center></html>");
		//setSize( getPreferredSize());
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
	
    }

        /* open a set of Patches (Bank) from a file */
    void openBank() {
	//System.out.println("Open a Patch");
	int returnVal = fc.showOpenDialog(this);
	int len = Patch.PATCH_SIZE;

	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		byte[] data = getBytesFromFile(f);
		for (int i=0; i< data.length/len; i++) {
		    byte[] temp = new byte[len];
		    System.arraycopy(data, i*Patch.PATCH_SIZE, temp, 0, len);
		    if (Patch.check(temp)) 
			this.add( new Patch(this, temp) );
		    else
			ed.setMessage("<html><center><b>Preset Format Error</b><br><br>"+f.getName()+"<br>Incorrect SysEx file</center></html>");
		}
		//setSize( getPreferredSize());
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
	
    }

    /* get a Patch from Nova : need connection */
    void getPatch() {
	gpd.open();
    }

    /* create Syst config from scratch and open it */
    void newSystem() {
 	System.out.println("New System Settings");
   }

    /* open Syst config from a file */
    void openSystem() {
	//System.out.println("Open a System Dump");
	int returnVal = fc.showOpenDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		byte[] data = getBytesFromFile(f);
		// TODO check whether Data is a SystemDump
		if (SystemDump.check(data)) {
		    SystemDump ns = new SystemDump( this, f.getName(), data );
		    this.add( ns );
		} else { 
		    ed.setMessage("<html><center><b>SystemDump Format Error</b><br><br>"+f.getName()+"<br>Incorrect SysEx file</center></html>");
		}
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
	
    }

    /* get Syst config from Nova */
    void getSystem() {
 	//System.out.println("Get System Dump");
	//gsdd.open();
	Integer id = getCurrentSysExID();
	getSystemDump(id.byteValue());
   }

    /* create Syst config from scratch and open it */
    void newVar() {
 	System.out.println("New Variation");
   }

    /* open a Variation (favorite block setting) from a file */
    void openVar() {
	//System.out.println("Open a Variation");
	int returnVal = fc.showOpenDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		byte[] data = getBytesFromFile(f);
		// TODO check whether Data is a Variation (same format as Preset)
		if (Patch.check(data)) {
		    Variation var = new Variation( this, data );
		    this.add( var );
		} else { 
		    ed.setMessage("<html><center><b>Variation Format Error</b><br><br>"+f.getName()+"<br>Incorrect SysEx file</center></html>");
		}
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
	
    }

    /* get Variations from Nova */
    void getVar() {
 	//System.out.println("Get Variation Dump");
	gvd.open();
   }

    /* refreshes accessible menus */
    void setStandalone(boolean state) {
	standalone = state;

	for (int i=0; i<getMenu.getItemCount(); i++) {
	    JMenuItem mi = getMenu.getItem(i);
	    mi.setEnabled( !standalone );
	}
	//getMenu.validate();

	//sendMenu.setEnabled( true ); // You can send PC/CC to any device (not Nova nor SysEx dependant)
	for (int i=0; i< tab.getTabCount(); i++) {
	    Component c = tab.getComponentAt(i);
	    if (c instanceof Patch) {
		Patch p = (Patch) c;
		//p.refreshNovasList();
		p.setStandalone(state);
	    } else if (c instanceof SystemDump) {
		SystemDump p = (SystemDump) c;
		//p.refreshNovasList();
		p.setStandalone(state);
	    }
	}
    }

    /* Read bytes from a file */
    static byte[] getBytesFromFile(File file) throws IOException {
	InputStream is = new FileInputStream(file);
	long length = file.length();
	
	if (length < 520 ) {
	    // File is too short
	    System.err.println("Input File is too short");
	}
	
	// Create the byte array to hold the data
	byte[] bytes = new byte[(int)length];
	
	// Read in the bytes
	int offset = 0;
	int numRead = 0;
	while (offset < bytes.length
	       && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
	    offset += numRead;
	}
	
	// Ensure all the bytes have been read in
	if (offset < bytes.length) {
	    throw new IOException("Could not completely read file "+file.getName());
	}
	
	// Close the input stream and return bytes
	is.close();
	//System.out.println(""+ bytes.length + " bytes read from " + file.getName());

	return bytes;
    }

    /* print the active tab */
    private void print() {
	PrintablePanel pp = (PrintablePanel) tab.getSelectedComponent();
	if ( pp != null ) {
	    pp.print();
	}
    }

    /* print all tabs */
    private void printAll() {
	// Gather all tab prints in a single PrintableBook
	PrintableBook b = new PrintableBook();
	for (int i=0; i< tab.getTabCount(); i++) {
	    PrintablePanel pp = (PrintablePanel) tab.getComponentAt(i);	   
	    b.addTab( pp );
	}
	
	PrintRequestAttributeSet prs =  new HashPrintRequestAttributeSet();
	prs.add(OrientationRequested.LANDSCAPE);

	PrinterJob printJob = PrinterJob.getPrinterJob();
	printJob.setJobName( "NovaManager" );
	printJob.setPrintable( b );

	if (printJob.printDialog(prs)) { 
	    try{
		printJob.print(prs) ;
	    }catch (PrinterException exception){
		JOptionPane.showMessageDialog(this, exception);
	    }
	}
	
    }

    /* === ACCESSIBLE METHODS === */

    void removeTab(JPanel p) {
	tab.remove(p);
	tab.setSelectedIndex( tab.getTabCount()-1 );

    }

    void add( Patch p) {
	tab.add(p);
	tab.setSelectedComponent( p );
	p.setStandalone( standalone );
	pack();
    }
	
    void add( SystemDump p) {
	tab.add(p);
	tab.setSelectedComponent( p );
	p.setStandalone( standalone );
 	pack();
   }

    void add( SystemDump2 p) {
	tab.add(p);
	tab.setSelectedComponent( p );
	p.setStandalone( standalone );
 	pack();
   }
	
    JFileChooser getFileChooser() {
	return fc;
    }

    void send(byte[] data) {
	mi.send(data);
    }

    void getPatch(byte id, byte num) {
	mi.sendPatchRequest(id, num);
    }

    void getBank(byte id, int first, int last) {
	final int f = first;
	final int l = last;
	final byte sid = id;
	final NovaManager papa = this;
	final ErrorDialog em = new ErrorDialog(papa);

	Thread t = new Thread() {
		public void run() {
		    for (int i=f; i<=l; i++) {
			getPatch(sid, (byte) (i));
			try { 
			    sleep(500);
			} catch (InterruptedException e) {}
		    }
		    em.setVisible(false);
		}
	    };

	t.start();
	em.setMessage("Download in progress. Please wait");
	em.setSize(new Dimension(300, 200));
	   
    }

   void getSystemDump(byte id) {
       mi.sendSystemDumpRequest(id);
    }

    boolean getStandalone() {
	return standalone;
    }

    /*
    List getNovas() {
	return novas;
    }
    */

    void setEditable(boolean state) {
	editable = state;
    }

    boolean isEditable() {
	return editable;
    }

    /*
    void setNovas(List list) {
	novas = list;
	if ( (novas != null) && (novas.size() >0)) setStandalone( false );
	else setStandalone( true );
    }
    */

    Integer getCurrentSysExID() {
	return mi.getCurrentSysExID();
    }

    void setPopup(Window w) {
	if (popup != null) popup.setVisible(false);
	popup = w;
	if (editable) popup.setVisible(true);
    }

    void refreshTabName(String s) {
	int i = tab.getSelectedIndex();
	if (i>=0) tab.setTitleAt(i, s);
    }
    
    public void stateChanged(ChangeEvent e) {
	if (popup != null) popup.setVisible(false);
	popup = null;
    }

    protected void logSilent(String s) {
	log.appendSilent("\n"+s);
    }

    protected void logSilent() {
	logSilent("");
    }

    protected void log(String s) {
	//log.append("\n"+s);
	log.appendSilent("\n"+s);	
    }

    protected void log() {
	log("");
    }

    public static void main(String[] args) {
	NovaManager nm = new NovaManager();
    }
}
			

