package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// Get the first sysex message
// Change the Preset Name
// Patch Name and Number can be changed without the checksum to be recomputed

public class Patch extends PrintablePanel implements ActionListener, MouseListener {
    protected static final int PATCH_SIZE = 520 ;
    protected static final int MESSAGE_ID_OFFSET = 6;

    protected static final int PRESET_NUMBER_OFFSET = 8;
    protected static final int PRESET_NAME_OFFSET = 10;
    protected static final int PRESET_NAME_LENGTH = 24;

    protected static final int CHECKSUM_OFFSET = 518 ;
    protected static final int CHECKSUM_START = PRESET_NAME_OFFSET + PRESET_NAME_LENGTH ;
    protected static final int CHECKSUM_END = CHECKSUM_OFFSET - 1 ;

    protected String[] exp_pedal_valueList; //List of values for Expression Pedal (dynamic)

    protected byte[] data ;
    private String preset_name ;
    private int message_id;
    private int checksum;
    private int bankN;
    private int presN;
    private int numcode;
    private File saveFile;
    private boolean isVariation = false;

    protected static final int NIBBLE_LENGTH = 7*16+9;
    protected Vector<Nibble> nibbles;
    protected Vector<Block> blocks;

    JRootPane pane ;
    JPanel titlePane;
    Global global;

    JButton closeButton = new JButton("Close");
    JButton saveButton = new JButton("Save SysEx");
    JButton saveXML = new JButton("Export XML");
    JButton saveHTML = new JButton("Export HTML");
    JButton send = new JButton("Send to Nova");

    NovaManager parent;

    public Patch( NovaManager nm, byte[] sysex, boolean variation) {	
	super( );
	this.parent = nm;
	setLog(nm.log);
	this.isVariation = true;
	
	data = new byte[ PATCH_SIZE ];
	int i;
	for (i=0; i < PATCH_SIZE ; i++) {
	    data[i] = sysex[i];
	}

	nibbles = new Vector<Nibble>();
	try {
	    for (i=0; i< NIBBLE_LENGTH; i++) {
		byte[] temp = new byte[4];
		for (int j=0; j<4; j++) temp[j] = data[ CHECKSUM_START + 4*i + j];
		nibbles.add(new Nibble(this, temp, CHECKSUM_START + 4*i ));
	    }
	} catch (NibbleException ne) {
	    parent.log(ne.getMessage());
	}
	titlePane = new JPanel();
	titlePane.setLayout( new BorderLayout());
	titlePane.setOpaque(false);
    }
	
    public Patch( NovaManager nm, byte[] sysex ) {	
	super( );
	this.parent = nm;
	setLog(nm.log);
	
	data = new byte[ PATCH_SIZE ];
	int i;
	for (i=0; i < PATCH_SIZE ; i++) {
	    data[i] = sysex[i];
	}

	nibbles = new Vector<Nibble>();
	try {
	    for (i=0; i< NIBBLE_LENGTH; i++) {
		byte[] temp = new byte[4];
		for (int j=0; j<4; j++) temp[j] = data[ CHECKSUM_START + 4*i + j];
		nibbles.add(new Nibble(this, temp, CHECKSUM_START + 4*i ));
	    }
	} catch (NibbleException ne) {
	    parent.log(ne.getMessage());
	}


	//titlePane is declared here but composed by Block 'global'
	titlePane = new JPanel();
	titlePane.setLayout( new BorderLayout());
	titlePane.setOpaque(false);
	refresh();

	setName( preset_name );
	blocks = new Vector<Block>();

	// build blocks from data
	global = new Global(this);
	Drive drive = new Drive(this);
	Comp comp = new Comp(this);
	Gate gate = new Gate(this);
	Boost boost = new Boost(this);
	
	EQ eq = new EQ(this);
	Modulation mod = new Modulation(this);
	Pitch pitch = new Pitch(this);
	Delay delay = new Delay(this);
	Reverb reverb = new Reverb(this);

	blocks.add(global);
	blocks.add(drive);
	blocks.add(comp);
	blocks.add(gate);
	blocks.add(boost);
	
	blocks.add(eq);
	blocks.add(mod);
	blocks.add(pitch);
	blocks.add(delay);
	blocks.add(reverb);
	
	// Buttons
	closeButton.addActionListener( this );
	closeButton.setActionCommand( "close" );

	saveButton.addActionListener( this );
	saveButton.setActionCommand( "save" );

	saveXML.addActionListener( this );
	saveXML.setActionCommand( "saveXML" );

	saveHTML.addActionListener( this );
	saveHTML.setActionCommand( "saveHTML" );

	send.addActionListener( this );
	send.setActionCommand( "send" );
	send.setEnabled( !parent.getStandalone() );

	// Layout
	Box h1 = Box.createHorizontalBox();
	h1.add( eq );
	h1.add( mod );
	h1.add( pitch );
	h1.add( delay );
	h1.add( reverb);
	
	Box h2 = Box.createHorizontalBox();
	h2.add( global);
	h2.add( drive);
	h2.add( comp);
	h2.add( gate);
	h2.add( boost);
	
	Box p = Box.createVerticalBox();
	p.add(h2);
	p.add( Box.createVerticalStrut(10));
	p.add(h1);

	JPanel boutons = new JPanel();
	boutons.setOpaque(false);
	boutons.add( closeButton );
	boutons.add( saveButton );
	boutons.add( saveXML );
	boutons.add( saveHTML );
	boutons.add( send );

	setLayout( new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = 0;
	c.gridy = 0;
	c.weightx =1.0;
	c.weighty = 0.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.PAGE_START;
	add(titlePane,c);
	c.weighty = 0;
	c.gridy++;
	c.fill = GridBagConstraints.NONE;
	add(p,c);
	c.gridy++;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.PAGE_END;
	add(boutons,c);
	setOpaque(true);
	setBackground( Constants.grisvert );

    }

    boolean isVariation() {
	return isVariation;
    }

    /** Calculate settings from sysex
	Refresh widgets
    */
    void refresh() {
	checksum = data[ CHECKSUM_OFFSET ];
	message_id = data[ MESSAGE_ID_OFFSET ];
	numcode = data[ PRESET_NUMBER_OFFSET ];
	bankN = (numcode - 31) / 3;
	presN = (numcode - 31 ) % 3 +1;
	preset_name ="";

	
	for (int i=0; (i<PRESET_NAME_LENGTH) && (data[ PRESET_NAME_OFFSET + i]!=0x00) ; i++) 
	    preset_name += (char) data[ PRESET_NAME_OFFSET +i];

    }

    void setExpressionPedalValueList() {
	int mod = getNibble(41).getValue();
	int dly = getNibble(57).getValue();
	int pit = getNibble(105).getValue();
	exp_pedal_valueList = new String[20];
	exp_pedal_valueList[0] = "Off";
	for (int i=0; i<2; i++) exp_pedal_valueList[1+i] = Constants.DRV_EXP[ i ];
	for (int i=0; i<4; i++) exp_pedal_valueList[3+i] = Constants.MOD_EXP[ mod*4 + i];
	for (int i=0; i<6; i++) exp_pedal_valueList[7+i] = Constants.DLY_EXP[ dly*6 + i];
	for (int i=0; i<4; i++) exp_pedal_valueList[13+i] = Constants.REV_EXP[ i ];
	for (int i=0; i<3; i++) exp_pedal_valueList[17+i] = Constants.PIT_EXP[ pit*3 + i ];
    }

    Vector<String> getExpressionPedalValueList() {
	setExpressionPedalValueList();
	
	Vector<String> v = new Vector<String>();
	for (int i=0; i<20; i++) 
	    if (exp_pedal_valueList[i] != "") v.add( exp_pedal_valueList[i]);
	return v;
    }

    String getOriginalName() {
	return preset_name;
    }

    void setPresetCode(int i) {
	numcode = i;
    }

    void setBankNumber(int i) {
	bankN = i;
    }

    void setPresetNumber(int i) {
	presN = i;
    }

    void setPresetName(String s) {
	preset_name = s;
    }

    int getPresetCode() {
	return numcode;
    }

    int getOriginalBankNumber() {
	return bankN;
    }

    int getOriginalPresetNumber() {
	return presN;
    }

    JPanel getTitlePane() {
	return titlePane;
    }

    void refreshTitle() {
	global.refreshTitle();
	parent.refreshTabName( preset_name );
    }

    void saveValues() {
	data[ PRESET_NUMBER_OFFSET ] = (byte) ( numcode );

	for (int i=0;i< PRESET_NAME_LENGTH; i++)
	    if ( i < preset_name.length() ) 
		data[PRESET_NAME_OFFSET+i] = (byte) preset_name.charAt(i);
	    else data[PRESET_NAME_OFFSET+i]= 0x00;

	// Save nibbles
	for (int i=0; i<nibbles.size(); i++) getNibble(i).save(data);

	data[ CHECKSUM_OFFSET ] = (byte)calculateChecksum();
    }

    int calculateChecksum() {
	return calculateChecksum(data);
    }
    static int calculateChecksum(byte[] d) {
	int sum = 0;
	for (int i=CHECKSUM_START; i<=CHECKSUM_END; i++) sum+=d[i];
	return sum % 128;
    }

    /* check whether an array of bytes conforms to patch format
       check length, preamble and checksum
    */
    public static boolean check(byte[] d) {
	boolean ok = true;
	if ( ok && (d.length != PATCH_SIZE)) ok = false;
	if ( ok && (d[0] != MidiInterface.aPreset[0])) ok = false;
	if ( ok && (d[1] != MidiInterface.aPreset[1])) ok = false;
	if ( ok && (d[2] != MidiInterface.aPreset[2])) ok = false;
	if ( ok && (d[3] != MidiInterface.aPreset[3])) ok = false;
	if ( ok && (d[5] != MidiInterface.aPreset[5])) ok = false;
	if ( ok && (d[6] != MidiInterface.aPreset[6])) ok = false;
	if ( ok && (d[7] != MidiInterface.aPreset[7])) ok = false;
	if ( ok && (d[PATCH_SIZE-1] != (byte) 0xF7)) ok = false;
	if ( ok && (d[CHECKSUM_OFFSET] != calculateChecksum(d))) ok = false;
	return ok;
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();

	if (e.getActionCommand().equals("close")) {
	    closeTab();
	} else if (e.getActionCommand().equals("save")) {
	    saveValues();
	    saveSysEx();
	} else if (e.getActionCommand().equals("saveXML")) {
	    saveValues();
	    saveXML();
	} else if (e.getActionCommand().equals("saveHTML")) {
	    saveValues();
	    saveHTML();	    
	} else if (e.getActionCommand().equals("send")) {	    
	    Integer id = parent.getCurrentSysExID();
	    if (id != null) data[4] = id.byteValue();
	    saveValues();
	    parent.send( data );
	}	    
    }
    
    public void writeBytesToFile(File file, byte[] bArray) {	
	try
	    {
		FileOutputStream fos = new FileOutputStream( file );
		
		byte[] sysex = new byte[11];
		sysex[0] = (byte) 0xF0; // Begin Sysex
		sysex[1] = (byte) 0x00; // TC
		sysex[2] = (byte) 0x20; // TC
		sysex[3] = (byte) 0x1F; // TC
		sysex[4] = (byte) 0x7F; // Device ID
		sysex[5] = (byte) 0x63; // Nova System
		sysex[6] = (byte) 0x45; // Message ID = Request
		sysex[7] = (byte) 0x01; // Data Type = Preset
		sysex[8] = (byte) 0x1F; // Preset Number
		sysex[9] = (byte) 0x00; // void
		sysex[10] = (byte) 0xF7; // End Sysex
		fos.write( sysex );
		
		fos.close(); 
		parent.logSilent(""+ sysex.length + " bytes written to " + file.getName());
	    }
	catch(FileNotFoundException ex)
	    {
		parent.log("FileNotFoundException : " + ex);
	    }
	catch(IOException ioe)
	    {
		parent.log("IOException : " + ioe);
	    }
	
    }
    
    public static byte[] getBytesFromFile(File file) throws IOException {
	InputStream is = new FileInputStream(file);
	long length = file.length();
	
	if (length < 520 ) {
	    // File is too large
	    System.err.println("Input File is too short");
	}
	
	// Create the byte array to hold the data
	byte[] bytes = new byte[(int)length];
	
	// Read in the bytes
	int offset = 0;
	int numRead = 0;
	while (offset < bytes.length
	       && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
	    offset += numRead;
	}
	
	// Ensure all the bytes have been read in
	if (offset < bytes.length) {
	    throw new IOException("Could not completely read file "+file.getName());
	}
	
	// Close the input stream and return bytes
	is.close();
	//System.out.println(""+ bytes.length + " bytes read from " + file.getName());

	return bytes;
    }

    Nibble getNibble(int i) {
	return nibbles.elementAt(i);
    }

    String toHTML() {
	String s="";
	s += "<HTML>\n";
	s += "  <BODY>\n";
	s += "    <H1>Preset #"+numcode+" ["+Constants.PRESET[numcode]+"] -- "+preset_name+"</H1>\n";
	s += "    <TABLE border=\"1\" cellpadding=\"10\" width=\"1000\">\n";
	s += "      <TR>\n";
	for (int i=0; i<5; i++) {
	    Block b = (Block) blocks.elementAt(i);
	    s+= b.toHTML();
	}
	s += "      </TR>\n";
	s += "      <TR>\n";
	for (int i=5; i<blocks.size(); i++) {
	    Block b = (Block) blocks.elementAt(i);
	    s+= b.toHTML();
	}
	s += "      </TR>\n";
	s += "    </TABLE>\n";
	s += "  </BODY>\n";
	s += "</HTML>\n";
	return s;
    }

    public String toXML() {
	String s="";

	s += "<patch vendorId=\"00201F\" vendor=\"TC Electronic\" ";
	s += "modelId=\"63\" model=\"Nova System\" ";
	s += " n=\"" +numcode ;
	s += "\" bank=\"" +bankN ;
	s += "\" preset=\"" +presN ;
	s += "\" name=\"" +preset_name;
	s += "\">\n";

	for (int n=0; n<blocks.size(); n++) {
	    Block b = (Block) blocks.elementAt(n);
	    s += b.toXML();
	}

	s += "</patch>";

	return s;
    }
    

    void closeTab() {
	parent.removeTab(this);
    }

    void saveSysEx() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileOutputStream fos = new FileOutputStream( f );
		fos.write( data );
		fos.close(); 	    
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }
    void saveXML() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileWriter outFile = new FileWriter(f);
		PrintWriter out = new PrintWriter(outFile);
		out.println( toXML() );
		out.close();
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }

    void saveHTML() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileWriter outFile = new FileWriter(f);
		PrintWriter out = new PrintWriter(outFile);
		out.println( toHTML() );
		out.close();
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }

    void setStandalone( boolean state) {
	send.setEnabled(!state);
    }

   // == MOUSE LISTENER INTERFACE //
    public void mouseClicked(MouseEvent e) {
	Component c = (Component) e.getSource();
	Point pos = c.getLocationOnScreen();
	int x = e.getX() + pos.x;
	int y = e.getY() + pos.y;
	if (x<0) x=0;
	if (y<0) y=0;

	parent.en.update(this,x,y); 
    }

    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }

    public static void main(String[] args) {
	// Arg 0 : name
	// Arg 1 : input file

	if (args.length != 1) {
	    System.out.println("Syntax : java nova.Patch <Sysex inputFile>");
	} else {
	    File inputFile = new File( args[0] );

	    try {
		final byte[] bArray = getBytesFromFile( inputFile );
		SwingUtilities.invokeLater( new Runnable() {
			public void run() {
			    NovaManager nm = new NovaManager();	
			    nm.setEditable(true);
			    nm.add( new Patch( nm, bArray ));    
			}
		    });
	    } catch (IOException ioe) {
		System.err.println("File not found : "+args[1]);
	    }
	    
	}
    }
}

