package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


/** The Pitch Block
 * Extension of Block class to manage Pitch parameters
 * in Presets and Variations
 */
public class Pitch extends CopyBlock {

    Nibble type;
    Nibble v1;
    Nibble v2;
    Nibble p1;
    Nibble p2;
    Nibble d1;
    Nibble d2;
    Nibble fb1;
    Nibble fb2;
    Nibble l1;
    Nibble l2;
    Nibble pitch;
    Nibble range;
    Nibble mix;
    Nibble onoff;

    Nibble direction;
    Nibble key;
    Nibble scale;


    public Pitch(Patch par) {	
	super("Pitch", par, true);

	activeNibbles = new int[] { 120, 105, 106, 107, 108, 109,
				    110, 111, 112, 113, 114, 115, 116, 117} ;

	try {
	    for (int i=0; i<activeNibbles.length; i++) 
		nibbles.add( parent.getNibble( activeNibbles[i]));

	    type = parent.getNibble(105);
	    v1 = parent.getNibble(106);
	    v2 = parent.getNibble(107);
	    p1 = parent.getNibble(108);
	    p2 = parent.getNibble(109);
	    d1 = parent.getNibble(110);
	    d2 = parent.getNibble(111);
	    fb1 = parent.getNibble(112);
	    fb2 = parent.getNibble(113);
	    l1 = parent.getNibble(114);
	    l2 = parent.getNibble(115);
	    range = parent.getNibble(116);
	    mix = parent.getNibble(117);
	    onoff = parent.getNibble(120);

	    type.updateNibble("Type", Nibble.TYPE_PITCH);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Pitch Block");
	}

	direction  = l2;
	key = fb1;
	scale = fb2; 
	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();
	for (int j=2; j<activeNibbles.length; j++) {
	    Nibble n = parent.getNibble( activeNibbles[j]);
	    n.setType(Nibble.TYPE_INT);
	}
	int i=0;

	box.add( new Row("Pitch", onoff, i++));
	box.add( new Row("Pitch", type, i++));

	switch (type.getValue()) {
	case 0: //Shifter
	    activeNibbles = new int[] { 120, 105,
					106, 107, 
					108, 109,
					110, 111,
					112, 113,
					114, 115,
					117
	    };
	    v1.updateNibble("Voice 1 (cents)",Nibble.TYPE_N2400_2400);
	    v2.updateNibble("Voice 2 (cents)",Nibble.TYPE_N2400_2400);
	    p1.updateNibble("Pan 1", Nibble.TYPE_N50_50);
	    p2.updateNibble("Pan 2", Nibble.TYPE_N50_50);
	    d1.updateNibble("Delay 1 (ms)", Nibble.TYPE_0_350);
	    d2.updateNibble("Delay 2 (ms)",Nibble.TYPE_0_350);
	    fb1.updateNibble("Feedback 1 (%)",Nibble.TYPE_0_100);
	    fb2.updateNibble("Feedback 2 (%)",Nibble.TYPE_0_100);
	    l1.updateNibble("Level 1 (dB)",Nibble.TYPE_N100_OFF_0);
	    l2.updateNibble("Level 2 (dB)",Nibble.TYPE_N100_OFF_0);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("Pitch", v1, i++));
	    box.add( new Row("Pitch", v2, i++));
	    box.add( new Row("Pitch", p1, i++));
	    box.add( new Row("Pitch", p2, i++));
	    box.add( new Row("Pitch", d1, i++));
	    box.add( new Row("Pitch", d2, i++));
	    box.add( new Row("Pitch", fb1, i++));
	    box.add( new Row("Pitch", fb2, i++));
	    box.add( new Row("Pitch", l1, i++));
	    box.add( new Row("Pitch", l2, i++));
	    box.add( new Row("Pitch", mix, i++));
	    break;
	case 1: //Octaver
	    activeNibbles = new int[] { 120, 105,
					115, 116, 117
	    };

	    direction.updateNibble("Direction",Nibble.TYPE_DOWNUP);
	    range.updateNibble("Range (oct)",Nibble.TYPE_1_2);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("Pitch", direction, i++));
	    box.add( new Row("Pitch", range, i++));
	    box.add( new Row("Pitch", mix, i++));
	    break;
	case 2: //Whammy
	    activeNibbles = new int[] { 120, 105,
					114, 115, 116
	    };

	    pitch = l1;
	    pitch.updateNibble("Pitch (%)", Nibble.TYPE_0_100);
	    direction.updateNibble("Direction",Nibble.TYPE_DOWNUP);
	    range.updateNibble("Range (oct)",Nibble.TYPE_1_2);

	    direction.updateNibble("Direction",Nibble.TYPE_DOWNUP);

	    box.add( new Row("Pitch", pitch, i++));
	    box.add( new Row("Pitch", direction, i++));
	    box.add( new Row("Pitch", range, i++));
	    break;
	case 3: //Detune
	    activeNibbles = new int[] { 120, 105,
					106, 107, 
					110, 111,
					117
	    };

	    v1.updateNibble("Voice 1 (cents)",Nibble.TYPE_N100_100);
	    v2.updateNibble("Voice 2 (cents)",Nibble.TYPE_N100_100);
	    d1.updateNibble("Delay 1 (ms)", Nibble.TYPE_0_50);
	    d2.updateNibble("Delay 2 (ms)",Nibble.TYPE_0_50);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("Pitch", v1, i++));
	    box.add( new Row("Pitch", v2, i++));
	    box.add( new Row("Pitch", d1, i++));
	    box.add( new Row("Pitch", d2, i++));
	    box.add( new Row("Pitch", mix, i++));	   
	    break;
	case 4: //Intell.P.	    
	    activeNibbles = new int[] { 120, 105,
					112, 113,
					106, 107, 
					114, 115,
					108, 109,
					110, 111,
					117
	    };

	    key.updateNibble("Key",Nibble.TYPE_KEY);
	    scale.updateNibble("Scale",Nibble.TYPE_SCALE);
	    v1.updateNibble("Voice 1 (degree)", Nibble.TYPE_DEGREES);//TYPE_DEGREES
	    v2.updateNibble("Voice 2 (degree)", Nibble.TYPE_DEGREES);//
	    l1.updateNibble("Level 1 (dB)",Nibble.TYPE_N100_OFF_0);
	    l2.updateNibble("Level 2 (dB)",Nibble.TYPE_N100_OFF_0);
	    p1.updateNibble("Pan 1", Nibble.TYPE_N50_50);
	    p2.updateNibble("Pan 2", Nibble.TYPE_N50_50);	    
	    d1.updateNibble("Delay 1 (ms)", Nibble.TYPE_0_350);
	    d2.updateNibble("Delay 2 (ms)",Nibble.TYPE_0_350);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);

	    box.add( new Row("Pitch", key, i++));
	    box.add( new Row("Pitch", scale, i++));
	    box.add( new Row("Pitch", v1, i++));
	    box.add( new Row("Pitch", v2, i++));
	    box.add( new Row("Pitch", l1, i++));
	    box.add( new Row("Pitch", l2, i++));
	    box.add( new Row("Pitch", p1, i++));
	    box.add( new Row("Pitch", p2, i++));
	    box.add( new Row("Pitch", d1, i++));
	    box.add( new Row("Pitch", d2, i++));
	    box.add( new Row("Pitch", mix, i++));
	    break;
	}
	while (i<14)
	    box.add( new EmptyRow(true, i++));    
    }

    int getBlockType(Box b) {
	return PITCH;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

