package nova;

import java.awt.*;
import javax.swing.*;
import java.awt.print.*;

public class PrintUtilities implements Printable {
    private Component componentToBePrinted;
    
    private PageFormat mPageFormat;

    public static void printComponent(Component c) {
	new PrintUtilities(c).print();
    }
    
    public PrintUtilities(Component componentToBePrinted) {
	this.componentToBePrinted = componentToBePrinted;
	mPageFormat = new PageFormat();
	mPageFormat.setOrientation(PageFormat.LANDSCAPE);
    }
  
    PageFormat getPageFormat() {
	return mPageFormat;
    }

    public void print() {
	PrinterJob printJob = PrinterJob.getPrinterJob();
	// set Orientation to LANDSCAPE
	printJob.setPrintable(this, mPageFormat);
	// set JobName according to that of the tab
	printJob.setJobName( componentToBePrinted.getName() );
    
	if (printJob.printDialog())
	    try {
		printJob.print();
	    } catch(PrinterException pe) {
		System.out.println("Error printing: " + pe);
	    }
    }

    public int print(Graphics g, PageFormat pageFormat, int pageIndex) {
	if (pageIndex > 0) {
	    return(NO_SUCH_PAGE);
	} else {
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
	    // Scale down to fit in a landscape-format page
	    g2d.scale(0.7, 0.7);
	    disableDoubleBuffering(componentToBePrinted);
	    componentToBePrinted.paint(g2d);
	    enableDoubleBuffering(componentToBePrinted);
	    return(PAGE_EXISTS);
	}
    }

    public static void disableDoubleBuffering(Component c) {
	RepaintManager currentManager = RepaintManager.currentManager(c);
	currentManager.setDoubleBufferingEnabled(false);
    }

    public static void enableDoubleBuffering(Component c) {
	RepaintManager currentManager = RepaintManager.currentManager(c);
	currentManager.setDoubleBufferingEnabled(true);
    }
}
