package nova;

import java.awt.Graphics;
import java.awt.print.*;
import java.util.Vector;

/** Extension of a Book so that a Book becomes Printable
 * Used to work around a bug in setPageable implementation
 * Gathers the prints of each Tab into a single document
 * to print all tabs at once
 */
public class PrintableBook extends Book implements Printable {
    Vector<PrintablePanel> pages;

    public PrintableBook() {
	super();
	pages = new Vector<PrintablePanel>();
    }

    public int getNumberOfPages() {
	return pages.size();
    }

    public void addTab(PrintablePanel pp) {
	append(pp, pp.getPageFormat());
	pages.add(pp);
    }

    public int print(Graphics g, PageFormat pf, int pageIndex) {
	if (pageIndex >= pages.size()) 
	    return NO_SUCH_PAGE;
	else {
	    PrintablePanel pp = pages.elementAt( pageIndex );
	    return pp.print(g,pf,0);
	}
    }

    public PageFormat getPageFormat(int pageIndex)
	throws IndexOutOfBoundsException {
	PrintablePanel pp = pages.elementAt( pageIndex );
	return pp.getPageFormat();
    }

    public Printable getPrintable(int pageIndex)  
	throws IndexOutOfBoundsException {
	return pages.elementAt( pageIndex );	
    }
}
