package nova;

import java.awt.*;
import javax.swing.*;
import java.awt.print.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;

/* Makes a JPanel printable
 * => Patch, Variation and SystemDump are therefore printable
 */
public class PrintablePanel extends JPanel implements Printable {
    private PageFormat mPageFormat;
    private double scale = 0.7;
    private PrinterJob printJob;
    private NovaLog log;

    public PrintablePanel() {
	super();

	printJob = PrinterJob.getPrinterJob();
	mPageFormat = printJob.defaultPage();
	mPageFormat.setOrientation(PageFormat.LANDSCAPE);
    }

    void setLog(NovaLog l) {
	this.log = l;
    }

    void log(String s) {
	log.append("\n== [ "+getName()+" ] ==");
	log.append(s);
    }

    public PageFormat getPageFormat() {
	return mPageFormat;
    }

    public void print() {    
	printJob.setPrintable(this, mPageFormat);
	printJob.setJobName( this.getName() );
	
	PrintRequestAttributeSet prs =  new HashPrintRequestAttributeSet();
	prs.add(OrientationRequested.LANDSCAPE);

	if (printJob.printDialog(prs))
	    try {
		printJob.print(prs);
	    } catch(PrinterException pe) {
		log("Error printing: " + pe);
	    }
    }

    public int print(Graphics g, PageFormat pageFormat, int pageIndex) {
	if (pageIndex > 0) {
	    return(NO_SUCH_PAGE);
	} else {
	    Graphics2D g2d = (Graphics2D)g;
	    g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
	    // Scale down to fit in a landscape-format page
	    g2d.scale(scale, scale);
	    disableDoubleBuffering(this);
	    this.paint(g2d);
	    enableDoubleBuffering(this);
	    return(PAGE_EXISTS);
	}
    }

    public SimpleDoc getDoc() {
	return new SimpleDoc(this, DocFlavor.SERVICE_FORMATTED.PRINTABLE, null);
    }

    public static void disableDoubleBuffering(Component c) {
	RepaintManager currentManager = RepaintManager.currentManager(c);
	currentManager.setDoubleBufferingEnabled(false);
    }

    public static void enableDoubleBuffering(Component c) {
	RepaintManager currentManager = RepaintManager.currentManager(c);
	currentManager.setDoubleBufferingEnabled(true);
    }
}
