package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** The Reverb Block
 * Extension of Block class to manage Reverb parameters
 * in Presets and Variations
 */
public class Reverb extends CopyBlock {
    Nibble type;
    Nibble decay;
    Nibble preDly;
    Nibble shape;
    Nibble size;
    Nibble hicolor;
    Nibble hilvl;
    Nibble locolor;
    Nibble lolvl;
    Nibble roomlvl;
    Nibble revlvl;
    Nibble diffuse;
    Nibble mix;
    Nibble onoff;

    public Reverb(Patch par) {	
	super("Reverb", par, true);

	try {
	    type = parent.getNibble(73);
	    type.updateNibble("Type", Nibble.TYPE_REVERB);
	    nibbles.add( type );

	    decay = parent.getNibble(74);
	    decay.updateNibble("Decay (s)", Nibble.TYPE_DECI_01_20);
	    nibbles.add( decay );

	    preDly = parent.getNibble(75);
	    preDly.updateNibble("PreDelay (ms)", Nibble.TYPE_0_100);
	    nibbles.add( preDly );

	    shape = parent.getNibble(76);
	    shape.updateNibble("Shape",Nibble.TYPE_SHAPE);
	    nibbles.add( shape );

	    size = parent.getNibble(77);
	    size.updateNibble("Size",Nibble.TYPE_SIZE);
	    nibbles.add( size );

	    hicolor = parent.getNibble(78);
	    hicolor.updateNibble("Hi Color",Nibble.TYPE_HICOL);
	    nibbles.add( hicolor );

	    hilvl = parent.getNibble(79);
	    hilvl.updateNibble("Hi Fac",Nibble.TYPE_N25_25);
	    nibbles.add( hilvl );

	    locolor = parent.getNibble(80);
	    locolor.updateNibble("Lo Color",Nibble.TYPE_LOCOL);
	    nibbles.add( locolor );

	    lolvl = parent.getNibble(81);
	    lolvl.updateNibble("Lo Fac",Nibble.TYPE_N25_25);
	    nibbles.add( lolvl );

	    roomlvl = parent.getNibble(82);
	    roomlvl.updateNibble("Room Lvl (dB)",Nibble.TYPE_N99_0);
	    nibbles.add( roomlvl );

	    revlvl = parent.getNibble(83);
	    revlvl.updateNibble("Rev Lvl (dB)",Nibble.TYPE_N99_0);
	    nibbles.add( revlvl );

	    diffuse = parent.getNibble(84);
	    diffuse.updateNibble("Diffuse",Nibble.TYPE_N25_25);
	    nibbles.add( diffuse );

	    mix = parent.getNibble(85);
	    mix.updateNibble("Mix (%)",Nibble.TYPE_0_100);
	    nibbles.add( mix );

	    onoff = parent.getNibble(88);
	    onoff.updateNibble("On",Nibble.TYPE_ONOFF);
	    nibbles.add( onoff );

	} catch (Exception e) {
	    System.err.println("Not enough Nibbles to load Drive Block");
	}

	refresh();
    }

    /* reformat the widgets */
    void refresh() {
	box.removeAll();

	int i = 0;
	activeNibbles = new int[]{ 88, 73,  
				   74, 75, 85,  
				   77, 78, 79,  
				   80, 81, 82, 
				   83, 84, 76 
	};

	Row r = new Row("Reverb", onoff,i++);
	box.add(r);
	r = new Row("Reverb", type, i++);
	box.add(r);

	r = new Row("Reverb", decay, i++);
	box.add(r);
	r = new Row("Reverb", preDly, i++);
	box.add(r);
	r = new Row("Reverb", mix, i++);
	box.add(r);

	r = new Row("Reverb", size, i++);
	box.add(r);
	r = new Row("Reverb", hicolor, i++);
	box.add(r);
	r = new Row("Reverb", hilvl, i++);
	box.add(r);

	r = new Row("Reverb", locolor, i++);
	box.add(r);
	r = new Row("Reverb", lolvl, i++);
	box.add(r);
	r = new Row("Reverb", roomlvl, i++);
	box.add(r);

	r = new Row("Reverb", revlvl, i++);
	box.add(r);
	r = new Row("Reverb", diffuse, i++);
	box.add(r);
	r = new Row("Reverb", shape, i++);
	box.add(r);
    }

    int getBlockType(Box b) {
	return REVERB;
    }

    int[] getActiveNibbles(Box b) {
	return activeNibbles;
    }

    void setActiveNibbles(Box b, int[] an) {
	activeNibbles = an;
    }
}

