package nova;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

/** Row class is used to display the value of a Nibble in a tabular form
 * Row background will alternate either between 2 gray values or between white and blue
 * First Row is darker and can display a "On/Off" rectangle
 */
public class Row extends JPanel implements MouseListener {
    String name;
    int rank ;
    Nibble parent;
    private NovaManager nova = null;

    static final Color gray0 = new Color(72, 74, 79);
    static final Color gray1 = new Color(48, 50, 55);
    static final Color gray2 = new Color(56, 56, 56);
    static final Color red =   new Color(161, 0,  0);
    static final Color green = new Color(2, 173,  2);
    static final Color blue =  new Color(219, 231, 253);
    static final Color light = new Color(255, 150,  60);
    static final Color brown = new Color(96, 26,  12);

    static final Font titleFont = new Font("Verdana", Font.BOLD, 12);
    static final Font onoffFont = new Font("Verdana", Font.PLAIN, 12);
    static final Font paramFont = new Font("Tahoma", Font.BOLD, 10);
    static final Font valueFont = new Font("Tahoma", Font.PLAIN, 10);

    boolean onoff = false;
    Color bg;
    int type = 0;
    String[] typeList;

    static final int TITLE = 0;
    static final int VAL1 = 1;
    static final int VAL2 = 2;
    static final int EMPTY1 = 3;
    static final int EMPTY2 = 4;

    static final Dimension rowDim = new Dimension(170, 20);
    static final Dimension onoffDim = new Dimension(30, 15);
    static final LineBorder border = new LineBorder( gray1, 1, false);

    JLabel l2;// display the value

    public Row( String title, Nibble parent, int rank ) {
	super();
	this.name = title;
	this.parent = parent ;
	this.rank = rank;

	this.setOpaque(true);

	if (parent !=null) this.typeList = parent.getTypeList();

	if ((parent !=null) && 
	    ((parent.getType()==Nibble.TYPE_ONOFF) || 
	     (parent.getType()==Nibble.TYPE_OFFON))) onoff = true;

	if ((rank % 2)==1)
	    if (parent != null) type = VAL1;
	    else type = EMPTY1;
	if ((rank % 2)==0)
	    if (parent != null) type = VAL2;
	    else type = EMPTY2;
	if (rank == 0) type = TITLE;

	switch (type) {
	case TITLE: setBackground(Constants.darkblue);
	    break;
	case VAL1: setBackground(blue);
	    break;
	case VAL2: setBackground(Color.white);
	    break;
	}

	setLayout(new GridBagLayout());
	//setSize( rowDim ); 
	setMinimumSize( rowDim ); 
	
	if ((parent !=null) || (name!=null)) init();
    }

    Block getBlock() {
	Container c = this.getParent();
	while (!(c instanceof Window)) {
	    if (c instanceof Block) break;
	    c = c.getParent();
	}
	if (c instanceof Block)
	    return (Block) c;
	else return null;
    }

    int getRank() {
	return rank;
    }

    public Dimension getMinimumSize() {
	return rowDim;
    }

    public Dimension getPreferredSize() {
	return rowDim;
    }

    void init() {
	GridBagConstraints c = new GridBagConstraints();
	c.gridy = 0;
	c.insets = new Insets(2,2,2,2);
	JLabel l1 = new JLabel("",SwingConstants.LEFT);
	c.gridx = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	if (onoff) c.weightx = 1.0;
	else c.weightx = 0.0;
	c.anchor = GridBagConstraints.LINE_START;

	if (type == TITLE) {
	    l1.setFont(titleFont);
	    l1.setForeground(Color.white);
	    l1.setText(name.toUpperCase());
	} else {
	    l1.setFont(paramFont);
	    l1.setForeground(Color.black);
	    if (parent != null) l1.setText(parent.getName()+":  ");
	}
	add(l1,c);

	String valueString = "";

	if (parent != null) valueString = parent.getStringValue();

	if (onoff) {
	    l2 = new JLabel("",SwingConstants.CENTER);
	    l2.setSize( onoffDim );
	    l2.setPreferredSize( onoffDim );
	    l2.setMaximumSize(onoffDim);
	    
	    l2.setFont(onoffFont);
	    l2.setForeground(Color.white);
	    l2.setOpaque(true);
	    if (valueString.equals("On"))
		l2.setBackground(light);
	    else l2.setBackground(brown);
	    
	    l2.setBorder( border );
	    l2.setText( valueString.toUpperCase());
	    c.gridx++;
	    c.weightx=0.0;
	    c.fill = GridBagConstraints.NONE;
	    c.anchor = GridBagConstraints.LINE_END;
	    add(l2,c);
	} else {	    
	    l2 = new JLabel("",SwingConstants.LEFT);	
	    l2.setFont(valueFont);
	    l2.setForeground(Color.black);
	    if (parent !=null) {
		if (parent.getType() == Nibble.TYPE_100_3000) {
		    int ms = parent.getValue();
		    int bpm = Nibble.toBPM(ms);
		    String s = ""+ ms +"ms   =  "+ bpm +" bpm";
		    l2.setText(s);
		} 
		else l2.setText(valueString);
	    }
	    c.gridx++;
	    c.weightx=1.0;
	    c.fill = GridBagConstraints.HORIZONTAL;
	    add(l2,c);
	}	
    
	if ( parent != null && parent.isEditable()) l2.addMouseListener(this);
    }    
	
    Nibble getNibble() {
	return parent;
    }

    void refresh() {
	//removeAll();
	//init();
	String stringValue = parent.getStringValue();
	if (onoff) {
	    l2.setText( stringValue.toUpperCase() );
	    
	    if (stringValue.equals("On"))
		l2.setBackground(light);
	    else l2.setBackground(brown);
	    
	} else 
	    if (parent.getType() == Nibble.TYPE_100_3000) {
		int ms = parent.getValue();
		int bpm = Nibble.toBPM(ms);
		String s = ""+ ms +"ms   =  "+ bpm +" bpm";
		l2.setText(s);
	    } 
	    else l2.setText(stringValue);   
	
    }

    private void log(String s) {
	if (parent != null) parent.log(s);
    }

    // == MOUSE LISTENER INTERFACE //
    public void mouseClicked(MouseEvent e) {

	Component c = (Component) e.getSource();
	Point pos = c.getLocationOnScreen();
	int x = e.getX() + pos.x;
	int y = e.getY() + pos.y;

	while (c != null) {
	    c = c.getParent();
	    if (c instanceof NovaManager) break;
	}

	if (c instanceof NovaManager) {
	    NovaManager nm = (NovaManager) c;

	    if (typeList.length < 30) {
		RowPopup np = new RowPopup( nm, this, x, y);
		np.pack();
		nm.setPopup(np);
	    } else {
		LongRowPopup np = new LongRowPopup(nm, this, x, y);
		np.pack();
		nm.setPopup(np);
	    }
	}
       
    }

    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
}

