package nova;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** A RowPopup is a Popup displaying allowed values for it 
*/
public class RowPopup extends Window implements MouseListener {

    private NovaManager parent;
    Row r;
    private int x;
    private int y;
    GridBagConstraints gc;

    final static Font normalFont = new Font("Tahoma", Font.PLAIN, 14);
    final static Font boldFont = new Font("Tahoma", Font.BOLD, 14);

    public RowPopup(NovaManager parent, Row r, int x, int y) {
	super(parent);
	this.parent = parent;
	this.r = r;
	this.x = x;
	this.y = y;

	setLayout( new GridBagLayout());
	gc = new GridBagConstraints();
	gc.insets = new Insets(2,5,2,5);
	setContent();
	setLocation(x,y);
    }

    void setContent() {
	removeAll();
	Nibble n = r.getNibble();
	
	String value = n.getStringValue();

	String[] typeList = n.getTypeList();

	if (typeList == null) {
	    parent.log("Popup : typeList is null\nNibble type is "+n.getType());
	} else {
	    Box b = Box.createVerticalBox();
	    for (int i=0; i<typeList.length; i++) {
		String s = typeList[i];
		if (s != "") {
		    JLabel lab = new JLabel(s);
		    if (value.equals(s)) lab.setFont(boldFont);
		    else lab.setFont(normalFont);
		    lab.addMouseListener(this);
		    b.add(lab);
		}
	    }
	    add(b, gc);
	}
    }
    
    /* call the parent component to set the new Nibble value and refresh */
    void update(String s) {
	Nibble n = r.getNibble();
	n.setValue(s);

	if (r.getRank()==1) {
	    // Effect type changed. Need to refresh the whole block and mapparam (global) TODO
	    Block b = r.getBlock();
	    if (b == null) r.refresh();
	    else {
		if (b instanceof Comp)
		    ((Comp) b).refresh();
		else if (b instanceof Modulation)
		    ((Modulation) b).refresh();
		else if (b instanceof Delay)
		    ((Delay) b).refresh();
		else if (b instanceof Pitch)
		    ((Pitch) b).refresh();
		else r.refresh();
		
		b.validate();
	    }

	    //Effect type changed.
	    //Exp.Pedal parameters list may need some refresh (if Patch and not Variation)
	    Component p = b;
	    while (p != null) {
		p = p.getParent();
		if (p instanceof Patch) {
		    Patch patch = (Patch) p;
		    if (patch.global != null) { 
			patch.global.refresh();
			patch.global.validate();
		    }
		    break;
		}
	    }
	} else
	    r.refresh();
    }

    public void setVisible(boolean state) {
	super.setVisible( state && parent.isEditable() );
    }

    // == MOUSE LISTENER INTERFACE //
    public void mouseClicked(MouseEvent e) {
	String s ="";
	JLabel lab = (JLabel) e.getSource();
	s = lab.getText();
	//System.out.println("You clicked on "+s);
	update(s);
	setVisible(false);
    }

    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
 
}