package nova;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;

/** Dialog called when a SystemDump has to be downloaded from NOVA
 */
public class SendCC extends JDialog implements ActionListener, TextListener {

    NovaManager parent;
    TextField channel = new TextField("1",3);
    TextField param = new TextField("0",3);
    TextField value = new TextField("0",3);
    JLabel ccLabel = new JLabel("", JLabel.CENTER);
    JButton send = new JButton("Send CC");

    String ccString = "";
    byte[] data = new byte[3];
	
    public SendCC(NovaManager nm) {
	super(nm, "Send Control Change", true);
	parent = nm;

	send.setActionCommand("send");
	send.addActionListener(this);

	java.awt.Container p = this.getContentPane();
	p.setLayout( new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	c.fill = GridBagConstraints.HORIZONTAL;
	c.ipadx=5;
	c.ipady=3;
	c.insets = new Insets(3,3,3,3);

	c.gridx=0;
	c.gridy=0;
	p.add(new JLabel("MIDI Channel (1..16):", JLabel.LEFT), c);

	c.gridy++;
	p.add(new JLabel("Param (0..127):", JLabel.LEFT), c);

	c.gridy++;
	p.add(new JLabel("Value (0..127):", JLabel.LEFT), c);
       
	c.gridx=1;
	c.gridy=0;
	channel.addTextListener(this);
	p.add( channel, c );

	c.gridy++;
	param.addTextListener(this);
	p.add( param, c );

	c.gridy++;
	value.addTextListener(this);
	p.add( value, c);

	c.gridy++;
	c.gridwidth=2;
	c.gridx=0;
	p.add( ccLabel, c);

	c.gridx=2;
	c.gridwidth=1;
	p.add( send, c );
	
	updateString();
	pack();
	setVisible(false);
    }

    private void updateString() throws NumberFormatException {
	//Calculate data and refresh label
	data[0] = (byte) (Integer.parseInt(channel.getText()) -1 + 0xB0 );
	data[1] = (byte) (Integer.parseInt(param.getText()) );
	data[2] = (byte) (Integer.parseInt(value.getText()) );

	ccString = "";
	ccString += Integer.toString( ( data[0] & 0xff ) + 0x100, 16).substring( 1 ) ;
	ccString += " "+ Integer.toString( ( data[1] & 0xff ) + 0x100, 16).substring( 1 ) ;
	ccString += " "+ Integer.toString( ( data[2] & 0xff ) + 0x100, 16).substring( 1 ) ;

	ccLabel.setText( ccString.toUpperCase() );
	ccLabel.validate();
    }

    void open() {
	setVisible(true);
    }

    public void textValueChanged(TextEvent e) {
	try {
	    updateString();
	} catch (NumberFormatException ex) {}
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	
	if (command.equals("send")) {
	    try {
		updateString();
		parent.send(data);
	    } 
	    catch (NumberFormatException ex) {
		parent.log("Sending CC: invalid values");
	    }
	}
	setVisible( false );

    }

    public static void main(String[] args) {
	NovaManager nm = new NovaManager();
	SendCC scc = new SendCC(nm);
	scc.open();
    }
}