package nova;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;

/** Dialog called when a SystemDump has to be downloaded from NOVA
 */
public class SendPC extends JDialog implements ActionListener, TextListener {

    NovaManager parent;
    TextField channel = new TextField("1",3);
    TextField param = new TextField("0",3);
    JLabel ccLabel = new JLabel("", JLabel.CENTER);
    JLabel presetLabel = new JLabel("", JLabel.CENTER);
    JButton send = new JButton("Send PC");

    String ccString = "";
    byte[] data = new byte[2];
	
    public SendPC(NovaManager nm) {
	super(nm, "Send Program Change", true);
	parent = nm;

	send.setActionCommand("send");
	send.addActionListener(this);

	java.awt.Container p = this.getContentPane();
	p.setLayout( new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	c.fill = GridBagConstraints.HORIZONTAL;
	c.ipadx=5;
	c.ipady=3;
	c.insets = new Insets(3,3,3,3);

	c.gridx=0;
	c.gridy=0;
	p.add(new JLabel("MIDI Channel (1..16):", JLabel.LEFT), c);
	c.gridy++;
	p.add(new JLabel("Program (0..89):", JLabel.LEFT), c);
       
	c.gridx=1;
	c.gridy=0;
	channel.addTextListener(this);
	p.add( channel, c );

	c.gridy=1;
	param.addTextListener(this);
	p.add( param, c );

	c.gridx = 2;
	p.add(presetLabel, c);

	c.gridy++;
	c.gridwidth=2;
	c.gridx=0;
	p.add( ccLabel, c);

	c.gridx=2;
	c.gridwidth=1;
	p.add( send, c );
	
	updateString();
	pack();
	setVisible(false);
    }

    private void updateString() throws NumberFormatException {
	//Calculate data and refresh label
	data[0] = (byte) (Integer.parseInt(channel.getText()) -1 + 0xC0 );
	data[1] = (byte) (Integer.parseInt(param.getText()) );

	int pnum = data[1] & 0xff;

	ccString = "";
	ccString += Integer.toString( ( data[0] & 0xff ) + 0x100, 16).substring( 1 ) ;
	ccString += " "+ Integer.toString( pnum + 0x100, 16).substring( 1 ) ;

	ccLabel.setText( ccString.toUpperCase() );
	ccLabel.validate();

	String preset = "";
	if ( (pnum >=0) && (pnum <90)) preset = "("+Constants.CPRESET[ pnum ]+")";
	presetLabel.setText( preset );
	presetLabel.validate();
    }

    void open() {
	setVisible(true);
    }

    public void textValueChanged(TextEvent e) {
	try {
	    updateString();
	} catch (NumberFormatException ex) {}
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	
	if (command.equals("send")) {
	    try {
		updateString();
		parent.send(data);
	    } 
	    catch (NumberFormatException ex) {
		parent.log("Sending PC: invalid values");
	    }
	}
	setVisible( false );

    }

    public static void main(String[] args) {
	NovaManager nm = new NovaManager();
	SendPC spc = new SendPC(nm);
	spc.open();
    }
}