package nova;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// Get the first sysex message
// Change the Preset Name
// Patch Name and Number can be changed without the checksum to be recomputed

public class SystemDump extends PrintablePanel implements ActionListener {
    private static final int DUMP_SIZE = 526 ;
    private static final int DATA_START = 0x008 ;
    private static final int DATA_END = 0x20B;
    private static final int CHECKSUM_POS = 0x20C;

 
    private byte[] data ;
    private String preset_name ;
    private int message_id;
    private int checksum;
    private int bankN;
    private int presN;
    private int numcode;
    private File saveFile;

    private MidiMapIn mapin;
    private MidiMapOut mapout;

    final static String[] NIB_NAMES = {	
	"Signature", "RoutingType",
	"ByPass", "Vol.Min (%)", "Vol.Mid (%)", "Vol.Max (%)",

	"PedalType", "Master", "CC TapTempo", "CC Comp",
	"CC Drv", "CC Mod", "CC Delay", "CC Rev",

	"CC NG", "CC Pitch", "CC EQ", "CC Boost",
	"CC Exp.Pedal", "MIDI Channel", "PrgChange.In", "PrgChange.Out",

	"Midi Clock", "SysEx ID", "Midi Sync", "#25",
	"#26", "TapLED Defeat", "Tap Master", "Boost Lock",

	"EQ Lock", "Routing Lock", "Factory Lock", "SpeakerSim",
	"AngleView", "FootSwitch", "Input Source", "Digital Clock",

	"#38", "FX Mute", "Dither", "Tempo",
	"Digital InGain (dB)", "Input InGain (dB)", "Advanced Mode", "Line Input Range (dB)",

	"Instr. Input Range (dB)", "BoostMax (dB)", "Output Range (dBu)", "Volume Level (dB)",
	"Volume Position", "KillDry", "Tuner Out", "Tuner Ref (Hz)",

	"Tuner Mode", "Tuner Range", "Send Tuner", "Current Preset",
	"#58", "edited", "#60", "Impedance", 
	"Calibration Min", "Calibration Max"
    };

    final static int[] NIB_TYPES = {	
	Nibble.TYPE_INT, Nibble.TYPE_ROUTING,
	Nibble.TYPE_OFFON, Nibble.TYPE_0_100, Nibble.TYPE_0_100, Nibble.TYPE_0_100,

	Nibble.TYPE_PEDAL, Nibble.TYPE_PEDALMASTER, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC,
	Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC,

	Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC,
	Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CHANNEL, Nibble.TYPE_OFFON, Nibble.TYPE_OFFON,

	Nibble.TYPE_OFFON, Nibble.TYPE_MIDI_SYSEX, Nibble.TYPE_OFFON, Nibble.TYPE_INT,
	Nibble.TYPE_INT, Nibble.TYPE_OFFON, Nibble.TYPE_TAPMASTER, Nibble.TYPE_OFFON,

	Nibble.TYPE_OFFON, Nibble.TYPE_OFFON, Nibble.TYPE_OFFON, Nibble.TYPE_OFFON,
	Nibble.TYPE_0_100, Nibble.TYPE_FOOTSWITCH, Nibble.TYPE_INPUTSRC, Nibble.TYPE_DCLOCK,

	Nibble.TYPE_INT, Nibble.TYPE_SOFTHARD, Nibble.TYPE_DITHER, Nibble.TYPE_100_3000,
	Nibble.TYPE_N100_6, Nibble.TYPE_N100_0, Nibble.TYPE_OFFON, Nibble.TYPE_0_24,

	Nibble.TYPE_N6_18, Nibble.TYPE_0_10, Nibble.TYPE_OUTPUTRANGE, Nibble.TYPE_N100_0,
	Nibble.TYPE_VOLUMEPOS, Nibble.TYPE_OFFON, Nibble.TYPE_TUNEROUT, Nibble.TYPE_420_460,

	Nibble.TYPE_TUNERMODE, Nibble.TYPE_TUNERRANGE, Nibble.TYPE_INT, Nibble.TYPE_CPRESET,
	Nibble.TYPE_INT, Nibble.TYPE_INT, Nibble.TYPE_INT, Nibble.TYPE_IMPEDANCE,

	Nibble.TYPE_INT, Nibble.TYPE_INT
    };
						
    
    private final int NIBBLE_LENGTH = 129;
    private Vector<Nibble> nibbles;

    JRootPane pane ;
    Global global;

    JButton closeButton = new JButton("Close");
    JButton saveButton = new JButton("Save SysEx");
    JButton saveXML = new JButton("Export XML");
    JButton saveHTML = new JButton("Export HTML");
    JButton send = new JButton("Send to Nova");

    String title;
    Vector<Block> blocks = new Vector<Block>();
    NovaManager parent;

    public SystemDump( NovaManager nm, byte[] sysex ) {	
	super( );
	this.parent = nm;
	setLog(nm.log);

	setOpaque(true);
	setBackground( Constants.grisvert );

	Calendar rightNow = Calendar.getInstance();
	title = ""+rightNow.get(Calendar.HOUR_OF_DAY);
	title += ":"+rightNow.get(Calendar.MINUTE);
	title += ":"+rightNow.get(Calendar.SECOND);

	this.data = sysex ;
	init();
    }

    public SystemDump( NovaManager nm, String s, byte[] sysex ) {	
	super( );
	this.parent = nm;
	setLog(nm.log);
	setOpaque(true);
	setBackground( Constants.grisvert );

	long millis = System.currentTimeMillis();

	this.title = s ;
	this.data = sysex ;
	init();
    }

    private void init() {
	setName( title ) ;	

	nibbles = new Vector<Nibble>();
	try {
	    for (int i=0; i< NIBBLE_LENGTH; i++) {
		byte[] temp = new byte[4];
		for (int j=0; j<4; j++) temp[j] = data[ DATA_START + 4*i + j];
		Nibble n = new Nibble(this, temp, DATA_START + 4*i );
		if (i < 64) n.updateNibble(NIB_NAMES[i], NIB_TYPES[i]);
		nibbles.add(n);
		
	    }
	} catch (NibbleException ne) {
	    parent.log(ne.getMessage());
	}

	/*
	int chk = calculateChecksum();
	int datachk = (int) data[ CHECKSUM_POS ];
	parent.log("Checksum read = "+datachk+" ; Checksum computed = "+chk);
	*/

	// Group by blocks
	blocks.add( new Levels( this ));
	blocks.add( new Pedal( this ));
	blocks.add( new Routing( this ));
	blocks.add( new CurrentPreset( this ));

	blocks.add( new Utility( this ));
	blocks.add( new TapTempo( this ));

	blocks.add( new MidiCC( this ));
	blocks.add( new MidiSetUp( this ));
	blocks.add( new Tuner( this ));

	int lines = 14;

	Nibble nib;

	Box levels = Box.createVerticalBox();
	levels.setBorder( Block.border );
	int i=0;
	levels.add( new Row("levels", null, i++));
	levels.add( new Row("volume", nibbles.elementAt(49), i++));
	levels.add( new Row("volpos", nibbles.elementAt(50), i++));
	levels.add( new Row("inputsrc", nibbles.elementAt(36), i++));
	levels.add( new Row("inputingain", nibbles.elementAt(43), i++));
	levels.add( new Row("LineInputRange", nibbles.elementAt(45), i++));
	levels.add( new Row("InstrInputRange", nibbles.elementAt(46), i++));
	levels.add( new Row("advmode", nibbles.elementAt(44), i++));
	levels.add( new Row("boostmax", nibbles.elementAt(47), i++));
	levels.add( new Row("output range", nibbles.elementAt(48), i++));
	levels.add( new Row("digital clock", nibbles.elementAt(37), i++));
	levels.add( new Row("digital ingain", nibbles.elementAt(42), i++));
	levels.add( new Row("dither", nibbles.elementAt(40), i++));
	while (i< lines ) levels.add( new EmptyRow(false, i++));
	
	Box pedal = Box.createVerticalBox();
	pedal.setBorder( Block.border );
	i=0;
	pedal.add( new Row("Pedal", null, i++));
	pedal.add( new Row("master", nibbles.elementAt(7), i++));
	pedal.add( new Row("pedaltype", nibbles.elementAt(6), i++));
	pedal.add( new Row("volmin", nibbles.elementAt(3), i++));
	pedal.add( new Row("volmid", nibbles.elementAt(4), i++));
	pedal.add( new Row("volmax", nibbles.elementAt(5), i++));
	nib = nibbles.elementAt(61);
	nib.setEditable(false);
	pedal.add( new Row("imp", nib, i++));
	nib = nibbles.elementAt(62);
	nib.setEditable(false);
	pedal.add( new Row("calibMin", nib, i++));
	nib = nibbles.elementAt(63);
	nib.setEditable(false);
	pedal.add( new Row("calibMax", nib, i++));
	pedal.add( new Row("Routing", null, 0)); i++;
	pedal.add( new Row("routing type", nibbles.elementAt(1), i++));
	pedal.add( new Row("bypass", nibbles.elementAt(2), i++));
	pedal.add( new Row("killdry", nibbles.elementAt(51), i++));
	while (i< lines ) pedal.add( new EmptyRow(false, i++));

	Box utility = Box.createVerticalBox();
	utility.setBorder( Block.border );
	i=0;
	utility.add( new Row("Utility", null, i++));
	utility.add( new Row("fx mute", nibbles.elementAt(39), i++));
	utility.add( new Row("boost lock", nibbles.elementAt(29), i++));
	utility.add( new Row("eq lock", nibbles.elementAt(30), i++));

	utility.add( new Row("routing lock", nibbles.elementAt(31), i++));
	utility.add( new Row("factory lock", nibbles.elementAt(32), i++));
	utility.add( new Row("SpeakerSim", nibbles.elementAt(33), i++));
	utility.add( new Row("footswitch", nibbles.elementAt(35), i++));

	utility.add( new Row("angle view", nibbles.elementAt(34), i++));

	utility.add( new Row("Tap Tempo", null, 0)); i++;
	utility.add( new Row("tap master", nibbles.elementAt(28), i++));
	utility.add( new Row("tap tempo", nibbles.elementAt(41), i++));
	utility.add( new Row("TapLED Def", nibbles.elementAt(27), i++));
	while (i< lines ) utility.add( new EmptyRow(false, i++));

	Box midi7up = Box.createVerticalBox();
	midi7up.setBorder( Block.border );
	i=0;
	midi7up.add( new Row("MIDI SetUp", null,i++));
  	midi7up.add( new Row("MIDI CHN", nibbles.elementAt(19), i++));
	midi7up.add( new Row("MIDI PCI", nibbles.elementAt(20), i++));
	midi7up.add( new Row("MIDI PCO", nibbles.elementAt(21), i++));
	midi7up.add( new Row("MIDI SYS", nibbles.elementAt(23), i++));

	midi7up.add( new Row("MIDI SYN", nibbles.elementAt(24), i++));
	midi7up.add( new Row("MIDI CLOCK", nibbles.elementAt(22), i++));
	midi7up.add( new EmptyRow(false, i++));
	midi7up.add( new EmptyRow(false, i++));
	midi7up.add( new Row("Tuner", null, 0)); i++;
	midi7up.add( new Row("tuner out", nibbles.elementAt(52), i++));
	midi7up.add( new Row("tuner ref", nibbles.elementAt(53), i++));
	midi7up.add( new Row("tuner mode", nibbles.elementAt(54), i++));
	midi7up.add( new Row("tuner range", nibbles.elementAt(55), i++));
	while (i< lines ) midi7up.add( new EmptyRow(false, i++));

	Box midi = Box.createVerticalBox();
	midi.setBorder( Block.border );
	i=0;
	midi.add( new Row("MIDI CC", null, i++));
	midi.add( new Row("MIDI TT", nibbles.elementAt(8), i++));
	midi.add( new Row("MIDI DRV", nibbles.elementAt(10), i++));
	midi.add( new Row("MIDI CMP", nibbles.elementAt(9), i++));
	midi.add( new Row("MIDI NG", nibbles.elementAt(14), i++));
	midi.add( new Row("MIDI EQ", nibbles.elementAt(16), i++));
	midi.add( new Row("MIDI BST", nibbles.elementAt(17), i++));
	midi.add( new Row("MIDI MOD", nibbles.elementAt(11), i++));
	midi.add( new Row("MIDI PIT", nibbles.elementAt(15), i++));
	midi.add( new Row("MIDI DLY", nibbles.elementAt(12), i++));
	midi.add( new Row("MIDI REV", nibbles.elementAt(13), i++));
	midi.add( new Row("MIDI EXP", nibbles.elementAt(18), i++));

	midi.add( new Row("Preset", null, 0)); i++;
	midi.add( new Row("current preset", nibbles.elementAt(57), i++));

	while (i< lines ) midi.add( new EmptyRow(false, i++));

	Box others = Box.createVerticalBox();
	others.setBorder( Block.border );

	i=0;
	Box hb = Box.createHorizontalBox();
	hb.add(levels);
	hb.add(pedal);
	hb.add(utility);
	hb.add(midi7up);
	hb.add(midi);
	//hb.add(others);

	setLayout( new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0,0,0,0);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 1.0;
	c.weighty = 0.0;
	//add( hb, BorderLayout.PAGE_START);

	mapin = new MidiMapIn(this);
	mapout = new MidiMapOut(this);

	c.gridx = 0;
	c.gridy = 0;
	add( hb, c );
	c.gridy++;
	add( mapin, c );
	c.gridy++;
	add( mapout, c );

	// BUTTONS
	closeButton.addActionListener( this );
	closeButton.setActionCommand( "close" );

	saveButton.addActionListener( this );
	saveButton.setActionCommand( "save" );

	saveXML.addActionListener( this );
	saveXML.setActionCommand( "saveXML" );
	saveXML.setEnabled( true );

	saveHTML.addActionListener( this );
	saveHTML.setActionCommand( "saveHTML" );
	saveHTML.setEnabled( true );

	//refreshNovasList();
	send.addActionListener( this );
	send.setActionCommand( "send" );
	send.setEnabled( !parent.getStandalone() );

	JPanel boutons = new JPanel();
	boutons.setOpaque(false);
	boutons.add( closeButton );
	boutons.add( saveButton );
	boutons.add( saveXML );
	boutons.add( saveHTML );
	boutons.add( send );

	c.gridy++;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.PAGE_END;
	c.weighty = 1.0;
	setOpaque(true);
	setBackground( Constants.grisvert);
	add( boutons, c);
    }

    int calculateChecksum() {
	return calculateChecksum(data);
    }

    static int calculateChecksum(byte[] d) {
	int sum = 0;
	for (int i=DATA_START; i<=DATA_END; i++) sum+=d[i];
	return sum % 128;
    }

    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();

	if (command.equals("close")) {
	    closeTab();
	} else if (command.equals("save")) {
	    saveValues();
	    saveSysEx();
	} else if (command.equals("saveXML")) {
	    saveValues();
	    saveXML();
	} else if (command.equals("saveHTML")) {
	    saveValues();
	    saveHTML();	    
	} else if (command.equals("send")) {
	    Integer id = (Integer)parent.getCurrentSysExID();
	    data[4] = id.byteValue();
	    saveValues();
	    parent.send( data );
	}
    }
	    

    public void writeBytesToFile(File file, byte[] bArray) {	
	try
	    {
		FileOutputStream fos = new FileOutputStream( file );
		
		byte[] sysex = new byte[11];
		sysex[0] = (byte) 0xF0; // Begin Sysex
		sysex[1] = (byte) 0x00; // TC
		sysex[2] = (byte) 0x20; // TC
		sysex[3] = (byte) 0x1F; // TC
		sysex[4] = (byte) 0x7F; // Device ID
		sysex[5] = (byte) 0x63; // Nova System
		sysex[6] = (byte) 0x45; // Message ID = Request
		sysex[7] = (byte) 0x01; // Data Type = Preset
		sysex[8] = (byte) 0x1F; // Preset Number
		sysex[9] = (byte) 0x00; // void
		sysex[10] = (byte) 0xF7; // End Sysex
		fos.write( sysex );
		
		fos.close(); 
		parent.logSilent(""+ sysex.length + " bytes written to " + file.getName());
	    }
	catch(FileNotFoundException ex)
	    {
		parent.log("FileNotFoundException : " + ex);
	    }
	catch(IOException ioe)
	    {
		parent.log("IOException : " + ioe);
	    }
	
    }
    
    Nibble getNibble(int i) {
	return nibbles.elementAt(i);
    }

    int getNibbleCount() {
	return nibbles.size();
    }


    void closeTab() {
	parent.removeTab(this);
    }

    void saveSysEx() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileOutputStream fos = new FileOutputStream( f );
		fos.write( data );
		fos.close(); 	    
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }
    void saveXML() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileWriter outFile = new FileWriter(f);
		PrintWriter out = new PrintWriter(outFile);
		out.println( toXML() );
		out.close();
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }

    void saveHTML() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileWriter outFile = new FileWriter(f);
		PrintWriter out = new PrintWriter(outFile);
		out.println( toHTML() );
		out.close();
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }


    void saveValues() {

	for (int i=0; i<nibbles.size(); i++) {
	    getNibble(i).save(data);
	}

	data[ CHECKSUM_POS ] = (byte)calculateChecksum();
    }

    String toHTML() {
	String s="";
	s += "<HTML>\n";
	s += "<STYLE>TABLE.MIDI TD { font-size:10px } TABLE.MIDI {table-layout: fixed}</STYLE>\n";
	s += "  <BODY>\n";
	s += "    <H1>System Dump</H1>\n";
	s += "    <TABLE border=\"1\" cellpadding=\"10\" width=\"1000\">\n";
	s += "      <TR>\n";
	s += blocks.elementAt(0).toHTML();//Levels
	s += blocks.elementAt(1).toHTML();//Pedal
	s += blocks.elementAt(4).toHTML();//Utility
	s += blocks.elementAt(7).toHTML();//SetUp
	s += blocks.elementAt(6).toHTML();//Midi CC
	s += "      </TR>\n";
	s += "      <TR>\n";
	s += blocks.elementAt(2).toHTML();//Routing
	s += blocks.elementAt(5).toHTML();//Tempo
	s += blocks.elementAt(8).toHTML();//Tuner
	s += "      </TR>\n";
	s += "      <TR>\n";
	s += blocks.elementAt(3).toHTML();//Preset
	s += "      </TR>\n";
	s += "    </TABLE>\n";
	s += mapin.toHTML();
	s += mapout.toHTML();
	s += "  </BODY>\n";
	s += "</HTML>\n";
	return s;
    }

    public String toXML() {
	String s="";

	s += "<system vendorId=\"00201F\" vendor=\"TC Electronic\" ";
	s += "modelId=\"63\" model=\"Nova System\">\n";
	
	for (int n=0; n<blocks.size(); n++) {
	    Block b = (Block) blocks.elementAt(n);
	    s += b.toXML();
	}

	s += mapin.toXML();
	s += mapout.toXML();

	s += "</system>";

	return s;
    }

    void setStandalone( boolean state) {
	send.setEnabled(!state);
    }

    /** check whether this data is actually a System Dump */
    public static boolean check(byte[] data) {
	boolean ok = true;
	if ( ok && (data.length != DUMP_SIZE)) ok = false;
	if ( ok && (data[0] != MidiInterface.aSystem[0])) ok = false;
	if ( ok && (data[1] != MidiInterface.aSystem[1])) ok = false;
	if ( ok && (data[2] != MidiInterface.aSystem[2])) ok = false;
	if ( ok && (data[3] != MidiInterface.aSystem[3])) ok = false;
	if ( ok && (data[5] != MidiInterface.aSystem[5])) ok = false;
	if ( ok && (data[6] != MidiInterface.aSystem[6])) ok = false;
	if ( ok && (data[7] != MidiInterface.aSystem[7])) ok = false;
	if ( ok && (data[DUMP_SIZE-1] != (byte) 0xF7)) ok = false;
	if ( ok && (data[CHECKSUM_POS] != calculateChecksum(data))) ok = false;

	return ok;
    }


    public static void main(String[] args) {
	// Arg 0 : name
	// Arg 1 : input file

	if (args.length != 1) {
	    System.out.println("Syntax : java SystemDump <Sysex inputFile>");
	} else {
	    File inputFile = new File( args[0] );

	    try {
		byte[] bArray = NovaManager.getBytesFromFile( inputFile );
		NovaManager nm = new NovaManager();
		nm.setEditable(true);

		SystemDump p = new SystemDump( nm, args[0], bArray );
		nm.add(p);
	    } catch (IOException ioe) {
		System.err.println("File not found : "+args[1]);
	    }
	}
    }
}

