package nova;

import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// Get the first sysex message
// Change the Preset Name
// Patch Name and Number can be changed without the checksum to be recomputed

public class SystemDump2 extends PrintablePanel implements ActionListener {
    private static final int DUMP_SIZE = 526 ;
    private static final int DATA_START = 0x008 ;
    private static final int DATA_END = 0x20B;
    private static final int CHECKSUM_POS = 0x20C;

 
    private byte[] data ;
    private String preset_name ;
    private int message_id;
    private int checksum;
    private int bankN;
    private int presN;
    private int numcode;
    private File saveFile;

    private MidiMapIn mapin;
    private MidiMapOut mapout;

    final static String[] NIB_NAMES = {	
	"#0", "RoutingType",
	"#2", "Vol.Min (%)", "Vol.Mid (%)", "Vol.Max (%)",

	"PedalType", "Master", "CC TapTempo", "CC Comp",
	"CC Drv", "CC Mod", "CC Delay", "CC Rev",

	"CC NG", "CC Pitch", "CC EQ", "CC Boost",
	"CC Exp.Pedal", "MIDI Channel", "PrgChange.In", "PrgChange.Out",

	"#22", "SysEx ID", "Midi Sync", "#25",
	"#26", "#27", "Tap Master", "Boost Lock",

	"EQ Lock", "Routing Lock", "Factory Lock", "SpeakerSim",
	"AngleView", "FootSwitch", "InputSrc", "Digital Clock",

	"#38", "FX Mute", "Dither", "TapTempo (ms)",
	"Digital InGain (dB)", "#43", "Adv. Mode", "Input Gain (dB)",

	"#46", "BoostMax (dB)", "Output Range (dBu)", "Volume (dB)",
	"Volume Pos", "#51", "TunerOut", "TunerRef (Hz)",
	"#54", "#55", "#56", "Current Preset"
    };

    final static int[] NIB_TYPES = {	
	Nibble.TYPE_INT, Nibble.TYPE_ROUTING,
	Nibble.TYPE_INT, Nibble.TYPE_0_100, Nibble.TYPE_0_100, Nibble.TYPE_0_100,

	Nibble.TYPE_PEDAL, Nibble.TYPE_PEDALMASTER, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC,
	Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC,

	Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CC,
	Nibble.TYPE_MIDI_CC, Nibble.TYPE_MIDI_CHANNEL, Nibble.TYPE_OFFON, Nibble.TYPE_OFFON,

	Nibble.TYPE_INT, Nibble.TYPE_MIDI_SYSEX, Nibble.TYPE_OFFON, Nibble.TYPE_INT,
	Nibble.TYPE_INT, Nibble.TYPE_INT, Nibble.TYPE_TAPMASTER, Nibble.TYPE_OFFON,

	Nibble.TYPE_OFFON, Nibble.TYPE_OFFON, Nibble.TYPE_OFFON, Nibble.TYPE_OFFON,
	Nibble.TYPE_0_100, Nibble.TYPE_FOOTSWITCH, Nibble.TYPE_INPUTSRC, Nibble.TYPE_DCLOCK,

	Nibble.TYPE_INT, Nibble.TYPE_SOFTHARD, Nibble.TYPE_DITHER, Nibble.TYPE_100_3000,
	Nibble.TYPE_N100_6, Nibble.TYPE_INT, Nibble.TYPE_OFFON, Nibble.TYPE_0_24,

	Nibble.TYPE_INT, Nibble.TYPE_0_10, Nibble.TYPE_OUTPUTRANGE, Nibble.TYPE_N100_0,
	Nibble.TYPE_VOLUMEPOS, Nibble.TYPE_INT, Nibble.TYPE_TUNEROUT, Nibble.TYPE_420_460,
	Nibble.TYPE_INT, Nibble.TYPE_INT, Nibble.TYPE_INT, Nibble.TYPE_CPRESET
    };
						
    
    private final int NIBBLE_LENGTH = 129;
    private Vector<Nibble> nibbles;

    JRootPane pane ;
    Global global;

    JButton closeButton = new JButton("Close");
    JButton saveButton = new JButton("Save SysEx");
    JButton saveXML = new JButton("Export XML");
    JButton saveHTML = new JButton("Export HTML");
    JButton send = new JButton("Send to Nova");

    String title;
    Vector<Block> blocks = new Vector<Block>();
    NovaManager parent;

    public SystemDump2( NovaManager nm, byte[] sysex ) {	
	super( );
	this.parent = nm;
	setLog(nm.log);

	setOpaque(true);
	setBackground( Constants.grisvert );

	Calendar rightNow = Calendar.getInstance();
	title = ""+rightNow.get(Calendar.HOUR_OF_DAY);
	title += ":"+rightNow.get(Calendar.MINUTE);
	title += ":"+rightNow.get(Calendar.SECOND);

	this.data = sysex ;
	init();
    }

    public SystemDump2( NovaManager nm, String s, byte[] sysex ) {	
	super( );
	this.parent = nm;
	setLog(nm.log);
	setOpaque(true);
	setBackground( Constants.grisvert );

	long millis = System.currentTimeMillis();

	this.title = s ;
	this.data = sysex ;
	init();
    }

    private void init() {
	setName( title ) ;	

	nibbles = new Vector<Nibble>();
	try {
	    for (int i=0; i< NIBBLE_LENGTH; i++) {
		byte[] temp = new byte[4];
		for (int j=0; j<4; j++) temp[j] = data[ DATA_START + 4*i + j];
		Nibble n = new Nibble(this, temp, DATA_START + 4*i );
		switch (i) {
		case 0: n.updateNibble("#0 (1)", Nibble.TYPE_INT);
		    break;
		case 58: n.updateNibble("#58 (512)", Nibble.TYPE_INT);
		    break;
		case 59: n.updateNibble("#59 (1)", Nibble.TYPE_INT);
		    break;
		case 60: n.updateNibble("#60 (15)", Nibble.TYPE_INT);
		    break;
		case 62: n.updateNibble("#62 (810*)", Nibble.TYPE_INT);
		    break;
		case 63: n.updateNibble("#63 (975*)", Nibble.TYPE_INT);
		    break;
		case 127: n.updateNibble("#127 (15232)", Nibble.TYPE_INT);
		    break;
		default: n.updateNibble("#"+i+" (0)", Nibble.TYPE_INT);
		}

		nibbles.add(n);
		
	    }
	} catch (NibbleException ne) {
	    parent.log(ne.getMessage());
	}

	Box empty1 = Box.createVerticalBox();
	empty1.setBorder( Block.border );
	int i=0;
	empty1.add( new Row("Empty", null, i++));
  	empty1.add( new Row("2", nibbles.elementAt(2), i++));
	empty1.add( new Row("22", nibbles.elementAt(22), i++));
	empty1.add( new Row("25", nibbles.elementAt(25), i++));
	empty1.add( new Row("26", nibbles.elementAt(26), i++));
	empty1.add( new Row("27", nibbles.elementAt(27), i++));
	empty1.add( new Row("38", nibbles.elementAt(38), i++));
	empty1.add( new Row("43", nibbles.elementAt(43), i++));

	Box empty2 = Box.createVerticalBox();
	empty2.setBorder( Block.border );
	i=0;
	empty2.add( new Row("Empty", null, i++));
  	empty2.add( new Row("46", nibbles.elementAt(46), i++));
	empty2.add( new Row("51", nibbles.elementAt(51), i++));
	empty2.add( new Row("54", nibbles.elementAt(54), i++));
	empty2.add( new Row("55", nibbles.elementAt(55), i++));
	empty2.add( new Row("56", nibbles.elementAt(56), i++));
	empty2.add( new Row("61", nibbles.elementAt(61), i++));
	empty2.add( new Row("128", nibbles.elementAt(128), i++));

	Box unknown = Box.createVerticalBox();
	unknown.setBorder( Block.border );
	i=0;
	unknown.add( new Row("Non Empty", null, i++));
  	unknown.add( new Row("0", nibbles.elementAt(0), i++));
	unknown.add( new Row("58", nibbles.elementAt(58), i++));
	unknown.add( new Row("59", nibbles.elementAt(59), i++));
	unknown.add( new Row("60", nibbles.elementAt(60), i++));
	unknown.add( new Row("62", nibbles.elementAt(62), i++));
	unknown.add( new Row("63", nibbles.elementAt(63), i++));
	unknown.add( new Row("127", nibbles.elementAt(127), i++));

	Box hb = Box.createHorizontalBox();
	hb.add(empty1);
	hb.add(empty2);
	hb.add(unknown);

	setLayout( new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.insets = new Insets(0,0,0,0);
	c.fill = GridBagConstraints.HORIZONTAL;
	c.weightx = 1.0;
	c.weighty = 0.0;
	//add( hb, BorderLayout.PAGE_START);

	//mapin = new MidiMapIn(this);
	//mapout = new MidiMapOut(this);

	c.gridx = 0;
	c.gridy = 0;
	add( hb, c );
	c.gridy++;

	// BUTTONS
	closeButton.addActionListener( this );
	closeButton.setActionCommand( "close" );

	saveButton.addActionListener( this );
	saveButton.setActionCommand( "save" );

	saveXML.addActionListener( this );
	saveXML.setActionCommand( "saveXML" );
	saveXML.setEnabled( true );

	saveHTML.addActionListener( this );
	saveHTML.setActionCommand( "saveHTML" );
	saveHTML.setEnabled( true );

	send.addActionListener( this );
	send.setActionCommand( "send" );
	send.setEnabled( false );

	JPanel boutons = new JPanel();
	boutons.setOpaque(false);
	boutons.add( closeButton );
	boutons.add( saveButton );
	boutons.add( saveXML );
	boutons.add( saveHTML );
	boutons.add( send );

	c.gridy++;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.PAGE_END;
	c.weighty = 1.0;
	setOpaque(true);
	setBackground( Constants.grisvert);
	add( boutons, c);
    }

    int calculateChecksum() {
	return calculateChecksum(data);
    }

    static int calculateChecksum(byte[] d) {
	int sum = 0;
	for (int i=DATA_START; i<=DATA_END; i++) sum+=d[i];
	return sum % 128;
    }

    public void actionPerformed(ActionEvent e) {
	if (e.getActionCommand().equals("close")) {
	    closeTab();
	} else if (e.getActionCommand().equals("save")) {
	    saveValues();
	    saveSysEx();
	} else if (e.getActionCommand().equals("saveXML")) {
	    saveValues();
	    saveXML();
	} else if (e.getActionCommand().equals("saveHTML")) {
	    saveValues();
	    saveHTML();	    
	} else if (e.getActionCommand().equals("send")) {
	    saveValues();
	    parent.send( data );
	    // send to a Nova (choose ID)
	}
    }
	    

    public void writeBytesToFile(File file, byte[] bArray) {	
	try
	    {
		FileOutputStream fos = new FileOutputStream( file );
		
		byte[] sysex = new byte[11];
		sysex[0] = (byte) 0xF0; // Begin Sysex
		sysex[1] = (byte) 0x00; // TC
		sysex[2] = (byte) 0x20; // TC
		sysex[3] = (byte) 0x1F; // TC
		sysex[4] = (byte) 0x7F; // Device ID
		sysex[5] = (byte) 0x63; // Nova System
		sysex[6] = (byte) 0x45; // Message ID = Request
		sysex[7] = (byte) 0x01; // Data Type = Preset
		sysex[8] = (byte) 0x1F; // Preset Number
		sysex[9] = (byte) 0x00; // void
		sysex[10] = (byte) 0xF7; // End Sysex
		fos.write( sysex );
		
		fos.close(); 
		parent.logSilent(""+ sysex.length + " bytes written to " + file.getName());
	    }
	catch(FileNotFoundException ex)
	    {
		parent.log("FileNotFoundException : " + ex);
	    }
	catch(IOException ioe)
	    {
		parent.log("IOException : " + ioe);
	    }
	
    }
    
    Nibble getNibble(int i) {
	return nibbles.elementAt(i);
    }

    int getNibbleCount() {
	return nibbles.size();
    }


    void closeTab() {
	parent.removeTab(this);
    }

    void saveSysEx() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileOutputStream fos = new FileOutputStream( f );
		fos.write( data );
		fos.close(); 	    
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }
    void saveXML() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileWriter outFile = new FileWriter(f);
		PrintWriter out = new PrintWriter(outFile);
		out.println( toXML() );
		out.close();
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }

    void saveHTML() {
	JFileChooser fc = parent.getFileChooser();

	int returnVal = fc.showSaveDialog(this);
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File f = fc.getSelectedFile();
	    try {
		FileWriter outFile = new FileWriter(f);
		PrintWriter out = new PrintWriter(outFile);
		out.println( toHTML() );
		out.close();
	    } catch (IOException err) {
		err.printStackTrace();
	    }
	}
    }


    void saveValues() {

	for (int i=0; i<nibbles.size(); i++) {
	    getNibble(i).save(data);
	}

	data[ CHECKSUM_POS ] = (byte)calculateChecksum();
    }

    String toHTML() {
	String s="";
	s += "<HTML>\n";
	s += "<STYLE>TABLE.MIDI TD { font-size:10px } TABLE.MIDI {table-layout: fixed}</STYLE>\n";
	s += "  <BODY>\n";
	s += "    <H1>System Dump</H1>\n";
	s += "    <TABLE border=\"1\" cellpadding=\"10\" width=\"1000\">\n";
	s += "      <TR>\n";
	s += blocks.elementAt(0).toHTML();//Levels
	s += blocks.elementAt(1).toHTML();//Pedal
	s += blocks.elementAt(4).toHTML();//Utility
	s += blocks.elementAt(7).toHTML();//SetUp
	s += blocks.elementAt(6).toHTML();//Midi CC
	s += "      </TR>\n";
	s += "      <TR>\n";
	s += blocks.elementAt(2).toHTML();//Routing
	s += "      </TR>\n";
	s += "      <TR>\n";
	s += blocks.elementAt(3).toHTML();//Preset
	s += blocks.elementAt(5).toHTML();//Tempo
	s += blocks.elementAt(8).toHTML();//Tuner
	s += "      </TR>\n";
	s += "    </TABLE>\n";
	s += mapin.toHTML();
	s += mapout.toHTML();
	s += "  </BODY>\n";
	s += "</HTML>\n";
	return s;
    }

    public String toXML() {
	String s="";

	s += "<system vendorId=\"00201F\" vendor=\"TC Electronic\" ";
	s += "modelId=\"63\" model=\"Nova System\">\n";
	
	for (int n=0; n<blocks.size(); n++) {
	    Block b = (Block) blocks.elementAt(n);
	    s += b.toXML();
	}

	s += mapin.toXML();
	s += mapout.toXML();

	s += "</system>";

	return s;
    }

    /** check whether this data is actually a System Dump */
    public static boolean check(byte[] data) {
	boolean ok = true;
	if ( ok && (data.length != DUMP_SIZE)) ok = false;
	if ( ok && (data[0] != MidiInterface.aSystem[0])) ok = false;
	if ( ok && (data[1] != MidiInterface.aSystem[1])) ok = false;
	if ( ok && (data[2] != MidiInterface.aSystem[2])) ok = false;
	if ( ok && (data[3] != MidiInterface.aSystem[3])) ok = false;
	if ( ok && (data[5] != MidiInterface.aSystem[5])) ok = false;
	if ( ok && (data[6] != MidiInterface.aSystem[6])) ok = false;
	if ( ok && (data[7] != MidiInterface.aSystem[7])) ok = false;
	if ( ok && (data[DUMP_SIZE-1] != (byte) 0xF7)) ok = false;
	if ( ok && (data[CHECKSUM_POS] != calculateChecksum(data))) ok = false;

	return ok;
    }

    void setStandalone( boolean state) {
	send.setEnabled( !state );
    }

    public static void main(String[] args) {
	// Arg 0 : name
	// Arg 1 : input file

	if (args.length != 1) {
	    System.out.println("Syntax : java SystemDump <Sysex inputFile>");
	} else {
	    File inputFile = new File( args[0] );

	    try {
		byte[] bArray = NovaManager.getBytesFromFile( inputFile );
		NovaManager nm = new NovaManager();
		nm.setEditable(false);

		SystemDump2 p = new SystemDump2( nm, args[0], bArray );
		nm.add(p);
	    } catch (IOException ioe) {
		System.err.println("File not found : "+args[1]);
	    }
	}
    }
}

