package nova;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// Get the first sysex message
// Change the Preset Name
// Patch Name and Number can be changed without the checksum to be recomputed

public class Variation extends Patch implements ActionListener {

    private static final int CMP = 0;
    private static final int DRV = 1;
    private static final int MOD = 2;
    private static final int DLY = 3;
    private static final int REV = 4;
    private static final int EQ = 5;
    private static final int PIT = 6;

    private static final String[] VARNAME = {
	"COMP", "DRIVE/BOOST", "MODULATION", "DELAY", "REVERB", "EQ/GATE", "PITCH"};

    private int numcode;
    private int blockType;
    private String variationName;
    private int n;// 1 to 4

    JRootPane pane ;
    JPanel titlePane;
    Global global;

    JButton closeButton = new JButton("Close");
    JButton saveButton = new JButton("Save SysEx");
    JButton saveXML = new JButton("Export XML");
    JButton saveHTML = new JButton("Export HTML");
    JButton send = new JButton("Send to Nova");

    public Variation( NovaManager nm, byte[] sysex ) {	
	super(nm, sysex, true);
	refresh();
	setName( variationName );

	blocks = new Vector<Block>();

	// build blocks from data
	global = new Global(this);
	DriveBoost drive = new DriveBoost(this);
	Comp comp = new Comp(this);	
	EQGate eq = new EQGate(this);
	Modulation mod = new Modulation(this);
	Pitch pitch = new Pitch(this);
	Delay delay = new Delay(this);
	Reverb reverb = new Reverb(this);
		
	// Order changed to follow to BlockType order
	blocks.add(comp);
	blocks.add(drive);
	blocks.add(mod);
	blocks.add(delay);
	blocks.add(reverb);
	blocks.add(eq);
	blocks.add(pitch);
	blocks.add(global);
	
	// Buttons
	closeButton.addActionListener( this );
	closeButton.setActionCommand( "close" );

	saveButton.addActionListener( this );
	saveButton.setActionCommand( "save" );

	saveXML.addActionListener( this );
	saveXML.setActionCommand( "saveXML" );

	saveHTML.addActionListener( this );
	saveHTML.setActionCommand( "saveHTML" );

	//refreshNovasList( );
	send.addActionListener( this );
	send.setActionCommand( "send" );
	send.setEnabled( !parent.getStandalone() );

	// Layout
	Block b;
	switch (blockType) {
	case CMP: b = comp;
	    break;
	case DRV: b = drive;
	    break;
	case MOD: b = mod;
	    break;
	case DLY: b = delay;
	    break;
	case REV: b = reverb;
	    break;
	case EQ: b = eq;
	    break;
	case PIT: b = pitch;
	    break;
	default:
	    b = null;
	}

	JPanel boutons = new JPanel();
	boutons.setOpaque(false);
	boutons.add( closeButton );
	boutons.add( saveButton );
	boutons.add( saveXML );
	boutons.add( saveHTML );
	boutons.add( send );

	setLayout( new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();
	c.weightx =1.0;
	c.weighty = 0.0;
	c.gridx = 0;
	c.gridy = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.PAGE_START;
	add(getTitlePane(),c);

	c.weighty = 0;
	c.weightx = 0;
	c.gridy++;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.PAGE_START;
	add(b,c);
	c.gridy++;
	c.weightx =1.0;
	c.weighty = 1.0;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.PAGE_END;
	add(boutons,c);
	setOpaque(true);
	setBackground( Constants.grisvert );
    }

    int getNumber() {
	return n;
    }

    void setNumber(int i) {
	n = i;
	numcode = 90 + (blockType*4) + n-1;
	refreshVariationName();
    }

    void refreshVariationName() {
	variationName = VARNAME[blockType] + ": Variation #"+n;
	setName( variationName );
    }
    
    void refreshTitle() {
	refreshVariationName();
	global.refreshTitle();
	parent.refreshTabName( variationName );
    }

    /** Calculate settings from sysex
	Refresh widgets
    */
    void refresh() {
	super.refresh();
	numcode = getPresetCode();
	blockType = (numcode - 90)/4;
	n = (numcode - 90) % 4 +1 ;
	refreshVariationName();
    }

    /*
    void refreshNovasList( ) {
	send.removeActionListener(this);
	send.removeAllItems();
	send.addItem("Send to Nova");

	java.util.List novas = parent.getNovas();
	if (novas != null) {
	    for (int i=0; i<novas.size(); i++) {
		send.addItem( novas.get( i ));
	    }
	}
	send.addActionListener(this);
    }
    */

    String getVariationName() {
	return variationName;
    }

    void saveValues() {
	// Save Number
	data[ PRESET_NUMBER_OFFSET ] = (byte) ( numcode );

	// Save nibbles
	for (int i=0; i<nibbles.size(); i++) {
	    Nibble n = getNibble(i);
	    n.save(data);
	}

	// Refresh Checksum
	data[ CHECKSUM_OFFSET ] = (byte)calculateChecksum();
    }

    String toHTML() {
	String s="";
	s += "<HTML>\n";
	s += "  <BODY>\n";
	s += "    <H1>"+variationName+"</H1>\n";
	s += "    <TABLE border=\"1\" cellpadding=\"10\">\n";
	s += "      <TR>\n";
	
	Block b = (Block) blocks.elementAt(blockType);
	s+= b.toHTML();
	
	s += "      </TR>\n";
	s += "    </TABLE>\n";
	s += "  </BODY>\n";
	s += "</HTML>\n";
	return s;
    }

    public String toXML() {
	String s="";

	s += "<variation vendorId=\"00201F\" vendor=\"TC Electronic\" ";
	s += "modelId=\"63\" model=\"Nova System\" ";
	s += " n=\"" +numcode ;
	s += "\" block=\"" +blockType ;
	s += "\" blockName=\"" +VARNAME[blockType] ;
	s += "\" variation=\"" +n ;
	s += "\">\n";

	Block b = (Block) blocks.elementAt(blockType);
	s += b.toXML();
	

	s += "</variation>";

	return s;
    }
    
   public void mouseClicked(MouseEvent e) {
       Component c = (Component) e.getSource();
       Point pos = c.getLocationOnScreen();
       int x = e.getX() + pos.x;
       int y = e.getY() + pos.y;
       if (x<0) x=0;
       if (y<0) y=0;
       
       //parent.en.update(this,x,y); 
       VarPopup vp = new VarPopup(parent,this,x,y);
       vp.pack();
       parent.setPopup(vp);
   }

    public static void main(String[] args) {
	// Arg 0 : name
	// Arg 1 : input file

	if (args.length != 1) {
	    System.out.println("Syntax : java nova.Variation <Sysex inputFile>");
	} else {
	    File inputFile = new File( args[0] );

	    try {
		byte[] bArray = getBytesFromFile( inputFile );
		NovaManager nm = new NovaManager();
		nm.add( new Variation( nm, bArray ));
	    } catch (IOException ioe) {
		System.err.println("File not found : "+args[1]);
	    }	    
	}
    }
}

